/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.editor.SubscriptionManager");

(function(){

// These variables must match LinkCommand.java
var ADD_LINK= "addLink";
var REMOVE_LINK= "removeLink";
var SUBSCRIBER_TYPE= "subscriber";

// The following must match WorkItemRestService.javaa
var UPDATE_LINKS = "updateLinks";
    
    dojo.declare("com.ibm.team.workitem.web.client.internal.editor.SubscriptionManager", null, {
    
        constructor: function(parentController){
			this.parentController = parentController;
			this._subscribers = [];
            this._workItemSubscribers = [];
            this._updateListeners = [];
        },
		setWorkItem: function(workItem) {
            this._workItem = workItem;
            this._subscribers = [];
            this._workItemSubscribers = [];
            
            if (this._workItem.attributes && this._workItem.attributes.internalSubscriptions && this._workItem.attributes.internalSubscriptions.items) {
                for (var i = 0; i < this._workItem.attributes.internalSubscriptions.items.length; i++) {
                    this._subscribers.push(this._workItem.attributes.internalSubscriptions.items[i]);
                    this._workItemSubscribers.push(this._workItem.attributes.internalSubscriptions.items[i]);
                }
            }
			this._handleSubscriptionsSemanticChange();
		},
		addUpdateListener: function(listener) {
			for(var i=0;i<this._updateListeners.length;i++) {
				if(this._updateListeners[i] == listener) {
					return;
				}
			}
			this._updateListeners.push(listener);			
		}, 
		removeUpdateListener: function(listener) {
			var newListeners = [];
			for(var i=0;i<this._updateListeners.length;i++) {
				if(this._updateListeners[i] != listener) {
					newListeners.push(this._updateListeners[i]);
				}
			}
			this._updateListeners = newListeners;			
		}, 
        /*
         * Add the array of selected contributors to the 'subscribers' multi-
         * select box (in alphabetical ascending case-insensitive sort order).
         * If we have at least one new subscriber and the caller specified true
         * for markDirtyOnNew, mark the editor dirty.
         */
        addSubscribers: function(newContribs){
			if(!newContribs || !newContribs.length) return;
            for (var i = 0; i < newContribs.length; i++) {
                this.addSubscriber(newContribs[i]);
            }
            this._handleSubscriptionsSemanticChange();
        },
        deleteSubscribers: function(subscribers){
			if(!subscribers || !subscribers.length) return;
            for (var i = 0; i < subscribers.length; i++) {
	            this.deleteSubscriber(subscribers[i]);
            }
            this._handleSubscriptionsSemanticChange();
		},
         restoreSubscribers: function(subscribers){
			if(!subscribers || !subscribers.length) return;
            for (var i = 0; i < subscribers.length; i++) {
	            this.restoreSubscriber(subscribers[i]);
            }
            this._handleSubscriptionsSemanticChange();
		},
               
        addSubscriber: function(contrib){
            var found = false;
            for (var i = 0; i < this._subscribers.length; i++) {
                if (this._subscribers[i].itemId == contrib.itemId) {
                    found = true;
                    this._subscribers[i]._isDeleted = false;
                    break;
                }
            }
            if (!found) {
                contrib._isNew = true;
                this._subscribers.push(contrib);
            }
        },
         deleteSubscriber: function(contrib){
            for (var i = 0; i < this._subscribers.length; i++) {
                if (this._subscribers[i].itemId == contrib.itemId) {
					if (this._subscribers[i]._isNew) {
						this._subscribers.splice(i, 1);
					}
					else {
                    	this._subscribers[i]._isDeleted = true;
					}
                    break;
                }
            }
        },
         restoreSubscriber: function(contrib){
            for (var i = 0; i < this._subscribers.length; i++) {
                if (this._subscribers[i].itemId == contrib.itemId) {
                   	this._subscribers[i]._isDeleted = false;
                    break;
                }
            }
        },
        getSubscribers: function(){
            return this._subscribers;
        },
        isChanged: function(){
            var trueLength = 0;
            for (var i = 0; i < this._subscribers.length; i++) {
                if (!this._subscribers[i]._isDeleted) {
                    trueLength++;
                    if (this._workItemSubscribers) {
                        if (!this._findInArray(this._subscribers[i], this._workItemSubscribers)) {
                            return true;
                        }
                    }
                }
            }
            
            if (trueLength != this._workItemSubscribers.length) {
                return true;
            }
            return false;
        },
		isSubscriber: function(contributor){
			var itemId = contributor.itemId;
			var subscribers = this._subscribers;
			var len = subscribers.length;
            for (var i = 0; i < len; i++) {
				var s = subscribers[i];
                if (itemId == s.itemId) {
                    return !s._isDeleted;
				}
            }
            return false;
		},
        _findInArray: function(item, array){
            for (var i = 0; i < array.length; i++) {
                if (item.itemId == array[i].itemId) 
                    return true;
            }
            return false;
        },
        /**
         * Returns an array of the itemIds of the current subscribers.
         */
        getCurrentSubscribers: function(){
            var subscribers = [];
            
            if (this._workItem && this._workItem.attributes.creator && this._workItem.attributes.creator.itemId) {
                subscribers.push({
					itemId: this._workItem.attributes.creator.itemId
				});
            }
            for (var i = 0; i < this._subscribers.length; i++) {
                if (!this._subscribers[i]._isDeleted) 
                    subscribers.push({
						itemId: this._subscribers[i].itemId
					});
            }
            
            return subscribers;
        },
		/*
		 * Add values to the updateLinks commands
		 *
		 */
		 getSaveCommands: function (allCommands) {
		 	if(!this._subscribers) return allCommands;
		 	var commands = [];
			for(var i=0; i<this._subscribers.length; i++) {
				var subscriber = this._subscribers[i];
				if(subscriber._isNew) {
					if (!subscriber._isDeleted) {
						// For a new approval, consolidate all commands into one command
						var command = {
						cmd: ADD_LINK,
								type: SUBSCRIBER_TYPE,
								itemId: subscriber.itemId,
								name: subscriber.name // not needed... used in history
						}
						commands.push(command);
					}
				} else { // Not new
					if (subscriber._isDeleted) {
						// For a new approval, consolidate all commands into one command
						var command = {
								cmd: REMOVE_LINK,
								type: SUBSCRIBER_TYPE,
								itemId: subscriber.itemId,
								name: subscriber.name // not needed... used in history
						}
						commands.push(command);
					}				
				}
			}
			
			var updateLinks = allCommands[UPDATE_LINKS] ? allCommands[UPDATE_LINKS] : [];
			
			for(var i=0;i<commands.length;i++){
				updateLinks.push(commands[i]);
			}
			
			if(commands.length >0) {
				allCommands[UPDATE_LINKS] = updateLinks;
			}
			
			return allCommands;
		 },
        // called when subscribers are added or removed
        _handleSubscriptionsSemanticChange: function(){
			for(var i=0;i<this._updateListeners.length;i++) {
				if(this._updateListeners[i].handleSubscriptionChange) {
					this._updateListeners[i].handleSubscriptionChange();
				}
			}
        }
    });
    
})();
