/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");

dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletPage");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("net.jazz.ajax.core.internal.loader");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

//
// Aliases
//
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var EventConstants = com.ibm.team.dashboard.web.ui.internal.Constants.Events.DashboardContainer;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var MetaNav = com.ibm.team.dashboard.web.ui.internal.MetaNav;

var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var ViewletPage = com.ibm.team.dashboard.web.ui.internal.ViewletPage;
var ViewletRegistry = com.ibm.team.dashboard.web.client.internal.ViewletRegistry;

/**
 * The widget for the content area (right side) of the dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardContainer",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class='com-ibm-team-dashboard-content' id='com-ibm-team-dashboard-content'></div>",
	
	constructor: function(args) {
		this.dashboardPage = args.dashboardPage;
		this.selectedTabId = args.selectedTabId;
		this.metaNav = null;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	isVisible: function() {
		return this.dashboardPage.isVisible();
	},
	init: function() {
		this.metaNav = new MetaNav({dashboardContainer: this});
		this.dashboardPage._layout._metaAreaContainer.appendChild(this.metaNav.domNode);
	},
	reload: function() {
		if (this.metaNav) {
			this.metaNav._reload();
		}
	},
	
	/*
	 * Returns the container's ServiceTracker, which tracks outstanding
	 * service calls so we can abort them on destroy.
	 */
	/* ServiceTracker */ getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},

	/*
	 * Sets the dashboard content to the given model object containing
	 * the complete state of the dashboard.  Called by DashboardPage 
	 * once DashboardContainer.domNode is appended to layout.
	 */
	setContent: function(dashboardItem) {
		this._dashboardItem = dashboardItem;
		this._dashboardActive = true;

		if(this._dashboardItem.getPermissions().modify) {
			this.editMode = this.isPrivateContributorDashboard() ?
					Constants.EDIT_AUTO_SAVE :
					Constants.EDIT_DEFERRED_SAVE;
		} else {
			this.editMode = Constants.READ_ONLY;
		}
		
		this.viewletPages = new Array(dashboardItem.getViewletPages().length);
		var viewletDefinitions = dashboardItem.getViewletDefinitions();
		if (viewletDefinitions) {
			// dashboard came with definitions; add them
			ViewletRegistry.addDefinitions(viewletDefinitions);
			this._loadViewlets(viewletDefinitions);
		} else {
			// didn't come with definitions, get them
			var serviceObj = {
				self: this,
				success: function(/* ViewletDefinition[] */definitions){
					ViewletRegistry.addDefinitions(definitions);
					dojo.hitch(this.self,this.self._loadViewlets(definitions));
				},
				_logError: function(errorObj) {
					errorObj.message = this.self.messages.viewletDefinitionsError + errorObj.message;
					dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObj]);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "_logError");
			ViewletRegistry.getDefinitionsFor(dashboardItem, srh, this.getServiceTracker());
		}
	},
	setEditMode: function (editMode) {
		this.metaNav.setEditMode(editMode);
		for (var i = 0; i < this.viewletPages.length; i++ ) {
			if (this.viewletPages[i]) {
				if (editMode == Constants.EDIT_AUTO_SAVE)
					this.viewletPages[i].setEditMode(true);
				else if (editMode == Constants.EDIT_DEFERRED_SAVE)
					this.viewletPages[i].setEditMode(true);
				else
					this.viewletPages[i].setEditMode(false);
			}
		}
		this.editMode = editMode;
	},
	/*
	 * Loads the viewlet modules for the given definitions, if not already
	 * loaded, then shows the dashboard.
	 */
	_loadViewlets: function(/* ViewletDefinition[] */ definitions) {
		var moduleIds = new Array(definitions.length);
		for (var i=0;i<definitions.length;i++) {
			moduleIds[i] = definitions[i].widget;
		}
		// load the viewlets in a batch
		var loader = net.jazz.ajax.core.internal.loader;
		loader.batch_load_async(moduleIds, dojo.hitch(this, this._showDashboard));
	},
	_showDashboard: function() {
		if (!this._isDestroyed) {
			this.metaNav.init({title: this._dashboardItem.getTitle(), viewletPageHelpers: this._dashboardItem.getViewletPages(), selectedTabId: this.selectedTabId});
			this._yieldTimeout = setTimeout(dojo.hitch(this, this._showDashboard2), 0);
		}
	},
	_showDashboard2: function() {
		if (!this._isDestroyed) {
			delete this._yieldTimeout;
			this.setEditMode(this.editMode);
			this.dashboardPage.updateBrowserTitle();
		}
	},
	isPrivateContributorDashboard: function() {
		return (this._dashboardItem.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !this._dashboardItem.isPublic());
	},
	_createViewletPage: function(/* ViewletPageHelper */ viewletPageHelper,/* Integer */ index) {
		if (this.viewletPages[index]) {
			this.viewletPages[index].destroy();
		}
		this.viewletPages[index] = new ViewletPage({viewletPageHelper: viewletPageHelper, tabIndex: index, dashboardContainer: this});
	},
	showViewletPage: function(index, viewletPageHelper) {
		var init = false;
		if (!this.viewletPages[index] || this.viewletPages[index].dirty) {
			this._createViewletPage(viewletPageHelper, index);
			init = true;
		}
		if (this.domNode.lastChild) {
			this.domNode.replaceChild(this.viewletPages[index].domNode, this.domNode.lastChild);
		}
		else {
			this.domNode.appendChild(this.viewletPages[index].domNode);
		}
		this.currentPage = this.viewletPages[index];
		if (init) {
			this.viewletPages[index].init();
		} else {
			this.viewletPages[index].connectAddViewletButton();
		}
	},
	moveViewletPage: function(startIndex, endIndex) {
		if (this.viewletPages[startIndex]) {
			var page = this.viewletPages[startIndex];
			this.viewletPages.splice(startIndex,1);
			this.viewletPages.splice(endIndex, 0, page);
		}
	},
	deleteCachedViewletPage: function(index) {
		this.viewletPages[index].destroy();
		this.viewletPages.splice(index, 1);
	},
	setDirty: function(index) {
		if (this.viewletPages[index]){
			this.viewletPages[index].dirty = true;	
		}
	},
	getCurrentPage: function() {
		return this.currentPage;
	},
	checkEmpty: function() {
		for (var i = 0; i < this.viewletPages.length; i++) {
			if (this.viewletPages[i]) {
				this.viewletPages[i].checkEmpty();
			}
		}
	},
	createTitle: function(title, node, handler, context, maxlength, width) {
		if (!context) {
			context = this;
		}
		if (!width) {
			width = "100%";
		}
		var title = document.createTextNode(title);
		node.appendChild(title);
        var inlineEdit = new dijit.InlineEditBox({autoSave: true, width: width, noValueIndicator: "?", editorParams: {maxLength: "60"}, onChange: dojo.hitch(context, handler)}, node);
        inlineEdit.startup();
		return inlineEdit;
	},
	addListener: function(/* int */ type, /* function */ callback) {
		switch(type) {
			case EventConstants.DASHBOARD_RENAMED:
				if (!this._dashboardRenamedListeners)
					this._dashboardRenamedListeners = new Array();
				this._dashboardRenamedListeners.push(callback);
				break;
		}
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		if (this._currentTab)
			this._currentTab.destroy();
		for(var i = 0; this.viewletPages && i < this.viewletPages.length; i++) {
			if (this.viewletPages[i]) {
				this.viewletPages[i].destroy();	
				this.viewletPages[i] = null;
			}		
		}
		this.metaNav.destroy();
		this.inherited(arguments);
		this._isDestroyed = true;
	},
	navigateAwayEvent: function(workbenchNavigation, onBeforeUnload) {
		if (onBeforeUnload) {
			if (!this.metaNav.autoSave && this._dashboardItem.isDirty()) {
				//block the user's navigate away attempt with a popup.
				return this.messages.unsavedWork;
			} else if (this._isEditingContents()) {
				return this.messages.editingContents;
			} else {
				return false;
			}
		}
		
		var navigateAwayParams = new Object(); //set defaults for return object
		navigateAwayParams.doNavigate = true;
		navigateAwayParams.destroyWidget = false;
		
		if (this.deleted) return navigateAwayParams;

		if (!this.metaNav.autoSave && this._dashboardItem.isDirty()) {
			var doConfirmDialog = true;
		} else {
			if (this._isEditingContents()) {
				var doConfirmDialog = true;
				var confirmText = this.messages.navigateAwayPrefix + this.messages.editingContentsUnload;
			}
		}

		if (!confirmText) var confirmText = this.messages.navigateAwayPrefix + this.messages.unsavedChanges;
		
		if (workbenchNavigation) confirmText += this.messages.canReturn;  //switching to another vertical, they can return to save changes later
		else confirmText += this.messages.willDiscard; //switching to another dashboard, they will lose changes if they continue.
		
		confirmText += this.messages.navigateAwaySuffix; 

		if (doConfirmDialog) {
			if (confirm(confirmText)) {
				//user wants to discard changes
				this._dashboardItem.unlock();
				if (workbenchNavigation && this._dashboardItem.isDirty()) {
					navigateAwayParams.destroyWidget = false;
				} else if (workbenchNavigation){
					var viewletPages = this.viewletPages;
					for (var i = 0; viewletPages && i < viewletPages.length; i++) {
						if (viewletPages[i]) {
							var columns = this.viewletPages[i].columns;
							for (var j = 0; columns && j < columns.length; j++) {
								var widgets = this.viewletPages[i].columns[j].widgets;
								for (var k = 0; widgets && k < widgets.length; k++) {
									var widget = widgets[k];
									if (widget && widget._viewlet)
										widget._viewlet.setActive(false);
								}
							}	
						}											
					}
				}
			} else {
				navigateAwayParams.doNavigate = false;
			}
		}
		if (navigateAwayParams.doNavigate) {
			this._dashboardActive = false;
			if (this.zoomDiv) this.zoomDiv.style.display = "none";
			if (this.currentPage && this.currentPage.killPopup) this.currentPage.killPopup(); 
		}
		return navigateAwayParams;
	},
	_isEditingContents: function() {
		//traverse the viewlet pages looking for viewlets that are in "Edit Contents" mode
		var viewletPages = this.viewletPages;
		for (var i = 0; viewletPages && i < viewletPages.length; i++) {
			if (viewletPages[i]) {
				var columns = this.viewletPages[i].columns;
				for (var j = 0; columns && j < columns.length; j++) {
					var widgets = this.viewletPages[i].columns[j].widgets;
					for (var k = 0; widgets && k < widgets.length; k++) {
						var widget = widgets[k];
						if (widget && widget._viewlet) {
							if (widget._viewlet.isEditing()) {
								return true;
							}
						}
					} 
				}	
			}											
		}
		return false;
	},
	isPageActive: function(pageId) {
		return this._dashboardActive && pageId == this.selectedTabId;
	},
	setActive: function() {
		this._dashboardActive = true;
		if (this.zoomDiv) this.zoomDiv.style.display = "block";
		if (this._updateZoomMargins) this._updateZoomMargins();
		if (this.currentPage) this.currentPage.setActive(true);
	},
	/*
	 * Returns the dashboard item being shown.
	 */	
	getDashboardItem: function() {
		return this._dashboardItem;
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
	},	
	_fireListeners: function(/* int */ type) {
		switch(type) {
			case EventConstants.DASHBOARD_RENAMED:
				if (this._dashboardRenamedListeners) {
					for (var i in this._dashboardRenamedListeners)
						this._dashboardRenamedListeners[i](this.getDashboardItem());
				}
				break;
		}
	},
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("DashboardContainer.js remote error: " + errorObj.message);
	}
});
})();
