/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.RelativeDateFormat");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.FormatUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");

dojo.require("net.jazz.ajax.ui.internal.HoverPopup");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "FeedMessages");

(function() {

/*
 * Aliases
 */
var FeedClient = com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient;
var PagedTableViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet;
var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var RelativeDateFormat = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.RelativeDateFormat;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FormatUtil = com.ibm.team.dashboard.web.ui.internal.util.FormatUtil;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

/*
 * Constants
 */
var PREFERENCE_ID_URL = "url";
var PREFERENCE_ID_USER = "user";
var PREFERENCE_ID_PASSWORD = "password";
var PREFERENCE_ITEMS_TO_SHOW = "items";
var _SERVICE_URI = "com.ibm.team.dashboard.viewlets.service.internal.feed.IFeedService";

var messages;

/*
 * A viewlet that displays a news feed.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet",
	PagedTableViewlet, {
	
	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "FeedMessages");
		this._entryCount = -1;
		this._columnCount = 1;
		this._trackReadMarks = true;
	},
	
	/*
	 * Initializes the feed viewlet.
	 */
	init: function() {
		if (this._trackReadMarks) {
			this.getSite().addMenuItem(messages.markAllRead, "com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet-markAllReadIcon", dojo.hitch(this, this._markAll, true));
			this.getSite().addMenuItem(messages.markAllUnread, "com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet-markAllUnreadIcon", dojo.hitch(this, this._markAll, false));
		}
		this.inherited(arguments);
	},

	/*
	 * Called by super class when entries are needed.
	 */
	fetch: function (/* ServiceResponseHandler */ srh, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details) {
		delete this._errorMsgNode;
		var url = this.getPreference(PREFERENCE_ID_URL);
		if (!url || url.length == 0) {
			if (!this._noFeedMsg) {
				// this part always shows
				this._noFeedMsg = document.createElement("span");
				this._noFeedMsg.appendChild(document.createTextNode(messages.noURL + " "));
				
				// this part only shows when dashboard in edit mode
				this._noFeedExtraMsg = document.createElement("span");
				var links = I18nUtil.addMessage(this._noFeedExtraMsg, messages.editSettings);
				this.connect(links[0], "onclick", dojo.hitch(this, function(e) {
					e.preventDefault();
					this.getSite().toggleSettings();
				}));
				this._noFeedExtraMsg.className = "editModeText";
				this._noFeedMsg.appendChild(this._noFeedExtraMsg);
				if (!this.getSite().isDashboardEditing()) {
					this._noFeedExtraMsg.style.display = "none";
				}
			}
			this._errorMsgNode = this._noFeedMsg;
		}
		var items = this.getPreference(PREFERENCE_ITEMS_TO_SHOW);
		if (!items || items < 1) {
			this._errorMsgNode = document.createTextNode(messages.errorItems)
		}
		
		// display error msg if there was an error
		if (this._errorMsgNode) {
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
		}
		else {
			// fetch icon if new url
			if (this._lastURL != url) {
				this._lastURL = url;
				this.getSite().setIcon(URLUtility.getContextRoot() + "/service/" + _SERVICE_URI + "/icon?url=" + encodeURIComponent(url));
			}
			// fetch feed
			var user = this.getPreference(PREFERENCE_ID_USER);
			var password = this.getPreference(PREFERENCE_ID_PASSWORD);
			var serviceObj = {
				_srh: srh,
				_viewlet: this,
				_entries: entries,
				_details: details,
				success: function(/* FeedResponseDTO */ response) {
					this._viewlet.handleFetch(response, this._entries, this._details, this._srh);
				},
				error: function(/* Error */ error) {
					this._srh.doErrorCallback(error);
				}
			};
			var srh2 = new ServiceResponseHandler(serviceObj, "success", "error");
			FeedClient.getFeed(url, user, password, null, this.getReferenceItemId(), startIndex, count, entries, details, !this._feedInfo, srh2, this.getServiceTracker());
		}
	},
	
	/*
	 * Called when an entry's tooltip is needed. Subclasses should override.
	 */
	fetchTooltip: function(/* EntryDTO */ entry, /* ServiceResponseHandler */ srh) {
		// fetch details of the specific item
		var url = this.getPreference(PREFERENCE_ID_URL);
		var user = this.getPreference(PREFERENCE_ID_USER);
		var password = this.getPreference(PREFERENCE_ID_PASSWORD);
		FeedClient.getFeed(url, user, password, entry.items[0].linkURL, null, null, null, true, true, false, srh, this.getServiceTracker());
	},
	
	/*
	 * Returns the css class for this viewlet; called by superclass.
	 */
	/* string */ getContentClass: function() {
		return "com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet";
	},
	
	/*
	 * Returns info for the current feed, or null if there is none or we don't
	 * have it yet.
	 */
	/* FeedInfoDTO */ getFeedInfo: function() {
		return this._feedInfo;
	},
	
	/*
	 * Returns the number of items to show per page. Called by superclass, and to
	 * be overriden by subclasses.
	 */
	/* number */ getItemsPerPage: function() {
		return this.getPreference(PREFERENCE_ITEMS_TO_SHOW);
	},
	
	/*
	 * Returns the message to display when there are no feed items to show.
	 */
	/* string */ getNoItemsMessage: function() {
		return messages.noItems;
	},
	
	/*
	 * Returns the message to display when there is no data available,
	 * including error messages.
	 */
	/* Node */ getNoResultsMessage: function() {
		if (this._errorMsgNode) {
			return this._errorMsgNode;
		}
		return document.createTextNode(this.getNoItemsMessage());
	},
	
	/*
	 * Returns the id of the latest reference feed item. The reference item is the
	 * first item of a feed at a given time (index 0).
	 */
	/* string */ getReferenceItemId: function() {
		if (this._trackReadMarks) {
			var readState = this.getContributorState().get("read");
			if (readState && readState[0]) {
				return readState[0];
			}
		}
	},
	
	/*
	 * Returns the total number of results in the feed; called by superclass.
	 */
	/* number */ getResultCount: function() {
		return this._entryCount;
	},
	
	/*
	 * Returns an appropriate URL for the viewlet's title link.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return this._feedInfo ? this._feedInfo.linkURL : "#";
	},
	
	/*
	 * Handles a feed request's response. Subclasses should use this handler
	 * for feed responses.
	 */
	handleFetch: function(/* FeedResponseDTO */ response, /* boolean */ entries, /* boolean */ details, /* ServiceResponseHandler */ srh) {
		// was there an error?
		if (response.errorMsg) {
			// show error msg
			this._errorMsgNode = document.createElement("span");
			this._errorMsgNode.className = "errorText";
			this._errorMsgNode.innerHTML = response.errorMsg;
			this._entryCount = 0;
			this._updateSecondaryTitle();
			srh.doCallback();
		}
		else {
			// if response had feed info, store it
			if (response.info) {
				this.setFeedInfo(response.info);
			}
			
			// update result count
			this._entryCount = response.entryCount;
			this._updateSecondaryTitle();

			// update read marks
			if (this._trackReadMarks) {
				this._loadReadMarks(response);
			}
			
			// package results
			if (response.entryCount > 0) {
				var result = new Object();
				if (entries) {
					result.entries = response.entries;
				}
				if (details) {
					result.details = response.entries;
				}
				srh.doCallback(result);
			}
			else {
				// superclass will show no entries msg
				srh.doCallback();
			}
		}
	},
	
	/*
	 * Called by platform to see if viewlet has any zoomable content.
	 */
	/* boolean */ hasZoomableContent: function() {
		return this._entryCount > 0;
	},
	
	/*
	 * Renders the details for the given entry.
	 */
	/* Node[] */ renderDetails: function(/* EntryDTO */ entry) {
		var columns = new Array();
		var td = document.createElement("td");
		td.className = "details";
		if (this._columnCount > 1) {
			columns.push(document.createElement("td"));
			td.colSpan = this._columnCount - 1;
		}
		var buf = [];
		var items = entry.items;
		for (var i=0;i<items.length;i++) {
			var item = items[i];
			buf.push("<div class=\"item\">");
			if (item.date) {
				buf.push("<div class=\"date\">" + RelativeDateFormat.formatDif(new Date(item.date), true, true) + "</div>");
			}
			if (item.description) {
				buf.push(HTMLUtility.sanitizeHTML(item.description));
			}
			else {
				buf.push("<span class=\"description-empty\">(" + messages.noDesc + ")</span>");
			}
			buf.push("</div>");
		}
		td.innerHTML = buf.join("");
		
		// set style if read
		if (this._trackReadMarks) {
			dojo.addClass(td, this._isRead(entry) ? "read-details" : "unread-details");
		}
		
		// mark read immediately after an item is zoomed
		this._markRead(entry, false);
		columns.push(td);
		return columns;
	},
	
	/*
	 * Renders an entry for the given entry.
	 */
	/* Node[] */ renderEntry: function(/* EntryDTO */ entry, /* EntryDTO */ entryDetails, /* boolean */ zoomed) {
		var columns = new Array();
		var firstItem = entry.items[0];
		
		// render icon if exists
		if (firstItem.category) {
			var td = document.createElement("td");
			dojo.addClass(td, "iconCell");
			var div = document.createElement("div");
			dojo.addClass(div, "icon");
			dojo.addClass(div, firstItem.category);
			var imgDiv = document.createElement("div");
			if (firstItem.overlay) {
				var img = document.createElement("img");
				img.src = firstItem.overlay;
				if (firstItem.overlayClass)
					dojo.addClass(imgDiv, firstItem.overlayClass);
				imgDiv.appendChild(img);
			} else if (dojo.isFF) {
				// workaround a firefox display bug when there is a row with an overlay above a row without one
				imgDiv.style.display = "none";
				var img = document.createElement("img");
				img.src = "";
				imgDiv.appendChild(img);
			}
			div.appendChild(imgDiv);
			td.appendChild(div);
			columns.push(td);
		}
		
		// render count for multi-item entries
		var td = document.createElement("td");
		var count = entry.items.length;
		if (count > 1) {
			var countSpan = document.createElement("span");
			countSpan.className = "count";
			countSpan.innerHTML = "[" + count + "] ";
			td.appendChild(countSpan);
		}
		
		// render link
		var a = document.createElement("a");
		if (!firstItem.disableLink)
			a.href = firstItem.linkURL;
		else
			a.href = "#";
		if (firstItem.targetLink)
			a.target = firstItem.targetLink;
		
		a.innerHTML = FormatUtil.insertWordBreaks(HTMLUtility.sanitizeHTML(firstItem.title), 20);
		var tooltipSpan = document.createElement("span");
		tooltipSpan.innerHTML = "<span class=\"com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet-tooltip\"><span class=\"loading\">" + messages.loading + "</span></span>";
		var tooltip = new HoverPopup({connectNode: [a], callback: dojo.hitch(this, this._onTooltipOpen), callbackArg: {tooltipSpan: tooltipSpan, entry: entry}, preDisplayCheck: dojo.hitch(this, this._onPreTooltipOpen, entry), closeCallback: dojo.hitch(this, this._onTooltipClose), repopulateDelay: -1}, tooltipSpan);
		a.style.zoom = "normal"; // Bug #44703
		td.appendChild(a);
		columns.push(td);
		
		if (firstItem.disableLink) {
			this.connect(a, "onclick", function(event) {
				event.preventDefault();
				tooltip.open();
			});
		}
		
		// set style if read
		if (this._trackReadMarks) {
			var className = this._isRead(entry) ? "read-entry" : "unread-entry";
			if (firstItem.critical) {
				className = "critical-" + className;
			}
			for (var i=0;i<columns.length;i++) {
				dojo.addClass(columns[i], className);
			}
		}

		// render date
		if (!zoomed && firstItem.date) {
			var separatorSpan = document.createElement("span");
			separatorSpan.innerHTML = "&nbsp ";
			td.appendChild(separatorSpan);
			var dateSpan = document.createElement("span");
			dateSpan.className = "date";
			dateSpan.innerHTML = RelativeDateFormat.formatDif(new Date(firstItem.date), false, true);
			td.appendChild(dateSpan);
		}
		
		entry.columns = columns;
		return columns;
	},

	/*
	 * Participate in the viewlet reset by setting number of entries
	 * back to initial state.
	 */
	reset: function(/* boolean */ showLoading) {
		this._entryCount = -1;
		this.inherited(arguments);
	},

	/*
	 * Sets the number of columns this viewlet had.
	 */
	setColumnCount: function(/* number */ columnCount) {
		this._columnCount = columnCount;
	},
	
	/*
	 * Called when the dashboard switches to or from edit mode.
	 */
	setDashboardEditing: function(editing) {
		// if the extra no feed text is visible or cached
		if (this._noFeedExtraMsg) {
			// show it only if dashboard is editing
			this._noFeedExtraMsg.style.display = editing ? "inline" : "none";
		}
	},
	
	/*
	 * Sets the info about the current feed; to be called by subclasses when
	 * they get the info.
	 */
	setFeedInfo: function(/* FeedInfoDTO */ feedInfo) {
		this._feedInfo = feedInfo;
	},
	
	/*
	 * Sets whether or not the viewlet should track read/unread marks. Must
	 * be called during initialization.
	 */
	setTrackReadMarks: function(/* boolean */ value) {
		this._trackReadMarks = value;
	},
	
	/*
	 * Refresh or update depending on which settings changed; called by platform.
	 */
	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings.preferences.url != oldSettings.preferences.url
			|| newSettings.preferences.user != oldSettings.preferences.user
			|| newSettings.preferences.password != oldSettings.preferences.password) {
			this.reset(true);
			this._entryCount = -1;
			this.getSite().refresh();
		}
		else {
			this.update();
			this.updateTotalPageFooter();
		}
	},
	
	/*
	 * Update secondary title on update.
	 */
	update: function() {
		this.inherited(arguments);
		this._updateSecondaryTitle();
	},
	
	/*
	 * Returns whether or not the given entry is read.
	 */
	_isRead: function(/* EntryDTO */ entry) {
		var items = entry.items;
		for (var i=0;i<items.length;i++) {
			if (!this._readMarks[items[i].index]) {
				return false;
			}
		}
		return true;
	},
	
	/*
	 * Marks all items in this feed as either read or unread.
	 */
	_markAll: function(/* boolean */ read) {
		if (this._trackReadMarks) {
			for (var i=0;i<this._readMarks.length;i++) {
				this._readMarks[i] = read;
			}
			this._unreadCount = read ? 0 : this._readMarks.length;
			this.update();
			this._updateSecondaryTitle();
			this._saveReadMarks();
		}
	},

	/*
	 * Marks the given entry as read, updates styles, and saves contributor
	 * memento.
	 * 
	 * @param entry the entry to mark as read
	 * @param immediate whether or not to update the UI immediately
	 */
	_markRead: function(/* EntryDTO */ entry, /* boolean */ immediate) {
		if (this._trackReadMarks) {
			var items = entry.items;
			for (var i=0;i<items.length;i++) {
				var item = items[i];
				if (!this._readMarks[item.index]) {
					this._readMarks[item.index] = true;
					this._unreadCount--;
				}
			}
			if (immediate) {
				var isCritical = entry.items.length > 0 ? entry.items[0].critical : false;
				for (var i=0;i<entry.columns.length;i++) {
					var td = entry.columns[i];
					dojo.removeClass(td, "unread-entry");
					dojo.removeClass(td, "critical-unread-entry");
					dojo.addClass(td, isCritical ? "critical-read-entry" : "read-entry");
				}
				this._updateSecondaryTitle();
			}
			this._saveReadMarks();
		}
	},
	
	/*
	 * Called when the hover tooltip is about to open. Returns whether or not
	 * it should go ahead with it.
	 */
	_onPreTooltipOpen: function(/* EntryDTO */ entry) {
		// stop refreshing as long as the hover is visible
		this.getSite().setAutoRefresh(false);
		
		// don't show tooltips when zoomed (info already visible)
		if (!this.isZoomed()) {
			if (this._trackReadMarks) {
				this._markRead(entry, true);
			}
			return true;
		}
		return false;
	},
	
	/*
	 * Called when the hover tooltip has been closed.
	 */
	_onTooltipClose: function() {
		// ok safe to refresh again
		this.getSite().setAutoRefresh(true);
	},
	
	/*
	 * Called when the hover tooltip is opened. It initially contains a loading
	 * message, and is populated here.
	 */
	_onTooltipOpen: function(/* object */ args) {
		var serviceObj = {
			_args: args,
			_viewlet: this,
			success: function(/* FeedResponseDTO */ response) {
				var html = "<span class=\"com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet-tooltip\">";
				if (response.errorMsg) {
					html += this._viewlet._messages.errorTooltip + ":<br/>" + response.errorMsg;
				}
				else {
					html += this._viewlet._renderTooltipEntry(response.entries);
				}
				html += "</span>";
				this._args.tooltipSpan.innerHTML = html;
				this._args.tooltip.refresh();
			},
			error: function(/* Error */ error) {
				this._args.tooltipSpan.innerHTML = "<span class=\"com-ibm-team-dashboard-viewlets-web-ui-internal-FeedViewlet-tooltip\">" + this._viewlet._messages.errorTooltip + ":<br/>" + error + "</span>";
				this._args.tooltip.refresh();
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "error");
		this.fetchTooltip(args.entry, srh);
	},
	
	/*
	 * Renders the tooltip for an entry containing one or more items.
	 */
	_renderTooltipEntry: function(/* EntryDTO[] */ entries) {
		if (entries && entries.length == 1) {
			var entry = entries[0];
			var items = entry.items;
			var num = items.length;
			var firstItem = items[num-1];
			var buf = [];
			buf.push("<div class=\"header-" + (num > 1 ? "multi" : "single") + "\">");
			buf.push("<a class=\"prev prev-disabled\" href=\"#\" title=\"" + messages.prevPage + "\" onclick=\"com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet.prevPage(event); return false;\" ondblclick=\"com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet.prevPage(event);return false;\"></a>");
			buf.push("<div class=\"pagenum\">1/" + items.length + "</div>");
			buf.push("<a class=\"next" + (num == 1 ? " next-disabled" : "") + "\" href=\"#\" title=\"" + messages.nextPage + "\" onclick=\"com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet.nextPage(event); return false;\" ondblclick=\"com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet.nextPage(event);return false;\"></a>");
			buf.push("</div>");
			buf.push("<div class=\"body-" + (num > 1 ? "multi" : "single") + "\" page=\"0\">");
			buf.push("<div class=\"title\">");
			if (!firstItem.disableLink)
				buf.push("<a href=\"" + firstItem.linkURL + "\">");
			buf.push(HTMLUtility.sanitizeHTML(firstItem.title));
			if (!firstItem.disableLink)
				buf.push("</a>");
			buf.push("</div>");
			buf.push("<div class=\"content\">");
			this._renderTooltipItem(firstItem, buf);
			buf.push("</div></div><div style=\"display: none\">");
			for (var i=num-1;i>=0;i--) {
				buf.push("<div>");
				this._renderTooltipItem(items[i], buf);
				buf.push("</div>");
			}
			buf.push("</div>");
			return buf.join("");
		}
		else {
			return "<span class=\"description-empty\">(" + messages.noDesc + ")</span>";
		}
	},

	/*
	 * Renders the tooltip for an item.
	 */
	_renderTooltipItem: function(/* ItemDTO[] */ item, /* string[] */ buf) {
		if (item.date) {
			buf.push("<div class=\"date\">" + dojo.string.substitute(messages.onDate, [ RelativeDateFormat.formatDif(new Date(item.date), true, true) ]) + ":</div>");
		}
		if (item.description) {
			buf.push(HTMLUtility.sanitizeHTML(item.description));
		}
		else {
			buf.push("<span class=\"description-empty\">(" + messages.noDesc + ")</span>");
		}
	},
	
	/*
	 * Updates the viewlet's secondary title based on the number of total
	 * items and unread items.
	 */
	_updateSecondaryTitle: function() {
		this.updateTotalPageFooter();
		var heading = "";
		if (this._trackReadMarks && this._unreadCount > 0) {
			heading = "(" + dojo.string.substitute(messages.titleNew, [ this._unreadCount ]) + ")";
		}
		this.getSite().setSecondaryTitle(heading);
	},
	
	/*
	 * Loads or updates the boolean array of read/unread marks for the items
	 * in this viewlet's feed.
	 */
	_loadReadMarks: function (/* FeedResponseDTO */ response) {
		if (this._trackReadMarks) {
			var state = this.getContributorState().get("read");
			this._unreadCount = 0;
			this._referenceItemId = response.ref;
			this._readMarks = new Array(response.itemCount);
			if ((!response.refIndex && response.refIndex != 0) || response.refIndex == -1 || !state) {
				for (var i = 0; i < this._readMarks.length; i++) {
					this._readMarks[i] = false;
					this._unreadCount++;
				}
			}
			else {
				var prevIndexes = this._unserialize(state[1]);
				for (var i = 0; i < response.refIndex; i++) {
					this._readMarks[i] = false;
					this._unreadCount++;
				}
				for (var i = response.refIndex; i < this._readMarks.length; i++) {
					if (prevIndexes[i - response.refIndex]) {
						this._readMarks[i] = true;
					}
					else {
						this._readMarks[i] = false;
						this._unreadCount++;
					}
				}
			}
			this._updateSecondaryTitle();
		}
	},
	
	/*
	 * Saves the read/unread marks in the contributor memento.
	 */
	_saveReadMarks: function() {
		if (this._trackReadMarks) {
			var newState = [this._referenceItemId, this._serialize(this._readMarks)];
			this.getContributorState().set("read", newState);
			// store version for future migration needs
			if (this.getContributorState().get("version") != 1) {
				this.getContributorState().set("version", 1);
			}
		}
	},
	
	/*
	 * Serializes an array of booleans into a compact form containing the number
	 * of elements and a series of indexes where the read/unread state changes, starting
	 * from false (delta modulation).
	 * 
	 * This is effective because feed items typically come in bunches of read and unread
	 * items. Typically it's a bunch of newer unread item followed by a bunch of older read
	 * ones.
	 * 
	 * e.g.
	 * [true, true, false, false] -> [4, 2]
	 * [false, false, false] -> [3]
	 * [true, true, true, true, true] -> [5, 0]
	 * [true, false, true, false] -> [4, 1, 2, 3, 4]
	 */
	/* number[] */ _serialize: function(/* boolean[] */ array) {
		var serialized = [array.length];
		var currentState = false;
		for (var i=0;i<array.length;i++) {
			if (array[i] != currentState) {
				serialized.push(i);
				currentState = array[i];
			}
		}
		return serialized;
	},
	
	/*
	 * Unserializes a previously serialized array of booleans (see this._serialize()).
	 */
	/* boolean[] */ _unserialize: function(/* number[] */ serialized) {
		var unserialized = new Array(serialized[0]);
		var currentIndex = 1;
		var currentState = false;
		for (var i=0;i<unserialized.length;i++) {
			if (currentIndex < serialized.length && i == serialized[currentIndex]) {
				currentState = !currentState;
				currentIndex++;
			}
			unserialized[i] = currentState;
		}
		return unserialized;
	}
});

/*
 * Static methods.
 */
dojo.mixin(com.ibm.team.dashboard.viewlets.web.ui.internal.FeedViewlet, {
	
	prevPage: function(/* Event */ event) {
		this._flipPage(event, false);
	},
	
	nextPage: function(/* Event */ event) {
		this._flipPage(event, true);
	},
	
	_flipPage: function(/* Event */ event, /* boolean */ isNext) {
		dojo.stopEvent(event);
		var target = event.target;
		if (!target) {
			target = event.srcElement;
		}
		if (target) {
			// find nodes using relation to target
			var topNodes = target.parentNode.parentNode.childNodes;
			var headerNodes = topNodes[0].childNodes;
			var bodyDiv = topNodes[1];
			var hiddenDiv = topNodes[2];
			var itemDivs = hiddenDiv.childNodes;
			var num = itemDivs.length;
			if (!bodyDiv._page) {
				bodyDiv._page = 0;
			}
			
			// change content
			if (isNext) {
				if (bodyDiv._page < num - 1) {
					bodyDiv._page++;
					bodyDiv.lastChild.innerHTML = itemDivs[bodyDiv._page].innerHTML;
				}
			}
			else {
				if (bodyDiv._page > 0) {
					bodyDiv._page--;
					bodyDiv.lastChild.innerHTML = itemDivs[bodyDiv._page].innerHTML;
				}
			}
			
			// update page numbers
			var page = bodyDiv._page;
			var pagenumSpan = headerNodes[1];
			pagenumSpan.innerHTML = (page + 1) + "/" + num;
			
			// update page button enablement
			var func = (page == 0) ? dojo.addClass : dojo.removeClass;
			var prevLink = headerNodes[0];
			func(prevLink, "prev-disabled");
			func = (page >= (num - 1)) ? dojo.addClass : dojo.removeClass;
			var nextLink = headerNodes[2];
			func(nextLink, "next-disabled");
		}
	}
});

})();
