/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.PlansViewlet");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.apt.web.client.util");
dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient");

dojo.require("com.ibm.team.apt.web.ui.internal.common.ProgressInformation");

dojo.require("dojox.data.dom");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PlansMessages");

(function() {

var replaceChildren			= dojox.data.dom.replaceChildren;

var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var PagedTableViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardPlansClient = com.ibm.team.dashboard.viewlets.web.client.internal.PlansClient;
var PlanUtil = com.ibm.team.apt.web.client.util;
var ProgressBar = com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar;
var ProgressInformation = com.ibm.team.apt.web.ui.internal.common.ProgressInformation;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages;
	
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.PlansViewlet",
	PagedTableViewlet, {
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PlansMessages");
	},
	
	onInit: function() {
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			var defaultLineId = this.getPreference("defaultLine");
			if (defaultLineId != -1) {
				this._defaultLine = -1;
				this._initDefaultLine(defaultLineId);
			}
		}
		this._updateCurrentIteration();
		this._updateTitle();
		this._linePrefOptions = new Object();
	},
	
	_initDefaultLine: function(id) {
		var serviceObject = {
			self: this,
			success: function(dto) {
				if (dto && dto.itemId) 
					this.self._defaultLine = dto.itemId;
				else {
					// set defaultLine to -2 as a flag to indicate that it's invalid
					this.self._defaultLine = -2;
					if (this.self._currentIterationTimout) {
						clearTimeout(this.self._currentIterationTimout);
						delete this.self._currentIterationTimout;
					}
					if (this.self._fetchTimout) {
						clearTimeout(this.self._fetchTimout);
						delete this.self._fetchTimout;
					}
					this.self.getSite().refresh();
					this.self.getSite().setLoading(false);
				}
			},
			failure: function(error) {
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		}
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardPlansClient.getDevelopmentLineById(id, srh, this.getServiceTracker());
	},
	
	onRefresh: function(){
		delete this._totalResults;
	},
	
	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		// handle Auto(none) scope
		if (!this.getScopeItem()) {
			this._totalResults = 0;
			this._totalResultsChanged();
			this.getSite().setSecondaryTitle("");
			srh.doCallback();
			return;
		}
		var serviceObject = {
			self: this,
			success: function(dto) {
				if (dto && dto.totalPlans > 0) {
					if (this.self._totalResults != dto.totalPlans) {
						this.self._totalResults = dto.totalPlans;
						this.self._totalResultsChanged();
					}
					if (details) {
						var count = dto.results.length;
						for (var i = 0; i < dto.results.length; i++) {
							var serviceObj = {
								result: dto.results[i],
								success: function(iteration) {
									this.result.iteration = iteration;
									count--;
									if (!count)
										srh.doCallback({entries: dto.results, details: dto.results});
								},
								failure: function(error) {
									console.error(error);
									srh.doErrorCallback(error);
								}
							}
							var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
							DashboardPlansClient.getIteration(dto.results[i].iteration, srh2, this.self.getServiceTracker());
						}
					} else {
						srh.doCallback({entries: dto.results});
					}
				} else {
					this.self._totalResults = 0;
					this.self._totalResultsChanged();
					srh.doCallback();
				}
			},
			failure: function(error) {
				console.error(error);
				srh.doErrorCallback(error);
			}
		};
		var params = {
			currentOnly: this.getPreference("active"),
			maxResults: count,
			startIndex: startIndex,
			includeTeamLoad: true
		};
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			params.projectAreaId = this.getScopeItem().itemId;
			params.relationshipType = PlanUtil.RELATED_DEVLINE;
			params.relatedItemId = this.getPreference("line");
			if (params.relatedItemId == -1)
				params.relatedItemId = this._defaultLine;
			if (params.relatedItemId == -2) {
				// if defaultLine is -2, the defaultLine preference was invalid
				return;
			}
			else if (params.relatedItemId == -1) {
				// if defaultLine is -1, then one has been specified, but is not back from the server yet
				// postpone the response
				this._fetchTimout = setTimeout(dojo.hitch(this, this.fetch, srh, startIndex, count, entries, details), 500);
				return;
			}
		} else if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			params.relationshipType = this.getScopeSubteams() ? PlanUtil.RELATED_TEAMHIERARCHY : PlanUtil.RELATED_TEAM;
			params.relatedItemId = this.getScopeItem().itemId;
		} else {
			params.relationshipType = PlanUtil.RELATED_CONTRIBUTOR;
			params.relatedItemId = this.getScopeItem().itemId;
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardPlansClient.searchIterationPlans(params, srh2, this.getServiceTracker());
	},
	
	getResultCount: function() {
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && (!this._defaultLine || this._defaultLine == -2))
			return 0;
		if (this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR && !this.getScopeItem())
			return 0;
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (!this.getScopeItem()) {
			return this._createNoScopeMsg();
		}
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && !this._defaultLine) {
			if (!this._notConfiguredMessage) {
				this._notConfiguredMessage = document.createElement("span");
				var links = I18nUtil.addMessage(this._notConfiguredMessage, messages.notConfigured);
				console.debug(links);
				this.connect(links[0], "onclick", this._toggleSetting);
			}
			return this._notConfiguredMessage;
		}
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && this._defaultLine == -2) {
			if (!this._invalidDefaultMessage) {
				this._invalidDefaultMessage = document.createElement("span");
				var links = I18nUtil.addMessage(this._invalidDefaultMessage, messages.invalidDefault);
				this.connect(links[0], "onclick", this._toggleSetting);
			}
			return this._invalidDefaultMessage;
		}
		return document.createTextNode(messages.noResults);
	},
	
	_toggleSetting: function (event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	getContentClass: function() {
		return "PlansViewlet";
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
	},
	
	/* Node[] */ renderEntry: function(/* object */ plan, /* object */ planDetails, /* boolean */ zoomed) {
		var columns = new Array();
		
		var href = PlanUtil.getPlanViewURI(plan.itemId);
		
		var nameCell = document.createElement("td");
		nameCell.className = "nameCell";
		var nameLink = document.createElement("a");
		nameLink.href = href;
		nameLink.innerHTML = HTMLUtility.escapeHTML(plan.label);
		nameLink.style.paddingRight = "0.5em";
		nameLink.style.zoom = "1";
		nameCell.appendChild(nameLink);
		columns.push(nameCell);
		
		var progressCell = document.createElement("td");
		progressCell.className = "progressCell";
		
		var progressWrapper = document.createElement("div");
		progressWrapper.className = "progressWrapper";
		
		if (zoomed) {
			var startDate = document.createElement("div");
			startDate.className = "startDate";
			startDate.appendChild(document.createTextNode(PlanUtil.formatDate(plan.iteration.startDate, messages.noStartDate)));
			progressWrapper.appendChild(startDate);
			
			var endDate = document.createElement("div");
			endDate.className = "endDate";
			endDate.appendChild(document.createTextNode(PlanUtil.formatDate(plan.iteration.endDate, messages.noEndDate)));
			progressWrapper.appendChild(endDate);
		}
		
		var constructorArgs= ProgressBar.constructorArgs(new ProgressInformation(plan.teamLoad));
		var loadBar = new ProgressBar(constructorArgs);
		loadBar.domNode.style.width = "";
		progressWrapper.appendChild(loadBar.domNode);
		
		if (zoomed) {
			var labelProvider= loadBar.getLabelProvider();

			var progressNumbers= document.createElement("div");
			progressNumbers.className = "progressNumbers";

			var hours = document.createElement("div");
			hours.className = "hours";
			replaceChildren(hours, labelProvider.getHoursTextNode());
			progressNumbers.appendChild(hours);
			
			var quality = document.createElement("div");
			quality.className = "quality";
			replaceChildren(quality, labelProvider.getEstimateTextNode());
			progressNumbers.appendChild(quality);
			
			progressWrapper.appendChild(progressNumbers);
		}
		
		progressCell.appendChild(progressWrapper);
		columns.push(progressCell);
		
		return columns;
	},
	
	renderDetails: function () {
		var cell = document.createElement("td");
		cell.colSpan = 2;
		cell.className = "divider";
		cell.innerHTML = "<div />";
		return [cell];
	},
	
	_updateCurrentIteration: function() {
		if (!this.getScopeItem() || this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR) {
			if (this.heading)
				this.heading.style.display = "none";
			return;
		}
		if (this.getPreference("line") == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA && (!this._defaultLine|| this._defaultLine == -2))
			return;
		var line = this.getPreference("line");
		if (line == -1 && this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			line = this._defaultLine;
			if (line == -1) {
				this._currentIterationTimout = setTimeout(dojo.hitch(this, this._updateCurrentIteration), 500);
				return;
			}
		}
		if (!this._currentIteration) {
			this.heading = document.createElement("div");
			this.heading.className = "globalHeading";
			this.heading.appendChild(document.createTextNode(messages.currentIteration));
			this._currentIteration = document.createElement("strong");
			this.heading.appendChild(this._currentIteration);
			this.setGlobalHeading(this.heading);
		} else {
			this.heading.style.display = "";
		}
		this._currentIteration.innerHTML = messages.loading;
		var serviceObject = {
			self: this,
			success: function(iteration) {
				this.self._currentIteration.innerHTML = HTMLUtility.escapeHTML(iteration.name);
			},
			failure: function(error) {
				//console.error(error);
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		}
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardPlansClient.getCurrentIteration(line == -1 ? "" : line, line == -1 ? this.getScopeItem().itemId : "", srh, this.getServiceTracker());
	},
	
	getTitleHyperlinkURL: function() {
		if (!this.getScopeItem()) {
			return null;
		}
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA) {
			var relationshipType = PlanUtil.RELATED_DEVLINE;
			var relatedItemId = this.getPreference("line");
			if (relatedItemId == -1)
				relatedItemId = this._defaultLine;
			if (!relatedItemId || relatedItemId == -2 || relatedItemId == -1)
				return "#action=jazz.viewPage&id=com.ibm.team.apt";
		} else if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA) {
			var relationshipType = this.getScopeSubteams() ? PlanUtil.RELATED_TEAMHIERARCHY : PlanUtil.RELATED_TEAM;
			var relatedItemId = this.getScopeItem().itemId;
		} else {
			var relationshipType = PlanUtil.RELATED_CONTRIBUTOR;
			var relatedItemId = this.getScopeItem().itemId;
		}
		return PlanUtil.getSearchURI(PlanUtil.createCustomQueryDescriptor(relationshipType, relatedItemId, this.getPreference("active") ? "true" : null));
	},
	
	settingChanging: function(/* string */ settingId) {
		var result = new Array();
		if (settingId == "scope" || settingId == "scopeItem")
			result.push("line");
		return result;
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var updateTitle = newSettings.userTitle != oldSettings.userTitle;
		if (oldSettings.preferences.line != newSettings.preferences.line ||
				newSettings.scope == DashboardConstants.SCOPE_CONTRIBUTOR)
			this._updateCurrentIteration();
		if (oldSettings.scopeItemId != newSettings.scopeItemId ||
				oldSettings.scopeSubteams != newSettings.scopeSubteams ||
				oldSettings.preferences.active != newSettings.preferences.active ||
				oldSettings.preferences.line != newSettings.preferences.line) {
			this.reset(true);
			this.getSite().refresh();
			updateTitle = true;
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
			this.update();
		}
		if (updateTitle)
			this._updateTitle();
	},
	
	getPreferenceOptions: function(/* string */ preferenceId, /* object */ currentSettings) {
		if (preferenceId == "line") {
			if (!currentSettings.scopeItemId || currentSettings.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
				return {options: [{label: messages.notApplicable, value: -1}]};
			}
			if (this._linePrefOptions[currentSettings.scopeItemId] && this._linePrefOptions[currentSettings.scopeItemId] != "fetching") {
				var result = dojo.clone(this._linePrefOptions[currentSettings.scopeItemId]);
				if (this.getPreference("line") == -1 && currentSettings.scope == DashboardConstants.SCOPE_PROJECT_AREA && this._defaultLine && this._defaultLine != -2)
					result.computedDefault = this._defaultLine;
				return result;
			}
			if (!this._linePrefOptions[currentSettings.scopeItemId]) {
				this._linePrefOptions[currentSettings.scopeItemId] = "fetching";
				var serviceObject = {
					self: this,
					success: function(dtos) {
						var options = new Array();
						for (var i = 0; i < dtos.length; i++) {
							options.push({label: dtos[i].name, value: dtos[i].itemId});
						}
						this.self._linePrefOptions[currentSettings.scopeItemId] = {options: options};
					},
					failure: function(error) {
						//console.error(error);
						dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
					}
				}
				var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
				DashboardPlansClient.getDevelopmentLines(currentSettings.scopeItemId, currentSettings.scope, srh, this.getServiceTracker());
			}
			return null;
		}
	},
	
	hasZoomableContent: function() {
		return this.getResultCount() > 0;
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var links = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			msgSpan.appendChild(editSpan);
			this.connect(links[0], "onclick", this._toggleSetting);
		}
		return msgSpan;
	},

	_updateTitle: function() {
		if (!this.getScopeItem())
			title = messages.titleDefault;
		else if (this.getScope() == DashboardConstants.SCOPE_CONTRIBUTOR) {
			title = dojo.string.substitute(this.getPreference("active") ? messages.titleUserCurrent : messages.titleUser, [this.getScopeItem().name]);
		} else {
			title = dojo.string.substitute(this.getPreference("active") ? messages.titleOtherCurrent : messages.titleOther, [this.getScopeItem().name]);
		}
		this.getSite().setTitle(title);
	}
});

})();
