/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.ServerStatusViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ServerStatusMessages");

(function() {

/*
 * Aliases
 */
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

var messages;
/*
 * A viewlet that summarizes the repository server's status.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.ServerStatusViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/ServerStatusViewlet.html"),

	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ServerStatusMessages");
		this._loading = messages.loading;
	},

	/*
	 * Returns an appropriate URL for the viewlet's title link.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return URLUtility.getContextRoot() + "/admin";
	},
	
	refresh: function() {
		this.getSite().setLoading(true);
		delete this._error;
		delete this._serverInfoDTO;
		var serviceObj = {
			_viewlet: this,
			success: function(/* ServerInfoDTO */ serverInfoDTO) {
				if (!this._viewlet.isDestroyed()) {
					this._viewlet._serverInfoDTO = serverInfoDTO;
					this._viewlet.update();
					this._viewlet.getSite().setLoading(false);
					this._viewlet.getSite().doneRefresh();
				}
			},
			failure: function(/* Error */ error) {
				if (!this._viewlet.isDestroyed()) {
					this._viewlet._error = error;
					this._viewlet.update();
					this._viewlet.getSite().setLoading(false);
					this._viewlet.getSite().doneRefresh();
					dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
				}
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		ServerStatusClient.getServerInfo(srh);
	},
	
	update: function() {
		if (this._error) {
			var errorSpan = document.createElement("span");
			errorSpan.innerHTML = "<span class=\"error\">" + messages.error + ": " + this._error.message + "</span>";
			this._content.replaceChild(errorSpan, this._content.firstChild);
		}
		else {
			var table = document.createElement("table");
			var tbody = document.createElement("tbody");
			this._addRow(messages.database, this._getDB(), tbody);
			this._addRow(messages.services, this._getServices(), tbody);
			this._addRow(messages.memory, this._getMemory(), tbody);
			this._addRow(messages.version, this._getVersion(), tbody);
			this._addRow(messages.uptime, this._getUptime(), tbody);
			table.appendChild(tbody);
			this._content.replaceChild(table, this._content.firstChild);
		}
	},
	
	_addRow: function(/* string */ label, /* string */ content, /* Node */ tbody) {
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.className = "label";
		td.innerHTML = label + ":&nbsp;&nbsp;";
		tr.appendChild(td);
		td = document.createElement("td");
		td.innerHTML = content;
		tr.appendChild(td);
		tbody.appendChild(tr);
	},
	
	_formatSizeInBytes: function(/* number */ size) {
		size = Math.floor(size/1024);
		var kb = size % 1024;
		size = Math.floor(size/1024);
		var mb = size;
		return mb < 1 ? kb + messages.kilobyte : mb + messages.megabyte;
	},

	_getDB: function() {
		if (this._serverInfoDTO.databaseAvailable) {
			return "<span class=\"ok-status\">" + messages.connected + "</span>";
		}
		else {
			return "<a class=\"error-status\" href=\"" + URLUtility.getContextRoot() + "/admin\">" + messages.unavailable + "</a>";
		}
	},
	
	_getMemory: function() {
		var info = this._serverInfoDTO;
		var percent = Math.round(info.freeMemory*100 / info.totalMemory);
		var allocated = this._formatSizeInBytes(info.totalMemory);
		var max = this._formatSizeInBytes(info.maxMemory);
		return dojo.string.substitute(messages.memoryInfo, [ percent, allocated, max ]);
	},
	
	_getServices: function() {
		var info = InitializationData.get("com.ibm.team.repository.service.serverInfoInitializer");
		if (info) {
			// number of failing services
			if (!info.servicesInitSuccess) {
				return "<span class=\"ok-status\">" + messages.ok + "</span>";
			}
			else {
				return "<a class=\"error-status\" href=\"" + URLUtility.getContextRoot() + "/admin\">" + messages.failures + "</a>";
			}
		}
		else {
			return "<span class=\"warning-status\">" + messages.unknown + "</span>";
		}
	},
		
	_getUptime: function() {
		var t = this._serverInfoDTO.uptime;
		t = Math.floor(t/1000);
		t = Math.floor(t/60);
		if (t < 1){
			return messages.lessThanOne;
		}
		var m = t % 60;
		t = Math.floor(t/60);
		var h = t % 24;
		t = Math.floor(t/24);
		var d = t;
		var str = "";
		if (d > 0) {
			str += d + " " + (d == 1 ? messages.day : messages.days);
		}
		if (h > 0) {
			str += (str.length > 0 ? ", " : "") + h + " " + (h == 1 ? messages.hour : messages.hours);
		}
		if (m > 0 && d == 0) {
			str += (str.length > 0 ? ", " : "") + m + " " + (m == 1 ? messages.minute : messages.minutes);
		}
		return str;
	},
	
	_getVersion: function() {
		var version = this._serverInfoDTO.version;
		var buildId = this._serverInfoDTO.buildId;
		if (version && buildId) {
			// both are available
			return version + " (" + buildId + ")";
		}
		else if (version) {
			// only version available
			return version;
		}
		else if (buildId) {
			// only build id available
			return buildId;
		}
		else {
			// none available
			return messages.notAvailable;
		}
	}
});

})();
