/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportBuildsViewlet");

/*
 * Imports 
 */
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");

(function() {
	
/*
 * Aliases 
 */
var TrendReportViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet;
var ReportsPreferenceProvider = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

/*
 * TODOs
 */
// Test frequent test failures / regressions routine to check if report found or not


/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportBuildsViewlet",
	TrendReportViewlet, {
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {
		// Call super
		this.inherited("init", arguments);

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		// Cascading parameters map
		this._groupParametersBuild = null;
		// Single parameters values
		this._singleParametersWhich = null;
		
		// Report class
		this._reportClass = this._static._VIEWLET_CLASS_NONE;
		this._setReportClass();
		
		// Parameter URI names
		this._parameterURINames = new Array();
		this._setParameterURINames();
		
		// Preference providers
		this._teamPreferenceProvider = null;
		this._componentPreferenceProvider = null;
	},	

	/*
 	 * Public Method
 	 * 
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		// Verify preferences are defined
		try {
			// Team
			this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_TEAM), true, "team");
			// Definition
			this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_DEFINITION), true, "definition");
			if ((this._reportClass == this._static._VIEWLET_CLASS_CODE) ||
				(this._reportClass == this._static._VIEWLET_CLASS_TEST)) {
				// Component
				this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_COMPONENT), true, "component");
			}
			if ((this._reportClass == this._static._VIEWLET_CLASS_BUILD) ||
				(this._reportClass == this._static._VIEWLET_CLASS_CODE)) {
				// Duration
				this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_DURATION), true, "duration");
				// Incomplete
				this._isBooleanDefined(this.getPreference(this._static._PREFERENCE_ID_INCOMPLETE), true, "incomplete");
			}
		} catch(errorObject) {
			this._displayMsgSelectPreferences();
			this.getSite().doneRefresh();
			return;
		}

		// Get the rendered report
		this._getReport(
			this._parameterURINames, 
			this.getParameterValues());		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		
		if (preferenceId == this._static._PREFERENCE_ID_TEAM) {
			// Team
			if (this._teamPreferenceProvider == null) {
				
				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.web/ui/internal/images/team-dash-icon.gif";
				
				this._teamPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceTeam",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizeTeamAreaPreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedBuild"
					});
			}
			return this._teamPreferenceProvider;		
				
		} else if (preferenceId == this._static._PREFERENCE_ID_COMPONENT) {
			// Component
			if (this._componentPreferenceProvider == null) {

				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.web/ui/internal/images/compont_obj.gif";

				this._componentPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceComponent",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizePreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedBuild"
					});
			}
			return this._componentPreferenceProvider;		
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedPreference, [preferenceId]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgPrefLoadingFailed, errorObject);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @param Object currentSettings
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceOptions: function(preferenceId, currentSettings) {
		// Note:  Incomplete preference not handled because it is not dynamic
		
		var result = null;
		// Get preference values
		if (preferenceId == this._static._PREFERENCE_ID_DEFINITION) {
			// Build Definition
			if (this._groupParametersBuild == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._groupParametersBuild == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionDefinition(currentSettings);
			
		} else if (preferenceId == this._static._PREFERENCE_ID_DURATION) {
			// Duration
			if (this._singleParametersWhich == null) {
				// The DTO has not been recieved from the REST service call yet.
				// Return null to start an interval-based polling of this method
				// until the DTO is recieved.
				return null;
			} else if (this._singleParametersWhich == this._staticCommon._DTO_PARAMETERS_FAILURE) {
				// Return an error list
				return this._getPreferenceFailureList();
			}
			return this._getPreferenceOptionDuration();
						
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedPreference, [preferenceId]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgPrefLoadingFailed, errorObject);
			return this._getPreferenceFailureList();
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Array
 	 * @overrides super
 	 */		
	settingChanging: function(preferenceId) {
		var depedencies = new Array();
		if (preferenceId == this._static._PREFERENCE_ID_TEAM) {
			// Team Area -> Build Definition
			depedencies.push(this._static._PREFERENCE_ID_DEFINITION);
		} else if (preferenceId == this._static._PREFERENCE_ID_DEFINITION) {
			if ((this._reportClass == this._static._VIEWLET_CLASS_CODE) ||
				(this._reportClass == this._static._VIEWLET_CLASS_TEST)) {
				// Build Definition -> Component
				depedencies.push(this._static._PREFERENCE_ID_COMPONENT);
			}
		}
		return depedencies;
	},

	// Private Methods -------------------------------------------------------->
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportBuildsViewlet";
		
		// Preferences
		this._static._PREFERENCE_ID_TEAM = "team";	
		this._static._PREFERENCE_ID_DEFINITION = "definition";	
		this._static._PREFERENCE_ID_COMPONENT = "component";	
		this._static._PREFERENCE_ID_DURATION = "duration";	
		this._static._PREFERENCE_ID_INCOMPLETE = "incomplete";	
		
		// Cascading parameter groups
		this._static._PARAMETER_GROUP_BUILD = "BuildResultsParameters";
		// Simple parameter values
		this._static._PARAMETER_SINGLE_WHICH = "WhichBuilds";
		// Not used.  Boolean values
		this._static._PARAMETER_SINGLE_INCOMPLETE = "IncludeIncompleteBuilds";

		this._static._VIEWLET_CLASS_NONE = 0;
		this._static._VIEWLET_CLASS_BUILD = 1;
		this._static._VIEWLET_CLASS_CODE = 2;
		this._static._VIEWLET_CLASS_TEST = 3;
		
	},		

	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceTeam: function(currentSettings) {
		// Team Area
		if (this._groupParametersBuild == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersBuild == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		return this._getPreferenceOptionTeam();
	},
	
	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceComponent: function(currentSettings) {
		// Team Area
		if (this._groupParametersBuild == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersBuild == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		return this._getPreferenceOptionComponent(currentSettings);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 * 			 1: Loaded
 	 * 			 0: Not Loaded
 	 * 			-1: Error
 	 */	
	_isParametersLoadedBuild: function() {
		return this._isParametersLoaded(this._groupParametersBuild);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setReportClass: function() {
		// Get the current report preference value
		this._reportUUID = this.getPreference(this._staticCommon._PREFERENCE_ID_REPORT);
		// Determine report class
		if (this._reportUUID == this._staticCommon._REPORT_BUILD_HEALTH) {
			// Build Health
			this._reportClass = this._static._VIEWLET_CLASS_BUILD;
			this._setMessageNoResults(this.messages.msgNoBuildResultsFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_BUILD_DURATION) {
			// Build Duration
			this._reportClass = this._static._VIEWLET_CLASS_BUILD;
			this._setMessageNoResults(this.messages.msgNoBuildResultsFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_JUNIT_TESTS) {
			// Number of Tests
			this._reportClass = this._static._VIEWLET_CLASS_CODE;
			this._setMessageNoResults(this.messages.msgNoJUnitDataFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_CODE_COVERAGE) {
			// Code Coverage
			this._reportClass = this._static._VIEWLET_CLASS_CODE;
			this._setMessageNoResults(this.messages.msgNoCodeCoverageFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_JUNIT_FAILURES) {
			// JUnit Test Failures
			this._reportClass = this._static._VIEWLET_CLASS_CODE;
			this._setMessageNoResults(this.messages.msgNoJUnitDataFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_COMPILATION_WARNINGS) {
			// Compilation Warnings
			this._reportClass = this._static._VIEWLET_CLASS_CODE;
			this._setMessageNoResults(this.messages.msgNoCompilationDataFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_STATIC_ANALYSIS) {
			// Static Analysis
			this._reportClass = this._static._VIEWLET_CLASS_CODE;
			this._setMessageNoResults(this.messages.msgNoStaticDataFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_FREQUENT_TEST_FAILURES) {
			// Frequent Test Failures
			this._reportClass = this._static._VIEWLET_CLASS_TEST;
			this._setMessageNoResults(this.messages.msgNoTestFailuresFound);
		} else if (this._reportUUID == this._staticCommon._REPORT_FREQUENT_TEST_REGRESSIONS) {
			// Frequent Test Regressions
			this._reportClass = this._static._VIEWLET_CLASS_TEST;
			this._setMessageNoResults(this.messages.msgNoTestRegressionsFound);
		} else {
			// BUG: Error message gets overidden in viewlet content area
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedReport, [this._reportUUID]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgReportSettingFailed, errorObject);
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setParameterURINames: function() {
		// Project
		this._parameterURINames.push("ProjectAreaName");
		// Team
		this._parameterURINames.push("TeamAreaName");
		// Definition
		this._parameterURINames.push("BuildTypeId");
		if ((this._reportClass == this._static._VIEWLET_CLASS_CODE) ||
			(this._reportClass == this._static._VIEWLET_CLASS_TEST)) {
			// Component
			this._parameterURINames.push("Component");
		}
		if ((this._reportClass == this._static._VIEWLET_CLASS_BUILD) ||
			(this._reportClass == this._static._VIEWLET_CLASS_CODE)) {
			// Duration
			this._parameterURINames.push("WhichBuilds");
			// Incomplete
			this._parameterURINames.push("IncludeIncompleteBuilds");
		}		
		// Zoom
		this._parameterURINames.push(this._staticCommon._PARAMETER_NAME_ZOOM);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_resetCascadingParameters: function() {
		this._groupParametersBuild = null;
		this._singleParametersWhich = null;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_getParameters: function() {
		// Update loading message
		this._updateLoadingMessage(this.messages.msgLoading);
		// Show the loading message
		this._showLoadingArea();
		// Asynchronous call
		this._getGroupParameters(
			this._static._PARAMETER_GROUP_BUILD,
			this._callBackSuccessGetGroupParameters,
			this._callBackErrorGetGroupParameters);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return boolean
 	 * @overrides super
 	 */		
	_isSettingsEqual: function(oldSettings, newSettings) {

		if (oldSettings.preferences.team != newSettings.preferences.team) {
			// Team
			return false;
		} else if (oldSettings.preferences.definition != newSettings.preferences.definition) {
			// Definition
			return false;
		} 

		if ((this._reportClass == this._static._VIEWLET_CLASS_CODE) ||
			(this._reportClass == this._static._VIEWLET_CLASS_TEST)) {

			if (oldSettings.preferences.component != newSettings.preferences.component) {
				// Component
				return false;
			}
		}

		if ((this._reportClass == this._static._VIEWLET_CLASS_BUILD) ||
			(this._reportClass == this._static._VIEWLET_CLASS_CODE)) {
				
			if (oldSettings.preferences.duration != newSettings.preferences.duration) {
				// Duration
				return false;
			} else if (oldSettings.preferences.incomplete != newSettings.preferences.incomplete) {
				// Incomplete
				return false;
			}
		}
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @return void
 	 * @overrides super
 	 */	
	_addParametersToNode: function(node) {
		// Create parameter names
		var parameterNames = new Array();
		// Create parameter values
		var parameterValues = new Array();		

		// Team
		var team = this.getPreference(this._static._PREFERENCE_ID_TEAM);
		team = this._normalizeMultiValue(team, "_normalizeContentAreaSingleTeam");
		parameterNames.push(this.messages.msgTeamArea);
		parameterValues.push(team);
		
		// Definition
		var definition = this.getPreference(this._static._PREFERENCE_ID_DEFINITION);		
		definition = this._normalizePreferenceValue(definition);
		parameterNames.push(this.messages.msgBuildDefinition);		
		parameterValues.push(definition);
		
		if ((this._reportClass == this._static._VIEWLET_CLASS_CODE) ||
			(this._reportClass == this._static._VIEWLET_CLASS_TEST)) {
			// Component
			var component = this.getPreference(this._static._PREFERENCE_ID_COMPONENT);
			component = this._normalizePreferenceMultiValue(component);
			parameterNames.push(this.messages.msgComponent);
			parameterValues.push(component);
		}

		if ((this._reportClass == this._static._VIEWLET_CLASS_BUILD) ||
			(this._reportClass == this._static._VIEWLET_CLASS_CODE)) {
				
			// Duration
			var duration = this.getPreference(this._static._PREFERENCE_ID_DURATION);
			duration = this._normalizePreferenceValue(duration);
			duration = this._normalizeDurationValue(duration);
			parameterNames.push(this.messages.msgDuration);
			parameterValues.push(duration);
			
			// Incomplete
			var incomplete = this.getPreference(this._static._PREFERENCE_ID_INCOMPLETE);
			parameterNames.push(this.messages.msgIncompleteBuilds);
			if (incomplete) {
				parameterValues.push(this.messages.msgYes);
			} else {
				parameterValues.push(this.messages.msgNo);
			}
		}

		this._displayParameters(node, parameterNames, parameterValues);
		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizePreferenceValue: function(value) {
		if (value == this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
			return this.messages.msgAll;
		}
		return value;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeDurationValue: function(value) {
		// Prepended number used for sorting in Reports
		// Label unavailable from framework
		// Remove prepended number manually
		var pattern = /^\d+\s*?/;
		return value.replace(pattern, "");
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @return String
 	 */	
	_normalizeParameterValue: function(value) {
		if (value == this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
			return this._quote("");
		}
		return this._quote(value);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array
 	 */	
	getParameterValues: function() {
		// Example
		// https://localhost:9443/jazz/secure/service/com.ibm.team.reports.common.internal.service.IReportRestService
		// /renderedReport?reportUUID=build.MicroBuildHealth
		// &embeddable=true
		// &parameterName=ProjectAreaName
		// &parameterName=TeamAreaName
		// &parameterName=BuildTypeId
		// &parameterName=WhichBuilds
		// &parameterName=IncludeIncompleteBuilds
		// &parameterValue='%7BCurrent%20Project%20Area%7D'
		// &parameterValue='%2FJazz%2FWork%20Item'
		// &parameterValue='continuous.workitem.jazz'
		// &parameterValue='7%20Last%2050%20Builds'
		// &parameterValue='true'
		// &webui=true
		
		// Example
		// Multiple parameter value:  '/Jazz/Build','/Jazz/Dashboard'
		// &parameterValue='%2FJazz%2FBuild'%2C'%2FJazz%2FDashboard'
		
		// Note:  No need to encode the parameter values; since, this will be
		// done when making the REST service call
		
		var parameterValues = new Array();
		
		// Project
		parameterValues.push(this._quote(this._staticCommon._SPECIAL_PROJECT_AREA));
		
		// Team
		var team = this.getPreference(this._static._PREFERENCE_ID_TEAM);
		team = this._normalizeParameterMultiValue(team);
		parameterValues.push(team);
		
		// Definition
		var definition = this.getPreference(this._static._PREFERENCE_ID_DEFINITION);		
		definition = this._normalizeParameterValue(definition);
		parameterValues.push(definition);

		if ((this._reportClass == this._static._VIEWLET_CLASS_CODE) ||
			(this._reportClass == this._static._VIEWLET_CLASS_TEST)) {
			// Component
			var component = this.getPreference(this._static._PREFERENCE_ID_COMPONENT);
			component = this._normalizeParameterMultiValue(component);
			parameterValues.push(component);
		}

		if ((this._reportClass == this._static._VIEWLET_CLASS_BUILD) ||
			(this._reportClass == this._static._VIEWLET_CLASS_CODE)) {
				
			// Duration
			var duration = this.getPreference(this._static._PREFERENCE_ID_DURATION);
			duration = this._normalizeParameterValue(duration);
			parameterValues.push(duration);
			
			// Incomplete
			var incomplete = this.getPreference(this._static._PREFERENCE_ID_INCOMPLETE);
			// Note: No need to normalize incomplete
			parameterValues.push(this._quote(incomplete));
		}
		
		// Zoom
		parameterValues.push(this._quote(this.isZoomed()));
	
		return parameterValues;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @param com.ibm.team.reports.rest:ReportParameterGroupDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetGroupParameters: function(self, groupName, dto) {
		// Create a map out of the DTO
		var map = self._processDTOGroupParameters(dto);
		// Store global
		self._groupParametersBuild = map;
		// Debug
		// console.debug(dto);
		// self._debugCascadingParameters(map, "", 0);
		
		if ((self._reportClass == self._static._VIEWLET_CLASS_BUILD) ||
			(self._reportClass == self._static._VIEWLET_CLASS_CODE)) {
			// Chain parameter call
			// Asynchronous call
			self._getSingleParameters(
				self._static._PARAMETER_SINGLE_WHICH,
				self._callBackSuccessGetSingleParameters,
				self._callBackErrorGetSingleParameters);
		} else {
			// Show the display area
			self._showDisplayArea();
		}
		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetSingleParameters: function(self, parameterName, dto) {
		// Create a custom object out of the DTO
		var result = self._processDTOSingleParameters(dto);
		// Store global
		self._singleParametersWhich = result;
		// Debug
		// console.debug(dto);
		// Show the display area
		self._showDisplayArea();
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @return void
 	 */
	_callBackErrorGetSingleParameters: function(self, parameterName) {
		// Set error
		self._singleParametersWhich = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},			

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @return void
 	 */
	_callBackErrorGetGroupParameters: function(self, groupName) {
		// Set error
		self._groupParametersBuild = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionTeam: function() {
		// Get parameters
		var projectAreas = this._groupParametersBuild;
		var teamArea = null;
		try {
			// Get team area
			teamArea = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA];
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(teamArea, "_normalizeTeamAreaPreferenceSingleValue");
		
		return result;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionDefinition: function(currentSettings) {
		// Get parameters
		var projectAreas = this._groupParametersBuild;
		var buildDefinitions;
		try {
			// Get the current preference value for the team area
			// Cascades: Team Area -> Build Definition
			var teamAreaValue = currentSettings.preferences.team;
			
			if (teamAreaValue === this._staticCommon._PREFERENCE_OPTION_VALUE_ALL) {
				buildDefinitions = this._getAllParameterChildren(
					projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA]);
			} else {
				buildDefinitions = new Array();
				var parsedTeamAreaNames = this._parsePreferenceMultiValue(teamAreaValue);
				// Get build definitions
				for (var index = 0; index < parsedTeamAreaNames.length; index++) {
					var value = parsedTeamAreaNames[index];
					var parameterObject = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA].map[value];
					buildDefinitions.push(parameterObject);
				}
			}
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(buildDefinitions[0], null);
		for (var index = 1; index < buildDefinitions.length; index++) {
			var nextResult = this._adaptParametersToPreferences(buildDefinitions[index], null);
			result = this._mergePreferences(result, nextResult);
		}
		
		// If there are no available options, add an option indicating so
		if (result.options.length === 0) {
			this._addNoValuesOption(result.options);		
		} else if (result.options.length > 1) {
			// Sort preferences
			result.options.sort(this._sortPreferencesArray);
		}
		
		return result;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object currentSettings
 	 * @return Object
 	 */		
	_getPreferenceOptionComponent: function(currentSettings) {
		// Get parameters
		var projectAreas = this._groupParametersBuild;
		var buildDefinition = null;
		var component = null;
		try {
			// Get the current preference value for the team area
			// Cascades: Team Area -> Build Definition
			var teamAreaValue = currentSettings.preferences.team;
			var parsedTeamAreaNames = this._parsePreferenceMultiValue(teamAreaValue);
			// Cascades: Build Definition -> Component
			var buildDefinitionName = currentSettings.preferences.definition;
			// Get build definitions
			for (var index = 0; index < parsedTeamAreaNames.length; index++) {
				var value = parsedTeamAreaNames[index];
				// Get build definition
				var buildDefinition = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA].map[value];
				// Get component
				// Iterate through component value sets until a defined
				// build definition is found (may be shared between multiply
				// selected team areas)
				// Many (Team Areas) to One (Build Definitions) to Many (Components)
				component = buildDefinition.map[buildDefinitionName];
				if (this._isObjectDefined(component, false, "component")) {
					break;
				}
			}
		} catch(errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(component, null);
		
		return result;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionDuration: function() {
		// Get parameters
		var duration = this._singleParametersWhich;
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(duration, null);
		
		return result;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String html
 	 * @return boolean
 	 * @overrides super
 	 */	
	_containsReport: function(html) {

		if (this._reportClass == this._static._VIEWLET_CLASS_TEST) {
			// Table results
			var matcher = new String(html);
			var matches = matcher.match(/<tr/g);

			if (this._isArray(matches, false, null) == false) {
				// No matches found
				return false;
			} else if (matches.length < 10) {
				// Expected 10 rows corresponding to 10 unit tests
				return false;
			}
			return true;
		} else {
			// Image results
			return this.inherited("_containsReport", arguments);
		}
	},	
	
	// Class Methods ---------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}

});


})();

