/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletTree");

/*
 * Imports 
 */

/*
 * Aliases 
 */

(function() {

var ViewletTree = com.ibm.team.dashboard.web.ui.internal.ViewletTree;

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletTree",
	dijit.Tree, {
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	
	
	/*
 	 * Private Constructor
 	 * 
 	 * @param Object[] args
 	 * @return <none>
 	 */
	_postCreateConstructor: function() {
		// Handle double-click events on tree nodes
		this.connect(this.domNode, "ondblclick", this._onDoubleClick);
		
		// Constants //------------------------------------------------------------>
		
		this._static._PRE_VIEWLET_ICON_SEGMENT = 
			"../../web/";
		
		this._static._DEFAULT_CATEGORY_ICON = 
			"../../web/com.ibm.team.dashboard.web/ui/internal/images/folder.gif";
		this._static._DEFAULT_VIEWLET_ICON = 
			"../../web/com.ibm.team.dashboard.web/ui/internal/images/tree-viewlet-icon.gif";	
		
		this._static._TYPE_CATEGORY = "category";
		this._static._TYPE_VIEWLET = "viewlet";
		
		// Instance Variables //--------------------------------------------------->
		
		this._selectionListenerCount = 0;
		this._selectionListenerList = new Array();
		
		this._doubleClickListenerCount = 0;
		this._doubleClickListenerList = new Array();
		
		// Map tracking first expansion of folders
		this._folderFirstExpandedMap = new Array();
		
		// Expand all top level nodes
		this._expandAllTopLevelNodes();
		
		// IMPORTANT NOTE: Perform operations on nodes from here on because the nodes
		// are lazily created and may not exist when overriding other tree methods		
		
		// Get the node map
		var nodeMap = this._itemNodeMap;
		// Dynamically update the tree icons
		this._updateIcons(nodeMap);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 * @overrides
 	 */
	_onClick: function(event) {
		// Handle the event as normal in the superclass
		com.ibm.team.dashboard.web.ui.internal.ViewletTree.superclass._onClick.call(this, event);
		// Get the target DOM element		
		var domElement = event.target;
		// Find the corresponding tree node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		// Ensure the node is defined
		if(!nodeWidget || 
			!nodeWidget.isTreeNode) {
			return;
		}
		// Check to see if the node was expanded
		if((domElement == nodeWidget.expandoNode) ||
			 (domElement == nodeWidget.expandoNodeText)) {
			this._handleEventNodeToggle(nodeWidget);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * @return <none>
 	 */
	_handleEventNodeToggle: function(node) {
		var label = node.label;
		// Check to see if this node has been expanded before
		if ((label in this._folderFirstExpandedMap) &&
			(this._folderFirstExpandedMap[label])) {
			// Nothing to do
			return;		
		}
		// Track which nodes have been expanded already
		this._folderFirstExpandedMap[label] = true;
		// Get the assoicate tree item
		var item = node.item;
		// Get the children
		var children = item.children;
		// Ensure we have an array
		if ((children instanceof Array) == false) {
			return;
		}
		// Get the node map
		var nodeMap = this._itemNodeMap;
		// Process children
		for (var i = 0; i < children.length; i++) {
			var child = children[i];
			// Get the child's name
			var name = new String(child.name);
			// Ensure name is defined and is in the node map
			if (name.length == 0) {
				continue;
			} else if ((name in nodeMap) == false) {
				continue;
			}
			// Get the corresponding node in the map
			var node = nodeMap[name];
			// Update the node's icon
			this._updateNodeIcon(node);
		}
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param Object args
 	 * 			Field: Object self
 	 * 				Method: _selectionChanged(event)
 	 * @return <none>
 	 */
	_addSelectionListener: function(args) {
		// Get the listener
		var listener = args.self;
		// Add the listener to the listener list
		this._selectionListenerList[this._selectionListenerCount] = listener;
		// Increment the listener list count
		this._selectionListenerCount++;
	},		
	
	/*
 	 * Method
 	 * 
 	 * @param Object args
 	 * 			Field: Object self
 	 * 				Method: _doubleClickOccurred(event)
 	 * @return <none>
 	 */
	_addDoubleClickListener: function(args) {
		// Get the listener
		var listener = args.self;
		// Add the listener to the listener list
		this._doubleClickListenerList[this._doubleClickListenerCount] = listener;
		// Increment the listener list count
		this._doubleClickListenerCount++;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param dojo.data item
 	 * @param TreeNode node
 	 * @return void
 	 * @overrides
 	 */	
	onClick: function(item, node) {
		// Notify all listeners
		for (var i = 0; i < this._selectionListenerList.length; i++) {
			this._selectionListenerList[i]._selectionChanged(item);
		}
	},	
			
	/*
 	 * Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 * @overrides
 	 */	
	onDoubleClick: function(event) {
		// Notify all listeners
		for (var i = 0; i < this._doubleClickListenerList.length; i++) {
			this._doubleClickListenerList[i]._doubleClickOccurred(event);
		}
	},			

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode[] nodeMap
 	 * @return <none>
 	 */		
	_updateIcons: function(nodeMap) {
		for (var name in nodeMap) {
			// Get the ndoe
			var node = nodeMap[name];
			// Process the node
			this._updateNodeIcon(node);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * @return <none>
 	 */		
	_updateNodeIcon: function(node) {
		// Process according to type
		if (node.item.type == this._static._TYPE_CATEGORY) {
			// Category node
	        this._updateCategoryIcon(node);
	    } else if (node.item.type == this._static._TYPE_VIEWLET) {
			// Viewlet node
			this._updateViewletIcon(node);
	    }
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element domNode
 	 * @param String attributeName
 	 * @param String attributeValue
 	 * @return Element
 	 */		
	_getChildDiv: function(domNode, attributeName, attributeValue) {
		var elementNodeType = 1;
		// Get the nodes children
		var children = domNode.childNodes;
		// Search children for child div
		for (var i = 0; i < children.length; i++) {
			// Get child
			var child = children[i];
			// Get it's name
			var name = child.nodeName;
			// Get it's type
			var type = child.nodeType;
			// Check for the matching div
			if (name != "DIV") {
				continue;
			} else if (type != elementNodeType) {
				continue;
			} else if (child.getAttribute(attributeName) != attributeValue) {
				continue;
			}
			// Match found
			return child;
		}
		// No match found
		return null;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * @return <none>
 	 */		
	_updateCategoryIcon: function(node) {
		// Set the icon
		this._updateIcon(node, this._static._DEFAULT_CATEGORY_ICON);
	},
		
	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * @param String image
 	 * @return <none>
 	 */		
	_updateIcon: function(node, image) {
		// Get the DOM node
		var domNode = node.domNode;
		// Get the child content div
		var contentDiv = this._getChildDiv(domNode, "dojoattachpoint", "contentNode");
		// Ensure we have a div
		if (contentDiv == null) {
			return;
		}
		// Get the child icon div
		var iconDiv = this._getChildDiv(contentDiv, "dojoattachpoint", "iconNode");
		// Ensure we have a div
		if (iconDiv == null) {
			return;
		}
		// Set the icon
		// A favicon - must add it as an img element so that we can
		// constrain its size to 16x16
		var imgElement;
		if (iconDiv.childNodes.length == 0) {
			imgElement = document.createElement("img");
			imgElement.style.width = "16px";
			imgElement.style.height = "16px";
			iconDiv.appendChild(imgElement);
		}
		else {
			imgElement = iconDiv.childNodes[0];
		}
		imgElement.src = image;
	},		
		
	/*
 	 * Private Method
 	 * 
 	 * @param dijit._TreeNode node
 	 * @return <none>
 	 */		
	_updateViewletIcon: function(node) {
		// Length of: resources/u
		var minLength = 11;
		// Get the icon	path
		var icon = node.item.icon;
		// Ensure the icon path is defined
		if ((icon instanceof Array) == false) {
			// Use default
			this._updateIcon(node, this._static._DEFAULT_VIEWLET_ICON);
			return;
		} else if (icon.length == 0) {
			// Use default
			this._updateIcon(node, this._static._DEFAULT_VIEWLET_ICON);
			return;
		} else if (typeof(icon[0]) != "string") {
			// Use default
			this._updateIcon(node, this._static._DEFAULT_VIEWLET_ICON);
			return;
		} else if (icon[0].length < minLength) {
			// Use default
			this._updateIcon(node, this._static._DEFAULT_VIEWLET_ICON);
			return;				
		}
		// Get the raw icon path
		var iconPath = new String(icon[0]);
		var iconSegment;
		if (iconPath.indexOf("://") == -1) {
			// Sample Input
			// com.ibm.team.dashboard.viewlets.web/ui/internal/images/icon-wiki.gif
			// Sample Normalization
			// "url('../../web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/icon-wiki.gif')
			// Remove 'resources'
			iconSegment = this._static._PRE_VIEWLET_ICON_SEGMENT + iconPath;
		}
		else {
			iconSegment = icon[0];
		}
		// Set the icon
		this._updateIcon(node, iconSegment);
	},		

	/*
 	 * Method
 	 * 
 	 * @param event - Event
 	 * @return <none>
 	 */		
	_onDoubleClick: function(event) {
		// Translate the current DOM node into a tree node
        var treeNode = dijit.getEnclosingWidget(event.target);
		// Ensure is defined
		if (treeNode == undefined) {
			return;
		} else if (treeNode == null) {
			return;
		}
		// Ensure has item property
		if (("item" in treeNode) == false) {
			return;
		}
		// Get the item currently selected in the tree
		var item = treeNode.item;		
		// Handle as an on double click event
		this.onDoubleClick(item);
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 * @overrides
 	 */	
	postCreate: function() {
		// Call super
		com.ibm.team.dashboard.web.ui.internal.ViewletTree.superclass.postCreate.call(this);
		// Call our private constructor
		this._postCreateConstructor();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dojo.data.Item item
 	 * @return String
 	 * @overrides super
 	 */	
	getLabelClass: function(item) {
		if (item == null) {
			// Error
			// This should not occur - perhaps the invisible root?
			return "";
		} else if (item.type[0] == this._static._TYPE_VIEWLET) {
			return "viewlet-entry";
		}
		return "category-entry";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param dojo.data.Item item
 	 * @return String
 	 * @overrides super
 	 */	
	getIconClass: function(item){
		return "tree-icon";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 * @overrides super
 	 */	
	_onTreeFocus: function(event){
		// Get the new focus node		
		var newFocusNode = dijit.getEnclosingWidget(event.target);
		// Get the old focus node
		var oldFocusNode = this.lastFocused;
		
		// Remove selection from old focus node
		if ((newFocusNode != oldFocusNode) &&
			(oldFocusNode != null)){
			dojo.removeClass(oldFocusNode.contentNode, "focus-entry");
		}
		// Add selection to new focus node
		if (newFocusNode != null) {
			dojo.addClass(newFocusNode.contentNode, "focus-entry");
			// Handle as an on click event	
			this.onClick(newFocusNode.item, newFocusNode);
		}
		
		this.inherited(arguments);
	},

	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_expandAllTopLevelNodes: function() {
		// Get the tree's top level nodes
		var treeNodeList = this.getChildren();
		// Ensure the list is defined
		if ((treeNodeList == null) ||
			(treeNodeList.length == 0)) {
			// Nothing to do
			return;
		}
		// Expand each top level node
		for (var i = 0; i < treeNodeList.length; i++) {
			// Get the tree node
			var treeNode = treeNodeList[i];
			// Ensure the node is defined and is a folder
			if (treeNode == null) {
				continue;
			} else if (treeNode.isFolder == false) {
				continue;
			}
			// Track which nodes have been expanded already
			this._folderFirstExpandedMap[treeNode.label] = true;
			// Expand the node
			this._expandNode(treeNode);
		}
	}	
	
});

})();
