/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.WorkItemQueriesViewlet");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooser");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");

dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.Constants");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.util");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {
	
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var WorkItemQueryChooser = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooser;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var Constants = com.ibm.team.workitem.web.client.internal.Constants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var IMG_PATH = "../../web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/";
var IMG_URL_CLOSED_OVERLAY = IMG_PATH + "wi_complete_ovr.gif";
var IMG_URL_IN_PROGRESS_OVERLAY = IMG_PATH + "wi_in_progress_ovr.gif";

var message;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.WorkItemQueriesViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/WorkItemQueriesViewlet.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");
		this.loading = messages.loading;
		this.noQueries = messages.noQueries;
		this.emptyQueries = messages.emptyQueries;
	},
	
	init: function() {
		this._loadItemIds();
		this._populateHelp(this._noQueriesEditContentsLink, messages.noQueriesHelp);
		this._populateHelp(this._emptyQueriesEditPrefslink, messages.emptyQueriesHelp);
	},
	
	_populateHelp: function(node, message) {
		var links = I18nUtil.addMessage(node, message);
		this.connect(links[0], "onclick", this._doEdit);
		if (links[1])
			this.connect(links[1], "onclick", this._doPreferences);
	},
	
	refresh: function() {
		this.getSite().setLoading(true);
		this._oldQueries = this._queries;
		this._queries = new Array();
		if (this._queryItemIds.length > 0 || this._isDefault || this._isTemplated) {
			var self = this;
			var serviceObject = {
				_successCallback: function(/*array*/queryDTOs){
					if (!self.isDestroyed()) {
						if (queryDTOs) {
							for (var i = 0; i < queryDTOs.length; i++) {
								self._queries.push({
									dto: queryDTOs[i]
								});
								if (self._isTemplated) 
									self._queryItemIds.push(queryDTOs[i].itemId);
							}
						}
						self._isTemplated = false;
						self.getSite().setLoading(false);
						self.update();
						self.getSite().doneRefresh();
					}
				},
				_defaultsSuccessCallback: function(/*array*/scopedQueriesDTO){
					if (!self.isDestroyed()) {
						self._isDefault = false;
						if (scopedQueriesDTO && scopedQueriesDTO.length > 0 && scopedQueriesDTO[0].queries && scopedQueriesDTO[0].queries.length > 0) {
							for (var i = 0; i < scopedQueriesDTO[0].queries.length && i < 15; i++) 
								self._queryItemIds.push(scopedQueriesDTO[0].queries[i].itemId);
							if (scopedQueriesDTO[0].queries.length > 15) 
								scopedQueriesDTO[0].queries.length = 15;
							this._successCallback(scopedQueriesDTO[0].queries);
						}
						else 
							this._successCallback([]);
					}
				},
				_errorCallback: function(error){
					dojo.debug(error.message);
					self.getSite().doneRefresh();
				}
			};
			// if the state is flagged as default, it has never been initialized
			// need to request the initial contents from the server and update the state
			if (this._isDefault) {
				// translate dashboard scope into work items scope
				var scope;
				switch (this.getScope()) {
					case DashboardConstants.SCOPE_CONTRIBUTOR:
						scope = QueryClient.SCOPE_PERSONAL;
						break;
					case DashboardConstants.SCOPE_TEAM_AREA:
						scope = QueryClient.SCOPE_TEAM_AREA;
						break;
					case DashboardConstants.SCOPE_PROJECT_AREA:
						scope = QueryClient.SCOPE_PROJECT_AREA;
						break;
				}
				var responseHandler = new ServiceResponseHandler(serviceObject, "_defaultsSuccessCallback", "_errorCallback");
				QueryClient.getScopedQueries(responseHandler, scope, this.getScopeItem().itemId, true, false);
			}
			else {
				var args = { includeCount: true };
				if (this._isTemplated)
					args.ids = this._queryStringIds;
				else
					args.itemIds = this._queryItemIds;
				var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
				QueryClient.getQueries2(responseHandler, args);
			}
		}
		else {
			this.getSite().setLoading(false);
			this.getSite().doneRefresh();
			this._updateVisibility(false);
		}
	},
	
	update: function() {
		// clear the queries list
		while (this._queriesList.firstChild)
			this._queriesList.removeChild(this._queriesList.firstChild);
		// if we just started editing, save the current queries
		if (this.isEditing() && !this._oldQueries && this._queries) {
			this._oldQueries = new Array();
			for (var i = 0; i < this._queries.length; i++) {
				this._oldQueries.push({dto: dojo.clone(this._queries[i].dto)});
			}
		}
		// if we just left edit mode, see if cancel was clicked
		else if (!this.isEditing() && this._oldQueries) {
			if (this._isValid()) {
				delete this._oldQueries;
			}
			else {
				this._queries = this._oldQueries;
				this._oldQueries = null;
			}
		}
		var showList = false;
		// if there are queries populate the list
		if (this._queries && this._queries.length > 0) {
			for(var i = 0; i < this._queries.length; i++) {
				// if we don't have a WorkItemsQueryLink for this query, create one
				if (!this._queries[i].linkWidget) {
					var widget = new com.ibm.team.dashboard.viewlets.web.ui.internal.WorkItemsQueryLink({
						queryDTO: this._queries[i].dto,
						viewlet: this
					});
					this._queries[i].linkWidget = widget;
				}
				this._queries[i].linkWidget.showDeleteButton(this.isEditing());
				// add the list item to the list
				if (this.isEditing() || this.getPreference("showEmpty") || this._queries[i].linkWidget.isLoading() || this._queries[i].dto.currentCount > 0) {
					this._queriesList.appendChild(this._queries[i].linkWidget.domNode);
					showList = true;
				}
			}
		}
		this._updateVisibility(showList);
		if (this.isEditing()) {
			this.disconnect(this._openAddQueriesEvent);
			this.disconnect(this._deleteAllQueriesEvent);
			this._site.getActionArea().innerHTML = "";
			
			var addDiv = document.createElement("div");
			addDiv.className = "action add";
			var addLink = document.createElement("a");
			addLink.innerHTML = messages.add;
			addLink.href = "";
			addDiv.appendChild(addLink);
			this._openAddQueriesEvent = this.connect(addLink, "onclick", this._openAddQueriesDialog);
			this._site.getActionArea().appendChild(addDiv);
			
			if (this._queries && this._queries.length > 0) {
				var deleteDiv = document.createElement("div");
				deleteDiv.className = "action delete";
				var deleteLink = document.createElement("a");
				deleteLink.innerHTML = messages.deleteAll;
				deleteLink.href = "";
				deleteDiv.appendChild(deleteLink);
				this._deleteAllQueriesEvent = this.connect(deleteLink, "onclick", this._deleteAllQueries);
				this._site.getActionArea().appendChild(deleteDiv);
			}
			dojo.addClass(this.domNode,"com-ibm-team-dashboard-viewlets-web-ui-internal-WorkItemQueriesViewlet-editing");
		}
		else {
			dojo.removeClass(this.domNode,"com-ibm-team-dashboard-viewlets-web-ui-internal-WorkItemQueriesViewlet-editing");
		}
		if (this._oldQueries) {
			for (var i = 0; i < this._oldQueries.length; i++) {
				if (this._oldQueries[i].linkWidget)
					this._oldQueries[i].linkWidget.destroy();
			}
			delete this._oldQueries;
		}
	},
	setDashboardEditing: function(editing) {
		this._noQueriesEditContentsLink.className = this.isEditing() || !editing ? "hidden" : "";
		this._emptyQueriesEditPrefslink.className = editing ? "" : "hidden";
	},
	
	_loadItemIds: function() {
		this._queryItemIds = this.getViewletState().get("queryItemIds");
		if (!this._queryItemIds) {
			this._queryItemIds = new Array();
			this._queryStringIds = this.getViewletState().get("queryStringIds");
			if (!this._queryStringIds) {
				this._isDefault = true;
			} else {
				this._isTemplated = true;
			}
		}
	},
	
	_updateVisibility: function(/* boolean */ queriesListVisible) {
		this._queriesList.className = queriesListVisible ? "" : "hidden";
		this._loadingDiv.className = "hidden";
		// if there are no visible queries and we are not editing, display a helpful message
		this._noQueriesDiv.className = !queriesListVisible && (!this._queries || this._queries.length == 0) ? "message" : "hidden";
		this._noQueriesEditContentsLink.className = this.isEditing() ? "hidden" : "";
		this._queriesFilteredDiv.className = !queriesListVisible && !this.isEditing() && this._queries && this._queries.length > 0 ? "message" : "hidden";
	},
	
	_openAddQueriesDialog: function(event) {
		dojo.stopEvent(event);
		
		new WorkItemQueryChooser({onOk: dojo.hitch(this,this._addQueries), scope: this.getScope(), scopeItemId: this.getScopeItem().itemId});
	},
	
	_addQueries: function(queryDTOs) {
		var newQueries = new Array();
		var queriesIndexMap = new Array();
		for (var i = 0; queryDTOs && i < queryDTOs.length; i++) {
			var isValid = true;
			for (var j = 0; j < this._queryItemIds.length; j++) {
				if (this._queryItemIds[j] == queryDTOs[i].itemId) {
					isValid = false;
					break;
				}
			}
			if (isValid) {
				if (!this._queries)
					this._queries = new Array();
				var index = this._queries.push({ dto: queryDTOs[i] });
				this._queryItemIds.push(queryDTOs[i].itemId);
				// information used to asynchronously load the count estimates for the new queries
				newQueries.push(queryDTOs[i].itemId);
				queriesIndexMap[queryDTOs[i].itemId] = index-1;
			}
		}
		if (newQueries.length > 0) {
			// update the UI
			this.update();
			// load the counts of the new queries
			this.getSite().setLoading(true);
			var self = this;
			var serviceObject = {
				_successCallback: function(/*array*/queryDTOs){
					for (var i = 0; i < queryDTOs.length; i++) {
						var queriesObj = self._queries[queriesIndexMap[queryDTOs[i].itemId]];
						if (queriesObj.linkWidget) 
							queriesObj.linkWidget.setCurrentCount(queryDTOs[i].currentCount);
						queriesObj.dto = queryDTOs[i];
					}
					self.getSite().setLoading(false);
				},
				_errorCallback: function(error){
					self.getSite().setLoading(false);
					dojo.debug(error.message);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
			QueryClient.getQueries(responseHandler, newQueries, true);
			// save state
			this.getViewletState().set("queryItemIds", this._queryItemIds);
		}
	},
	
	_deleteQuery: function(/* string */ itemId) {
		for (var i = 0; i < this._queryItemIds.length; i++) {
			if (itemId == this._queryItemIds[i]) {
				this._queryItemIds.splice(i, 1);
				break;
			}
		}
		for (var i = 0; i < this._queries.length; i++) {
			if (itemId == this._queries[i].dto.itemId) {
				this._queries[i].linkWidget.domNode.parentNode.removeChild(this._queries[i].linkWidget.domNode);
				this._queries.splice(i,1);
				break;
			}
		}
		if (this._queries.length == 0)
			this._updateVisibility(false); 
		// save state
		this.getViewletState().set("queryItemIds", this._queryItemIds);
	},
	
	_deleteAllQueries: function(event) {
		dojo.stopEvent(event);
		
		delete this._queries;
		this._queryItemIds = new Array();
		this.update();
		// save state
		this.getViewletState().set("queryItemIds", this._queryItemIds);
	},
	
	/* boolean */ _isValid: function() {
		if (!this._queries || this._queries.length == 0)
			return this._queryItemIds.length == 0;
		if (this._queryItemIds.length != this._queries.length)
			return false;
		for (var i = 0; i < this._queryItemIds.length; i++) {
			if (this._queryItemIds[i] != this._queries[i].dto.itemId)
				return false;
		}
		return true;
	},
	
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		this.update();
	},
	
	_doEdit: function(event) {
		dojo.stopEvent(event);
		
		this.getSite().editContents();
	},
	
	_doPreferences: function(event) {
		dojo.stopEvent(event);
		
		this.getSite().toggleSettings();
	},
	
	destroy: function() {
		if (this._queries) {
			for (var i = 0; i < this._queries.length; i++) {
				if (this._queries[i].linkWidget) 
					this._queries[i].linkWidget.destroy();
			}
		}
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.WorkItemsQueryLink",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<li><a dojoAttachPoint='_link'><div dojoAttachPoint='_deleteButton' class='hiddenButton'></div></a></li>",
	
	constructor: function(args) {
		this._queryDTO = args.queryDTO;
		this._onDelete = dojo.hitch(args.viewlet,args.viewlet._deleteQuery)
		this._viewlet = args.viewlet;
	},
	postCreate: function() {
		this._setLinkText();
		this._link.href = WorkItemUtil.getRunSavedQueryURI(this._queryDTO.itemId);
		this.connect(this._deleteButton, "onclick", this._handleDelete);
		var tip = document.createElement("span");
		this._tooltip = new HoverPopup({connectNode: [this._link], callback: dojo.hitch(this,this._populateTooltip), callbackArg: {tipSpan: tip, queryDTO: this._queryDTO}, closeCallback: dojo.hitch(this,this._onTooltipClose), repopulateDelay: 30000, preDisplayCheck: dojo.hitch(this,this.prePopupCheck)}, tip);
	},
	setCurrentCount: function(count) {
		this._queryDTO.currentCount = count;
		this._setLinkText();
	},
	
	isLoading: function() {
		return this._loading ? true : false;
	},
	
	prePopupCheck: function() {
		if (!this._viewlet.isEditing()) {
			// suspend auto-refresh while hovering
			this._viewlet.getSite().setAutoRefresh(false);
			return true;
		}
		return false;
	},
	
	_onTooltipClose: function() {
		// ok safe to refresh again
		this._viewlet.getSite().setAutoRefresh(true);
	},
	
	_setLinkText: function() {
		var count;
		if ("currentCount" in this._queryDTO) {
			count = this._queryDTO.currentCount;
			delete this._loading;
		}
		else {
			count = messages.loadingL;
			this._loading = true;
		}
		var linkText = this._queryDTO.name + " (" + count + ")";
		if (this._link.firstChild != this._deleteButton)
			this._link.removeChild(this._link.firstChild);
		this._link.insertBefore(document.createTextNode(linkText),this._deleteButton);
	},
	
	showDeleteButton: function(/* boolean */ visible) {
		if (visible)
			this._deleteButton.className = "deleteButton";
		else
			this._deleteButton.className = "hiddenButton";
	},
	
	_handleDelete: function(event) {
		event.preventDefault();
		event.cancelBubble = true;
		this._onDelete(this._queryDTO.itemId);
	},
	
	_populateTooltip: function(args) {
		args.tipSpan.innerHTML = "<div class='com-ibm-team-dashboard-viewlets-web-ui-internal-WorkItemQueriesViewlet-Tooltip'>" + messages.runningQuery + "</div>";
		var serviceObject = {
			success: function(resultSet) {
				// populate the tooltip with innerHTML since this is how the tooltip will render it anyway
				var buf = ["<div class='com-ibm-team-dashboard-viewlets-web-ui-internal-WorkItemQueriesViewlet-Tooltip'>"];
				if(resultSet && resultSet.totalCount > 0) {
					// create lookup map
					var indexes = {};
					for (var i=0;i<resultSet.headers.length;i++) {
						indexes[resultSet.headers[i].attributeId] = i;
					}

					// Empty table rows are used instead of padding so that dojo
					// is able to calculate their height to position the tooltip
					buf.push("<div class='heading'>" + HTMLUtility.escapeHTML(args.queryDTO.name) + "</div>");
					buf.push("<table border='0' cellspacing='0' cellpadding='0'>");
					buf.push("<tr><td class='empty-row'></td></tr>");
					var rows = resultSet.rows;
					for(var i = 0; i < rows.length; i++) {
						var resultDTO = rows[i];
						var href = WorkItemUtil.getViewWorkItemURI(resultDTO.id);
						buf.push("<tr>");
						buf.push("<td style='vertical-align: top;'>");
						buf.push("<div class='iconContainer'>");
						
						// add overlay if available/needed
						var isClosed = false;
						if (resultDTO.stateGroup) {
							var src = null;
							switch (resultDTO.stateGroup) {
								case Constants.CLOSED_STATES:	
									src = IMG_URL_CLOSED_OVERLAY;
									isClosed = true;
									break;
								case Constants.IN_PROGRESS_STATES:	
									src = IMG_URL_IN_PROGRESS_OVERLAY;
									break;
							}
							if (src) {
								buf.push("<img class='overlayIcon' src="+src+">");
							}
						}
						buf.push("<img " + (isClosed ? "class='closed' " : "") + "src="+resultDTO.iconUrls[indexes[WorkItemAttributes.TYPE]]+">");
						buf.push("</div>");
						buf.push("</td>");
						buf.push("<td class='spaced'><a href=" + href + ">" + resultDTO.id + ": " + HTMLUtility.escapeHTML(resultDTO.labels[indexes[WorkItemAttributes.SUMMARY]]) + "</a></td>");
						buf.push("</tr>");
					}
					var endMarkup = [];
					if (resultSet.totalCount > 10) {
						buf.push("<tr><td class='empty-row'></td></tr>");
						endMarkup.push("<div>");
						endMarkup.push(dojo.string.substitute(messages.resultCount, [resultSet.totalCount]));
						endMarkup.push("</div>");
						endMarkup.push("<div class='bottom-right'><a href=" + WorkItemUtil.getRunSavedQueryURI(args.queryDTO.itemId) + ">");
						endMarkup.push(messages.showAll);
						endMarkup.push("</a></div>");
					}
					buf.push("</table>" + endMarkup.join(""));
				}
				else {
					buf.push(messages.noResults2);
				}
				buf.push("</div>");	
				args.tipSpan.innerHTML = buf.join("");
				args.tooltip.refresh();
			},
			failure: function(error) {
				console.error(error.message);
			}
		}
		var responseHandler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var queryArgs = {
			itemId: args.queryDTO.itemId,
			startIndex: 0,
			maxResults: 10,
			projectAreaItemId: getCurrentProjectArea().itemId,
			columnIdentifiers: [WorkItemAttributes.TYPE, WorkItemAttributes.SUMMARY, WorkItemAttributes.STATE]
		};
		QueryClient.getResultSet({}, queryArgs, responseHandler);
	}, 
	destroy: function () {
		if(this._tooltip) this._tooltip.destroy();
		this.inherited(arguments);
	}
});

})();
