/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.Members"); //$NON-NLS-1$
 
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var ServiceRequest   = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var ACTION_PREFIX = "com.ibm.team.repository.editUser&itemId="; //$NON-NLS-1$
var MANAGE_ACTIVE_LINK = "#action=com.ibm.team.process.manageProjectAreas"; //$NON-NLS-1$

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.Members", com.ibm.team.repository.web.ui.internal.admin.ItemList, { //$NON-NLS-1$
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	
	postCreate: function() {
		var notModifier = ""; //$NON-NLS-1$
		if (this.notInProject === "true") { //$NON-NLS-1$
			notModifier = this.messages.Members_NotModifier;
		}
		
		setText(this.heading, dojo.string.substitute(this.messages.Members_Heading, [ notModifier, this.projectAreaName ]));
		setText(this.description, dojo.string.substitute(this.messages.Members_Description, [ notModifier ]));
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.tableControls.style.paddingTop = "0.5em"; //$NON-NLS-1$
		
		this.setHeaderRow([this.messages.Members_UserTableNameHeader, this.messages.Members_UserTableUserIdHeader, this.messages.Members_UserTableEmailHeader]);
		dojo.removeClass(this.headerRow, "sortable"); //$NON-NLS-1$
		
		this.connect(this.headerRow, "onmouseover", function (e) { //$NON-NLS-1$
			dojo.toggleClass(e.target, "selected", true); //$NON-NLS-1$
		});
		this.connect(this.headerRow, "onmouseout", function (e) { //$NON-NLS-1$
			dojo.toggleClass(e.target, "selected", false); //$NON-NLS-1$
		});
					
		this.showBreadcrumbs();
			
		this.refresh();
		
	},
	
	showBreadcrumbs: function () {
		var parent = this.heading.parentNode.parentNode;
		this.breadcrumbs = document.createElement("div");  //$NON-NLS-1$
		dojo.toggleClass(this.breadcrumbs, "breadcrumbs"); //$NON-NLS-1$
		var link = document.createElement("a"); //$NON-NLS-1$
		link.href = MANAGE_ACTIVE_LINK;
		link.appendChild(document.createTextNode(this.messages.Members_ActiveProjectAreas));
		this.breadcrumbs.appendChild(link);
		var span = document.createElement("span"); //$NON-NLS-1$
		span.appendChild(document.createTextNode(">")); //$NON-NLS-1$
		this.breadcrumbs.appendChild(span);
		parent.insertBefore(this.breadcrumbs, this.heading.parentNode);
	},
	
	showMembers: function (resultSet) {
		dojox.data.dom.removeChildren(this.tbody);
		dojo.toggleClass(this.tbody, "userTable", true); //$NON-NLS-1$
		var loggedInUser = getAuthenticatedContributor();
		if (resultSet == null || resultSet.length == 0) {
			var tr = document.createElement("tr"); //$NON-NLS-1$
			var td = document.createElement("td"); //$NON-NLS-1$
			td.appendChild(document.createTextNode(this.messages.Members_NoUsersFound));
			td.colSpan = 3;
			tr.appendChild(td);
			tr.onclick = function(e){
				dojo.stopEvent(e);
			}
			this.tbody.appendChild(tr);
		}
		else {
			dojo.forEach(resultSet, dojo.hitch(this, function(user){
				var row = this.addRow(ACTION_PREFIX + user.itemId, [user.name, user.userId, user.emailAddress]);
				if (user.userId !== loggedInUser.userId) {
					var archiveIcon = document.createElement("img"); //$NON-NLS-1$
					archiveIcon.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive.gif"); //$NON-NLS-1$ //$NON-NLS-2$
					archiveIcon.title = this.messages.Members_ArchiveUser;
					archiveIcon.className = "archiveIcon"; //$NON-NLS-1$
					archiveIcon.userItemId = user.itemId;
					this.connect(archiveIcon, "onmouseover", function(e){ //$NON-NLS-1$
						e.target.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive_hov.gif"); //$NON-NLS-1$ //$NON-NLS-2$
					});
					this.connect(archiveIcon, "onmouseout", function(e){ //$NON-NLS-1$
						e.target.src = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive.gif"); //$NON-NLS-1$ //$NON-NLS-2$
					});
					this.connect(archiveIcon, "onclick", function(e){ //$NON-NLS-1$
						dojo.stopEvent(e);
						if (confirm(this.messages.Members_PromptArchiveUser)) {
							this.archiveSingleUser(e.target.userItemId);
						}
					});
					row.cells[0].insertBefore(archiveIcon, row.cells[0].firstChild);
				}
			}));
			if (resultSet.length > 1) {
				dojox.data.dom.removeChildren(this.tableControls);
				this.archiveAllButton = document.createElement("button");  //$NON-NLS-1$
				this.archiveAllButton.appendChild(document.createTextNode(dojo.string.substitute(this.messages.Members_ArchiveAllUsers, [resultSet.length])));
				this.tableControls.appendChild(this.archiveAllButton);
				this.connect(this.archiveAllButton, "onclick", function(e){ //$NON-NLS-1$
					dojo.stopEvent(e);
					this.archiveAllErrorMessages = [];
					if (confirm(this.messages.Members_PromptArchiveAllUsers)) {
						this.archiveAllButton.disabled = true;
						dojo.forEach(resultSet, function(user) {
							var args = {
								"itemId": user.itemId, //$NON-NLS-1$
								"archived": true  //$NON-NLS-1$
							}
							var srh = new ServiceResponseHandler(this, "archiveAllUserCallback", "archiveAllUserErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
							var serviceRequest = new ServiceRequest(AdminClient.SERVICE_URI, "postArchiveUser", args); //$NON-NLS-1$
							// Use secret 3rd param to make it synchronous..since there is no multi-user archive method
							TeamServerClient.invokeService(serviceRequest, srh, true);
						}, this);
						this.refresh();
						if (this.archiveAllErrorMessages.length > 0) {
							this.displayErrorMessage(dojo.string.substitute(this.messages.Members_ErrorArchivingUser, [this.archiveAllErrorMessages]));
						}
					}
				}, this);
			}
		}
	},
	
	handleFetchMembersError: function(error) {
		this.displayErrorMessage(dojo.string.substitute(this.messages.Members_ErrorFetchingUsers, [error.message]));
	},
	
	refresh: function () {
		this.clearState();
		var tr = document.createElement("tr"); //$NON-NLS-1$
		var td = document.createElement("td"); //$NON-NLS-1$
		td.colSpan = 3;
		td.appendChild(document.createTextNode(this.messages.Members_SearchingForUsers));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		
		var srh = new ServiceResponseHandler(this, "showMembers", "handleFetchMembersError"); //$NON-NLS-1$ //$NON-NLS-2$
		ProcessClient.getMembers({projectAreaItemId: this.projectAreaItemId, notInProject: this.notInProject}, srh);

	},
	
	clearState: function () {
		dojox.data.dom.removeChildren(this.tableControls);
		dojox.data.dom.removeChildren(this.tbody);
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = ""; //$NON-NLS-1$
	},
	
	archiveSingleUser: function (itemId) {
		var args = {
			"itemId": itemId, //$NON-NLS-1$
			"archived": true  //$NON-NLS-1$
		}
		var srh = new ServiceResponseHandler(this, "archiveSingleUserCallback", "archiveSingleUserErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
		AdminClient.postArchiveUser(args, srh);
	},
	
	archiveSingleUserCallback: function (retVal) {
		var data = dojo.fromJson(retVal);
		if (data.errMessage) {
			this.displayErrorMessage(dojo.string.substitute(this.messages.Members_ErrorArchivingUser, [data.errMessage]));
		}
		else {
			this.refresh();
		}
	},
	
	archiveSingleUserErrorCallback: function (err) {
		this.displayErrorMessage(dojo.string.substitute(this.messages.Members_ErrorArchivingUser, [err.message]));
	},
	
	archiveAllUserCallback: function (retVal) {
		var data = dojo.fromJson(retVal);
		if (data.errMessage !== undefined) {
			this.archiveAllErrorMessages.push(data.errMessage);
		}
	},
	
	archiveAllUserErrorCallback: function (err) {
		this.archiveAllErrorMessages.push(err.message);
	}
	
	
	
});

})();