/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.common.WorkLoadLabelProvider"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var PlanningUtil		= com.ibm.team.apt.web.client.util;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.common.WorkLoadLabelProvider", null, { //$NON-NLS-1$

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_workLoadInfo: null,

	constructor: function(workLoadInfo) {
		this._workLoadInfo= workLoadInfo;
	},


	// ---- api ------------------------------------------------------------------------------------------------------------	

	getHoursTextNode: function() {
		var result= document.createElement("span"); //$NON-NLS-1$
		result.appendChild(this._textNode(MSGS.LOAD_HOUR_NODE_PREFIX, "type")); //$NON-NLS-1$
		result.appendChild(this._labelNode(this.getWorkHoursLeftLabel()));
		result.appendChild(this._textNode("/", "separator")); //$NON-NLS-1$ //$NON-NLS-2$
		result.appendChild(this._labelNode(this.getRealTimeLeftLabel()));
		result.appendChild(this._textNode("|", "separator")); //$NON-NLS-1$ //$NON-NLS-2$
		result.appendChild(this._labelNode(this.getDeltaLabel()));
		result.appendChild(this._textNode(MSGS.LOAD_HOUR_NODE_UNIT, "separator")); //$NON-NLS-1$

		return result;
	},

	getEstimateTextNode: function() {
		var result= document.createElement("span"); //$NON-NLS-1$
		result.appendChild(this._textNode(MSGS.LOAD_QOP_NODE_PREFIX, "type")); //$NON-NLS-1$
		result.appendChild(this._labelNode(this.getQualityOfPlanningLabel()));

		return result;
	},

	getDeltaLabel: function() {
		var result= this._createDefaultLabel("-", MSGS.LOAD_DELTA_LABEL); //$NON-NLS-1$

		var timeDelta= this._workLoadInfo.getRealTimeLeft() - this._workLoadInfo.getWorkHoursLeft();
		
		result.value= timeDelta + ""; //$NON-NLS-1$
		if (timeDelta > 0) 
			result.value= "+" + result.value; //$NON-NLS-1$
		result.className= this._getTimeDeltaClass();
		return result;
	},

	getWorkHoursLeftLabel: function(noValue) {
		var result= this._createDefaultLabel(noValue, MSGS.LOAD_WORKHOURSLEFT_LABEL);

		result.value= this._workLoadInfo.getWorkHoursLeft() + ""; //$NON-NLS-1$
		result.className= null;
		return result;
	},

	getRealTimeLeftLabel: function(noValue) {
		var result= this._createDefaultLabel(noValue, MSGS.LOAD_REALHOURSDONE_LABEL);

		result.value= this._workLoadInfo.getRealTimeLeft() + ""; //$NON-NLS-1$
		result.className= null;
		return result;
	},

	getQualityOfPlanningLabel: function(noValue) {
		var result= this._createDefaultLabel(noValue, MSGS.LOAD_QOP_LABEL);

		if (this._workLoadInfo.workItemCount != 0) {
			result.value= Math.round(this._workLoadInfo.getQualityOfPlanning()) + "%"; //$NON-NLS-1$
			result.className= null;
		}
		
		return result;
	},

	getTooltipContent: function() {
		var lines= [];
		lines.push(MSGS.LOAD_TOOLTIP_TITLE.bold());
		lines.push(bind(MSGS.LOAD_TOOLTIP_ASSIGNMENT, [this._workLoadInfo.getAssignment()]));

		if (this._workLoadInfo.isUsingDefaultAssignment()) {
			lines.push(MSGS.LOAD_TOOLTIP_ASSIGNMENT_DEFAULT);
		}

		lines.push(bind(MSGS.LOAD_TOOLTIP_AVAILABLE, [this._workLoadInfo.getRealTimeLeft()]));
		lines.push(bind(MSGS.LOAD_TOOLTIP_ESTIMATED, [this._workLoadInfo.getWorkHoursLeft()]));

		if (this._getTimeDeltaClass() == "overbooked") { //$NON-NLS-1$
			lines.push(bind(MSGS.LOAD_TOOLTIP_OVERBOOKED, [Math.abs(this._workLoadInfo.getRealTimeLeft() - this._workLoadInfo.getWorkHoursLeft())]));
		}

		lines.push(""); //$NON-NLS-1$
		lines.push(bind(MSGS.LOAD_TOOLTIP_QOP, [Math.round(this._workLoadInfo.getQualityOfPlanning())]));

		if (this._workLoadInfo.getAbsences().length > 0) {
			lines.push(""); //$NON-NLS-1$
			lines.push(MSGS.LOAD_TOOLTIP_ABSENCES);
			dojo.forEach(this._workLoadInfo.getAbsences(), function(absence) {
				lines.push(bind(MSGS.LOAD_TOOLTIP_ABSENCE_DETAIL, [absence.summary, PlanningUtil.formatDate(absence.startDate), PlanningUtil.formatDate(absence.endDate)]));
			});
		}

		return lines.join("<br />"); //$NON-NLS-1$
	},

	getMessage: function() {
		if (this._workLoadInfo.getRealTimeLeft() == 0) {
			return MSGS.LOAD_BAR_NOWORKTIMELEFT;
		} if (!this._hasWork()) {
			return MSGS.LOAD_BAR_NOWORK;
		} else if (!this._hasEstimatedWork()) {
			return MSGS.LOAD_BAR_NOTESTIMATED;
		}

		return null;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_hasWork: function() {
		return this._workLoadInfo.getCount() > 0;
	},

	_hasEstimatedWork: function() {
		return this._hasWork() && this._workLoadInfo.getEstimatedCount() > 0;
	},

	_createDefaultLabel: function(noValue, tooltip) {
		return {
			value: noValue || MSGS.LOAD_NOVALUE,
			className: "na", //$NON-NLS-1$
			tooltip: tooltip
		};
	},

	_getTimeDeltaClass: function() {
		var workHoursLeft= this._workLoadInfo.getWorkHoursLeft();
		var realTimeLeft= this._workLoadInfo.getRealTimeLeft();

		if (realTimeLeft > workHoursLeft) {
			return "underbooked"; //$NON-NLS-1$
		} else if (realTimeLeft < workHoursLeft) {
			return "overbooked"; //$NON-NLS-1$
		} 
		return "ontime"; //$NON-NLS-1$
	},

	_labelNode: function(label) {
		var result= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(result, "value"); //$NON-NLS-1$
		dojo.addClass(result, label.className || ""); //$NON-NLS-1$
		result.innerHTML= label.value; 
		result.setAttribute("title", label.tooltip || ""); //$NON-NLS-1$ //$NON-NLS-2$ 
		return result;
	},

	_textNode: function(text, className) {
		var result= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(result, className || ""); //$NON-NLS-1$
		result.appendChild(document.createTextNode(text));
		return result;
	},

	__sentinel: null // terminates this class definition
});

})();
