/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ListBox");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ListBoxMessages");

(function() {

/*
 * Aliases 
 */
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;

/*
 * TODOs 
 */
// Feature: Listener framework
// Support replace operation if ID is already defined

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ListBox",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Properties
	 */	
	width: "250px",
	height: "250px",
	margin: "5px",
	
	/*
	 * Specify the entry field to use when sorting list box entries
	 * Options: "label", "value", "id", "type"
	 */		
	sortField: "label",	
	
	/* 
 	 * Specify the callback function to use when an entry is double-clicked
 	 * 
 	 * Function (entry)
 	 * 
 	 * @param Object entry
 	 *				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type	 
 	 */
	callbackDoubleClick: null,

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/utils/ListBox.html"),
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ListBox
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ListBoxMessages");
		this.msgLoading = this.messages.msgLoading;
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		// Static Variables //------------------------------------------------->
		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		this._domMap = new Array();
		this._dataMap = new Array();
		this._iconMap = new Array();
		this._selected = new Array();

		this._ctrlPressed = false;
		this._shiftPressed = false;

		this._entryCount = 0;

		this._loadingFlag = false;
		
		this._lastSelectedEntryID = null;

		// Initialization //--------------------------------------------------->
		this._keyDownHandler = this.connect(dojo.doc, "onkeydown", this._handleEventKeyPressed);
		this._keyUpHandler = this.connect(dojo.doc, "onkeyup", this._handleEventKeyReleased);
		this._selectHandler = this.connect(dojo.doc, "onselectstart", this._handleEventSelectStart);

		this.domNode.style.height = this.height;
		this.domNode.style.width = this.width;
		this.domNode.style.margin = this.margin;		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ListBox";
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @return void
 	 */	
	addEntry: function(entry) {
		// Validate list		
		if (this._isObjectDefined(this.domNode) === false) {
			return;
		}

		var id = entry.id;
		// Check if the ID is already in use
		if (this._isObjectDefined(this._dataMap[id])) {
			// Ignore
			return;
		}

		// Create DOM representation
		var entryNode = document.createElement("div");
		entryNode.setAttribute("id", id);
		dojo.addClass(entryNode, "entry");
		this.connect(entryNode, "onmousedown", this._handleEventSelectEntry);		
		this._addIcon(entryNode, this._iconMap[entry.type]);
		this._addLabel(entryNode, entry.label);

		// Add double-click listener
		if (this.callbackDoubleClick !== null) {
			this.connect(entryNode, "ondblclick", this._handleEventDoubleClickEntry);
		}

		// Add to data map
		this._dataMap[id] = entry;
		// Add to DOM map		
		this._domMap[id] = entryNode;
		// Add to DOM itself
		this._addNode(this._contentArea, entryNode, entry);
		// Add to selection map
		this._selected[id] = false;
		// Update entry count
		this._entryCount++;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String iconPath
 	 * @return void
 	 */	
	_addIcon: function(parent, iconPath) {
		
		if (this._isStringDefined(iconPath) === false) {
			return;	
		}
		
		var icon = document.createElement("span");
		dojo.addClass(icon, "entry-icon");
		var image = document.createElement("img");
		dojo.addClass(image, "entry-image");
		image.setAttribute("src", iconPath);
		image.setAttribute("alt", this.messages.msgIcon);
		
		icon.appendChild(image);	
		parent.appendChild(icon);	
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param String labelText
 	 * @return void
 	 */	
	_addLabel: function(parent, labelText) {
		var label = document.createElement("span");
		dojo.addClass(label, "entry-label");
		label.innerHTML = HTMLUtility.sanitizeHTML(labelText);
		
		parent.appendChild(label);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element parent
 	 * @param Element targetChild
 	 * @param Object targetEntry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @return void
 	 */	
	_addNode: function(parent, targetChild, targetEntry) {

		var children = parent.childNodes;
		
		if (children.length === 0) {
			parent.appendChild(targetChild);
			return;
		}
		
		// Binary insert
		var low = 0;
		var high = children.length;
		while (low < high) {
			mid = Math.floor((low + high) / 2);
			
			var currentChild = children[mid];
			var currentEntry = this._dataMap[currentChild.id];
			
			if (currentEntry[this.sortField] < targetEntry[this.sortField]) {
				low = mid + 1; 
			} else {
				high = mid; 
			}
		}
		if (this._isObjectDefined(children[low])) {
			parent.insertBefore(targetChild, children[low]);
		} else {
			parent.appendChild(targetChild);
		}

	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventSelectEntry: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		// Get target
		if (this._hasProperty(event, "currentTarget") === false) {
			return false;
		}
		var target = event.currentTarget;
		if (this._isObjectDefined(target) === false) {
			return false;
		}
		// Get ID
		var id = target.id;
		// Get entry
		var entry = this._dataMap[id];
		// Determine selection
		var doSelect = true;
		if (this.isSelected(entry)) {
			doSelect = false;
		}
		
		// Clear previous selection unless CTRL is being pressed
		if (this._ctrlPressed === false) {
			this.removeAllSelections();
		}
		
		// Do selection
		this.setSelected(entry, doSelect);

		if (this._shiftPressed) {
			// Perform multi-selection
			this._doMultiSelectShiftKey(id);
		} else {
			// Remember this entry as the last selection
			this._lastSelectedEntryID = id;
		}
		
		return true;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param String targetSelectedEntryID
 	 * 
 	 * @return void
 	 */	
	_doMultiSelectShiftKey: function(targetSelectedEntryID) {
		// Get current entry DOM nodes
		var children = this._contentArea.childNodes;

		if (children.length <= 1) {
			// Not enough entries to do multi-select
			return;
		}
		// Entries >= 2
		
		if (this.getEntry(this._lastSelectedEntryID) === null) {
			// No last selected entry
			// Use the first entry as the last selected entry
			this._lastSelectedEntryID = children[0].id;
		}

		if (this._lastSelectedEntryID === targetSelectedEntryID) {
			// No range to select, the start and end entries are the same
			// Select the single entry
			this.setSelected(this.getEntry(targetSelectedEntryID), true);
			return;
		}

		var foundLastSelectedEntry = false;
		var foundTargetSelectedEntry = false;
		var doSelect = false;
		
		// Find the start and end entries and select the intermediate range
		for (var index = 0; index < children.length; index++) {
			// Get the ID of the current entry
			var currentEntryID = children[index].id; 
			
			if (currentEntryID === this._lastSelectedEntryID) {
				// Found the start or end entry of the selection range
				foundLastSelectedEntry = true;
				// Select it
				this.setSelected(this.getEntry(currentEntryID), true);
				// Set flag to select all intermediate entries encountered
				// after this
				doSelect = true;
				// Exit if this is the end entry of the selection range
				if (foundTargetSelectedEntry) {
					break;
				}
			} else if (currentEntryID === targetSelectedEntryID) {
				// Found the start or end entry of the selection range
				foundTargetSelectedEntry = true;
				// Select it
				this.setSelected(this.getEntry(currentEntryID), true);
				// Set flag to select all intermediate entries encountered
				// after this
				doSelect = true;
				// Exit if this is the end entry of the selection range
				if (foundLastSelectedEntry) {
					break;
				}
			} else if (doSelect) {
				// Start entry of the selection range was found 
				// Select intermediate entry
				this.setSelected(this.getEntry(currentEntryID), true);
			}
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventDoubleClickEntry: function(event) {
		dojo.stopEvent(event);

		// Get target
		if (this._hasProperty(event, "currentTarget") === false) {
			return false;
		}
		var target = event.currentTarget;
		if (this._isObjectDefined(target) === false) {
			return false;
		}

		// Get ID
		var id = target.id;

		// Get entry
		var entry = this._dataMap[id];
		
		// Execute callback
		this.callbackDoubleClick(entry);

		return true;
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventKeyPressed: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		
		if (event.keyCode == dojo.keys.CTRL) {
			this._ctrlPressed = true;
		} else if (event.keyCode == dojo.keys.SHIFT) {
			this._shiftPressed = true;
		}
		
		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventKeyReleased: function(event) {
		// Cancel event if default action is performed
		event.preventDefault();
		
		if (event.keyCode == dojo.keys.CTRL) {
			this._ctrlPressed = false;
		} else if (event.keyCode == dojo.keys.SHIFT) {
			this._shiftPressed = false;
		}
		
		return true;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventSelectStart: function(event) {
		dojo.stopEvent(event);
		// Prevent text selection (IE: on multi-select)		
		return false;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	addEntries: function(entries) {
		for (var index = 0; index < entries.length; index++) {
			this.addEntry(entries[index]);
		}
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 * @return void
 	 */	
	removeEntry: function(entry) {
		// Validate list		
		if (this._isObjectDefined(this.domNode) === false) {
			return;
		}
		
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}
		
		// Remove from list
		var node = this._domMap[id];
		this._contentArea.removeChild(node);
		// Remove from data map
		this._dataMap[id] = undefined;
		delete this._dataMap[id];
		// Remove from DOM map		
		this._domMap[id] = undefined;
		delete this._domMap[id];
		// Remove from selection map
		this._selected[id] = undefined;
		delete this._selected[id];
		// Update entry count
		this._entryCount--;

	},

	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return void
 	 */	
	removeEntries: function(entries) {
		for (var index = 0; index < entries.length; index++) {
			this.removeEntry(entries[index]);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	removeAllEntries: function() {
		for (var id in this._dataMap) {
			this.removeEntry(this._dataMap[id]);
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Number
 	 */	
	getEntryCount: function() {
		return this._entryCount;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param String id
 	 * @return Object
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 */	
	getEntry: function(id) {
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			return null;
		}
		return this._dataMap[id];
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getAllEntries: function() {
		var entryList = new Array();
		for (var id in this._dataMap) {
			entryList.push(this._dataMap[id]);
		}
		return entryList;
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */	
	getSelectedEntries: function() {
		var entryList = new Array();
		for (var id in this._selected) {
			if (this._selected[id]) {
				entryList.push(this._dataMap[id]);
			}
		}
		return entryList;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @param boolean select
 	 * @return void
 	 */	
	setSelectedEntries: function(entries, select) {
		for (var index = 0; index < entries.length; index++) {
			this.setSelected(entries[index], select);
		}
	},		

	/*
 	 * Public Method
 	 * 
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @param boolean select
 	 * @return void
 	 */	
	setSelected: function(entry, select) {
		var id = entry.id;
		// Check if this operation is applicable
		if (this._isObjectDefined(this._dataMap[id]) === false) {
			// NO-OP
			return;
		}
		// Check if a toggle is necessary
		if (this._selected[id] == select) {
			// NO-OP
			return;
		}
		
		// Store selection		
		this._selected[id] = select;
		// Update UI
		var node = this._domMap[id];
		if (select) {
			dojo.addClass(node, "entry-select");
		} else {
			dojo.removeClass(node, "entry-select");
		}
		
	},	

	/*
 	 * Public Method
	 *
 	 * @param Object entry
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return boolean
 	 */	
	isSelected: function(entry) {
		if (this._selected[entry.id]) {
			return true;
		}
		return false;
	},	

	/*
 	 * Public Method
	 *
 	 * @param void
 	 * @return void
 	 */	
	removeAllSelections: function() {
		for (var id in this._selected) {
			if (this._selected[id]) {
				// Get entry
				var entry = this._dataMap[id];
				// Unselect it
				this.setSelected(entry, false);
			}
		}
	},	

	/*
 	 * Public Method
 	 * 
 	 * Type is the key (Number)
 	 * Icon path is the value (String)
 	 * 
 	 * @param Array map : [Number]=String
 	 * @return void
 	 */	
	setIconMap: function(map) {
		this._iconMap = map;
	},	

	/*
 	 * Public Method
 	 * 
 	 * @param boolean show
 	 * @return void
 	 */	
	showLoading: function(show) {
		if (this._isObjectDefined(this.domNode) === false) {
			// Not initialized
			return;
		} else if (this._loadingFlag === show) {
			// Action already done
			return;
		} else if (show) {
			// Show message area
			this._showMessageArea();
		} else {
			// Show content area
			this._showContentArea();
		}
		
		// Update flag
		this._loadingFlag = show;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showMessageArea: function() {
		// Hide the content area
		this._updateVisibility(this._contentArea, false);
		// Show the message area		
		this._updateVisibility(this._messageArea, true);		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showContentArea: function() {
		// Hide the message area
		this._updateVisibility(this._messageArea, false);	
		// Show the content area
		this._updateVisibility(this._contentArea, true);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return void
 	 */	
	_updateVisibility: function(node, show) {
		if (show) {
			// Show
			if (node.style.display != "block") {
				node.style.display = "block";
			}
		} else {
			// Hide
			if (node.style.display != "none") {
				node.style.display = "none";
			}
		}
	},
		
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @return boolean
 	 */	
	_isObjectDefined: function(variable) {
	    if (typeof(variable) != 'object') {
			return false;
		} else if (variable === null) {
			return false;
		}
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return boolean
 	 */	
	_isStringDefined: function(text) {
	    if (typeof(text) != 'string') {
			return false;
		} else if (text.length === 0) {
			return false;
		}		
		return true;
	},			
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object variable
 	 * @param String property
 	 * @return boolean
 	 */	
	_hasProperty: function(variable, property) {
		if (property in variable) {
			return true;
		}
		return false;
	},		
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		// Disconnect handlers
		if (this._keyDownHandler !== null) {
			this.disconnect(this._keyDownHandler);
			this._keyDownHandler = null;
		}
		if (this._keyUpHandler !== null) {
			this.disconnect(this._keyUpHandler);
			this._keyUpHandler = null;
		}
		if (this._selectHandler !== null) {
			this.disconnect(this._selectHandler);
			this._selectHandler = null;
		}
		// Call super
		this.inherited(arguments);
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}		
	
});

})();
