/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");

/*
 * Imports 
 */
dojo.require("dojo.i18n");  
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectDialog");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TrendReportMessages");

/*
 * Aliases 
 */
var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var MultiSelectDialog = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MultiSelectDialog;

/*
 * TODOs 
 */
// Dialog needs loading indicator when waiting for parameters to populate

(function() {

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider",
	[dijit._Widget, dijit._Templated, CustomPreferenceProvider], {

	/*
	 * Properties
	 */	
	preferenceID: null,
	context: null,
	queryPreferenceMethodName: null,
	normalizeMultiValueMethodName: null,
	normalizeSingleValueMethodName: null,
	parseValueListMethodName: null,
	parseValueMapMethodName: null,
	iconMap: null,
	getIconTypeMethodName: null,
	isParametersLoadedMethodName: null,
	
	/*
	 * Templates
	 */	
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ReportsPreferenceProvider
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TrendReportMessages");
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited("postCreate", arguments);
		
		// Static Variables //------------------------------------------------->
		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		this._currentCallBack = null;
		this._currentValue = null;
		this._currentSettings = null;
		this._pollID = null;
		this._pollIterationCount = 0;
		this._dialog = null;

		// Initialization //--------------------------------------------------->

	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider";
		this._static._PREFERENCE_LABEL_SELECT = this.messages.msgSelect;
		
		this._static._POLL_INTERVAL = 250; // ms
		this._static._POLL_MAX_ITERATIONS = 480; // (4 * 250) = 1 sec * 120 sec = 2 min = 480  
		
		this._static._ID_PREFIX_UNSELECTED = "rpp-u";
		this._static._ID_PREFIX_SELECTED = "rpp-s";
		
		this._static._MULTI_PARAMETER_SEPARATOR = ",";

		this._static._PREFERENCE_OPTION_VALUE_NONE = "";
		this._static._PREFERENCE_OPTION_VALUE_ALL = "<<All>>";

		this._static._STYLE_LOADING = "color:#BBBBBB;";

	},

	/*
 	 * Public Method
 	 * 
 	 * @param String value
 	 * @return String
 	 * @overrides super
 	 */	
	getLabel: function(value) {
		// Check to see if the parameters are loaded
		if (this._isStringDefined(this.isParametersLoadedMethodName)) {
			var loadFlag = this.context[this.isParametersLoadedMethodName]();
			if (loadFlag === 0) {
				// Parameters have not been recieved yet
				// Return null to use the loading label and poll this method
				// again
				return null;
			} else if (loadFlag === -1) {
				// Parameters failed to load
				return this._buildLabel(this.messages.msgErrorLoadingParams, null);
			}
			// Parameters are loaded
		}

		// Check to see if a value was selected		
		if (this._isStringDefined(value) == false) {
			// No selected value 
			return this._buildLabel(this._static._PREFERENCE_LABEL_SELECT, null);
		}
		// Has selected value
		// Normalize
		value = this.context[this.normalizeMultiValueMethodName](value);
		
		return this._buildLabel(value, null);
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 * @overrides super
 	 */	
	getLoadingLabel: function() {
		return this._buildLabel(this.messages.msgLoading, this._static._STYLE_LOADING);
	},

	/*
 	 * Public Method
 	 * 
	 * @param Function onOk(String value, String prefId)
	 * @param Object settings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
	 * @param Object viewlet
	 * @return void
 	 * @overrides super
 	 */		
	createChooser: function(onOk, settings, viewlet) {
		// Store callback
		this._currentCallBack = onOk;
		// Store current value
		this._currentValue = settings.preferences[this.preferenceID];
		// Store current settings
		this._currentSettings = settings;
		// Create the dialog
		this._createDialog();
		// Get preferences
		var preferences = this.context[this.queryPreferenceMethodName](this._currentSettings);
		if (preferences == null) {
			// Poll parameters until they are recieved
			this._pollParameters();
		} else {
			// Populate dialog
			this._populateDialog(preferences);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	_createDialog: function() {
		// Create the dialog
		this._dialog = new MultiSelectDialog({
			sortField: "label"
		});
		this._dialog.setCallBackDialogExit(new Object({self: this}));
		this._dialog.setIconMap(this.iconMap);
		this._dialog.show();
		this._dialog.showLoading(true);				
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object preferences
 	 * 			. Array options : Object
 	 * 				. String label
 	 * 				. String value
 	 * @return void
 	 */		
	_populateDialog: function(preferences) {
		// Set unselected entries
		var unselectedEntries = this._adaptPreferencesToEntries(
			preferences,
			this._static._ID_PREFIX_UNSELECTED,
			this._currentValue);
		this._dialog.setUnselectedEntries(unselectedEntries);
		
		// Set selected entries
		var selectedEntries = this._adaptPreferenceValueToEntries(
			preferences,
			this._static._ID_PREFIX_SELECTED,
			this._currentValue);
		this._dialog.setSelectedEntries(selectedEntries);
		
		// Show entries
		this._dialog.showLoading(false);				
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_pollParameters: function() {
		// Stop previous poll (if any)
		this._stopPolling();
		// Start poll
		this._startPolling();
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	_startPolling: function() {
		var self = this;		
		// Start poll
		this._pollID = window.setInterval(
			function() {
				// Track number of polls
				self._pollIterationCount++;
				// Timeout if necessary
				if (self._pollIterationCount == self._static._POLL_MAX_ITERATIONS) {
					alert(self.messages.msgPollingTimedOut);
					// Stop the poll
					self._stopPolling();
					return;
				}
				// Get preferences
				var preferences = self.context[self.queryPreferenceMethodName](self._currentSettings);
				// Poll until preferences are recieved
				if (preferences == null) {
					return;
				}
				// Stop the poll
				self._stopPolling();
				// Populate dialog
				self._populateDialog(preferences);
 			}
			, this._static._POLL_INTERVAL);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	_stopPolling: function() {
		if (this._pollID == null) {
			return;
		}
		// Stop the poll
		window.clearInterval(this._pollID);
		this._pollID = null;
		this._pollIterationCount = 0;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object preferences
 	 * 			. Array options : Object
 	 * 				. String label
 	 * 				. String value
 	 * @param String idPrefix
 	 * @param String preferenceValue
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */		
	_adaptPreferencesToEntries: function(preferences, idPrefix, preferenceValue) {
	 	var options = preferences.options;
		var entries = new Array();
		if (preferenceValue == this._static._PREFERENCE_OPTION_VALUE_ALL) {
			// All values are selected
			return entries;
		} 
		
		// Get parameter name map
		var valueMap = this.context[this.parseValueMapMethodName](preferenceValue);
		
		for (var index = 0; index < options.length; index++) {
			var label = options[index].label;
			var value = options[index].value;
			
			// Omit selected preferences
			if (value in valueMap) {
				continue;
			}
			
			var iconType = this.context[this.getIconTypeMethodName](value, label);
			
			var entry = this._createEntry(
				value, 
				label, 
				idPrefix + "_" + index, 
				iconType);
			
			entries.push(entry);
		}
		return entries;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * @param String label
 	 * @param String id
 	 * @param Number type
 	 * @return Object
 	 * 				. String label
 	 * 				. String value
 	 * 				. String id
 	 * 				. Number type
 	 */	
	_createEntry: function (value, label, id, type) {
		var entry = new Object();
		entry.value = value;
		entry.label = label;
		entry.id = id;
		entry.type = type;
		
		return entry;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object preferences
 	 * 			. Array options : Object
 	 * 				. String label
 	 * 				. String value
 	 * @param String idPrefix
 	 * @param String preferenceValue
 	 * @return Array : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 */		
	_adaptPreferenceValueToEntries: function(preferences, idPrefix, preferenceValue) {
	 	var entries = new Array();
		
		if (preferenceValue == this._static._PREFERENCE_OPTION_VALUE_ALL) {
			// All values are selected
			var options = preferences.options;

			for (var index = 0; index < options.length; index++) {
				var label = options[index].label;
				var value = options[index].value;
				var iconType = this.context[this.getIconTypeMethodName](value, label);
				
				var entry = this._createEntry(
					value, 
					label, 
					idPrefix + "_" + index, 
					iconType);
				
				entries.push(entry);
			}
		} else {
			// Parse out parameter names
			parsedValues = this.context[this.parseValueListMethodName](preferenceValue);
			
			for (var index = 0; index < parsedValues.length; index++) {
				var value = parsedValues[index];
				var label = this.context[this.normalizeSingleValueMethodName](value);
				var iconType = this.context[this.getIconTypeMethodName](value, label);
				var entry = new Object();
				var entry = this._createEntry(
					value, 
					label, 
					idPrefix + "_" + index, 
					iconType);
				
				entries.push(entry);
			}
		}

		return entries;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object event
 	 * @return void
 	 */	
	handleEventMultiSelectDialogExit: function(event) {
		if (event.doIt == false) {
			return;
		}
		// Debug
		//console.debug(event.dialog.getSelectedEntries());
		//console.debug(event.dialog.getUnselectedEntries());
		var selectedEntries = event.dialog.getSelectedEntries();
		var unselectedEntries = event.dialog.getUnselectedEntries();
		
		var newPreferenceValue = null;
		if (unselectedEntries.length == 0) {
			// All entries were selected
			newPreferenceValue = this._static._PREFERENCE_OPTION_VALUE_ALL;
		} else {
			newPreferenceValue = this._adaptEntriesToPreferenceValue(selectedEntries);
		}
		
		// Execute call-back
		if (this._isValidPreferenceValueSize(newPreferenceValue)) {
			this._currentCallBack(newPreferenceValue, this.preferenceID);
		} else {
			// Fallback to using the old preference value
			this._currentCallBack(this._currentValue, this.preferenceID);
		}
		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String value
 	 * 
 	 * @return boolean
 	 */	
	_isValidPreferenceValueSize: function(value) {
		// REVISIT:  Post-1.0
		var fieldByteLimit = 250;
		var valueByteCount = this._countBytesInUTF8Text(value); 
		
		if (valueByteCount > fieldByteLimit) {
			alert(
				dojo.string.substitute(this.messages.msgNumParamsExceedLimit, [valueByteCount, fieldByteLimit])
			);
			return false;
		}
		
		return true;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * 
 	 * @return Number
 	 */	
	_countBytesInUTF8Text: function(text) {
		// Use encodeURI rather than encodeURIComponent so that the 
		// following characters are not encoded: ":", "/", ";", "?"
		var encodedText = encodeURI(text);
		// Debug
		// ~!@#$%^&*(){}[]=:/,;?+\'"\\  
		// = 25 bytes
		var byteCount = 0;
		
		if (encodedText.indexOf("%") != -1) {
			// Has encoded characters
			// Split the encoded text by its escape sequences
			var encodedElements = encodedText.split("%");
			// The number of encoded elements found equals the number of bytes
			// they occupy
			var encodedElementByteCount = encodedElements.length - 1;
			// e.g. %20
			var escapeSequenceLength = 3;		
			// Calculate the number of characters that make up the encoded
			// elements.  
			var encodedElementCharCount = encodedElementByteCount * escapeSequenceLength;		
			// The number non-encoded bytes equals the encoded text characters
			// minus the encoded element characters
			var nonEncodedElementByteCount = encodedText.length - encodedElementCharCount;
			// The number of bytes equals the number of encoded and non-encoded
			// bytes
			byteCount = encodedElementByteCount + nonEncodedElementByteCount;
		} else {
			// No encoded characters
			// The number of bytes equals the number of characters 
			byteCount = encodedText.length;
		}
		
		return byteCount;		
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Array entries : Object
 	 * 					. String label
 	 * 					. String value
 	 * 					. String id
 	 * 					. Number type
 	 * @return String
 	 */		
	_adaptEntriesToPreferenceValue: function(entries) {
		
		if (entries.length == 0) {
			return this._static._PREFERENCE_OPTION_VALUE_NONE;
		}
		
		var result = entries[0].value;
		for (var index = 1; index < entries.length; index++) {
			result = result + 
					 this._static._MULTI_PARAMETER_SEPARATOR +
					 entries[index].value;
		}
		return result;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @param String style
 	 * 
 	 * @return String
 	 */	
	_buildLabel: function (text, style) {
		var html = "<input type='text' readonly='readonly' ";
		if (this._isStringDefined(style)) {
			html =  html + 
					"style='" +
					style +
					"' "; 
		}
		return html +
				"value='" +
				text +
				"'/>";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return boolean
 	 */	
	_isStringDefined: function(text) {
	    if (typeof(text) != 'string') {
			return false;
		} else if (text.length == 0) {
			return false;
		}		
		return true;
	},			

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}		
	
});

})();
