/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooser");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dijit.Tree");
dojo.require("net.jazz.ajax.ui.internal.Dialog");

dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {

var DIALOG_DURATION = 100;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooser", null, {
	
	constructor: function(params){
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");
		if (!dojo.isFunction(params.onOk)) {
			throw new Error("Missing function for 'onOk' parameter");
		}
		if (params.scope != DashboardConstants.SCOPE_CONTRIBUTOR && params.scope != DashboardConstants.SCOPE_TEAM_AREA && params.scope != DashboardConstants.SCOPE_PROJECT_AREA) {
			throw new Error("Missing or invalid required parameter 'scope'");
		}
		if (!params.scopeItemId) {
			throw new Error("Missing required parameter 'scopeItemId'");
		}
		this._multiSelect = params.multiSelect == null || params.multiSelect; // null == true
		this._onOk = params.onOk;
		this._defaultValue = params.defaultValue ? params.defaultValue : null;
		
		// translate dashboard scope into work items scope
		switch (params.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				params.scope = QueryClient.SCOPE_PERSONAL;
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				params.scope = QueryClient.SCOPE_TEAM_AREA;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				params.scope = QueryClient.SCOPE_PROJECT_AREA;
				break;
		}
		var widgetParams= {};		
		widgetParams.primaryTitle = params.title ? params.title : messages.chooserTitle;

		var onOk = dojo.hitch(this, function(queries) {
			this._onOk(queries);
			onCancel();
		});
		var onCancel = dojo.hitch(this, function() {
			this.floatingPane.close();
		});
		
		this.floatingPane = /* createWidget */ new net.jazz.ajax.ui.internal.Dialog(widgetParams);
		this.contentWidget = new com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooserContent({onOk: onOk, onCancel: onCancel, multiSelect: this._multiSelect, scope: params.scope, scopeItemId: params.scopeItemId, defaultValue: this._defaultValue});
		this.floatingPane.contentNode.appendChild(this.contentWidget.domNode);
		this.contentWidget.init();
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooserContent",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/utils/WorkItemQueryChooser.html"),
	
	constructor: function(params){
		this.ok = messages.ok;
		this.cancel = messages.cancel;
		this._onOk = params.onOk;
		this._onCancel = params.onCancel;
		this._multiSelect = params.multiSelect;
		this._loadedCallback = params.loadedCallback;
		this._scope = params.scope;
		this._scopeItemId = params.scopeItemId;
		this._defaultValue = params.defaultValue;
	},
	
	postCreate: function() {
		this.connect(this._okCmd, "onclick", this._doOnOk);
		this.connect(this._cancelCmd, "onclick", this._onCancel);
		if (this._multiSelect)
			this._caption.innerHTML = messages.chooserInstructMulti;
		else
			this._caption.innerHTML = messages.chooserInstructSingle;
	},
	
	isMultiSelect: function() {
		return this._multiSelect;
	},
	
	init: function() {
		var self = this;
		var serviceObject = {
			_successCallback: function(/*array*/ scopedQueriesDTOs) {
				self._createTree(scopedQueriesDTOs);
				self._queryTreeArea.appendChild(self._tree.domNode);
			},
			_errorCallback: function(error) {
				dojo.debug(error.message);
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		QueryClient.getScopedQueries(responseHandler, this._scope, this._scopeItemId, false, true);
	},
	
	treeSelectionChanged: function(/* Array */ newSelection) {
		// since this is an associative array, newSelection.length reports 0
		// need to begin traversing its vars to see if there are any
		// set a flag on the first and break out of the loop
		for (var i in newSelection) {
			var hasSelection = true;
			break;
		}
		if (hasSelection) {
			this._okCmd.disabled = false;
		} else {
			this._okCmd.disabled = true;
		}
	},
	
	_doOnOk: function(event) {
		if (this.isMultiSelect()) {
			var result = new Array();
			var treeSelection = this._tree.getSelection();
			for (var i in treeSelection) {
				result.push(this._queriesMap[i]);
			}
			this._onOk(result);
		}
		else
			this._onOk(this._queriesMap[this._tree.getSelection()]);
	},
	
	_createTree: function(scopedQueriesDTOs) {
		var dataStore = this._createDataStore(scopedQueriesDTOs);
        var treeStore = new dojo.data.ItemFileReadStore({data: dataStore});
  		// Create the tree
		this._tree = new com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooserTree({
			store: treeStore,
			labelAttr: 'name',
			contentWidget: this,
			defaultValue: this._defaultValue
		});
		this._tree.init();
	},
	
	_createDataStore: function(scopedQueriesDTOs) {
		var dataStore = new Object();
		dataStore.label = 'name';
		dataStore.items = new Array();
		this._queriesMap = new Array();
		for (var i = scopedQueriesDTOs.length-1; i >= 0; i--) {
			var category = this._createCategory(scopedQueriesDTOs[i]);
			if (category.children.length > 0)
				dataStore.items.push(category);
		}
		dataStore.items.reverse();
		return dataStore;
	},
	
	_createCategory: function (scopedQueriesDTO) {
		var category = new Object();
		category.name = this._getCategoryName(scopedQueriesDTO);
		category.type = this._getCategoryType(scopedQueriesDTO);
		var children = new Array();
		for (var i in scopedQueriesDTO.queries) {
			if (!this._queriesMap[scopedQueriesDTO.queries[i].itemId]) {
				children.push({	name: scopedQueriesDTO.queries[i].name,	itemId: scopedQueriesDTO.queries[i].itemId });
				this._queriesMap[scopedQueriesDTO.queries[i].itemId] = scopedQueriesDTO.queries[i];
			}
		}
		category.children = children.sort(function(a, b) {
			return a.name < b.name ? -1 : a.name > b.name ? 1 : 0;
		});
		return category;
	},
	
	_getCategoryName: function (scopedQueriesDTO) {
		switch (scopedQueriesDTO.scope) {
			case QueryClient.SCOPE_PERSONAL:
				return messages.chooserHeadingPersonal;
			case QueryClient.SCOPE_OTHER_CONTRIBUTORS:
				return messages.chooserHeadingShared;
			case QueryClient.SCOPE_TEAM_AREA:
				return dojo.string.substitute(messages.chooserHeadingTeam, [ scopedQueriesDTO.scopeName ]);
			case QueryClient.SCOPE_PROJECT_AREA:
				return messages.chooserHeadingProject;
		}
	},
	
	_getCategoryType: function(scopedQueriesDTO) {
		if (scopedQueriesDTO.scope == QueryClient.SCOPE_TEAM_AREA)
			return "teamFolder";
		return "simpleFolder";
	},
	destroy: function() {
		if (this._tree)
			this._tree.destroy();
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryChooserTree",
	dijit.Tree, {
		
	constructor: function(args) {
		this._selected = new Array();
		this._contentWidget = args.contentWidget;
		this._defaultValue = args.defaultValue;
	},
	
	init: function() {
		this._expandAllTopLevelNodes();
		// if the tree is not multi select don't add ctrl listeners
		// so this._ctrlPressed will always be false and tree will be single select
		if (this._contentWidget.isMultiSelect()) {
			this.connect(dojo.doc, "onkeydown", this._handleKeyPressed);
			this.connect(dojo.doc, "onkeyup", this._handleKeyReleased);
		}
		if (this._defaultValue)
			this._setDefaultSelection();
	},
	
	getIconClass: function(/*dojo.data.Item*/ item){
		return item && item.type ? item.type : "query";
	},
		
	onClick: function (item, node) {
		if (!item.children) {
			var arg = {item: item, node: node};
			if (!this._ctrlPressed) {
				this._setSelection(arg);
			}
			else {
				if (!this._selected[item.itemId]) {
					this._addSelection(arg);
				}
				else {
					this._removeSelection(arg);
				}
			}
		} else {
			if (node.isExpanded)
				this._collapseNode(node);
			else
				this._expandNode(node);
		}
	},
	
	getSelection: function() {
		if (this._contentWidget.isMultiSelect())
			return this._selected;
		for (var i in this._selected)
			return i;
	},
	
	_setSelection: function(arg) {
		this._clearSelection(true);
		this._addSelection(arg);
	},
	
	_clearSelection: function(/* boolean */ blockEvent) {
		for (var i in this._selected)
			dojo.removeClass(this._selected[i].node.contentNode, "selectedNode");
		this._selected = new Array();
		if (!blockEvent && this._contentWidget)
			this._contentWidget.treeSelectionChanged(this._selected);
	},
	
	_addSelection: function(arg) {
		this._selected[arg.item.itemId] = arg;
		dojo.addClass(arg.node.contentNode, "selectedNode");
		if (this._contentWidget)
			this._contentWidget.treeSelectionChanged(this._selected);
	},
	
	_removeSelection: function(arg) {
		delete this._selected[arg.item.itemId];
		dojo.removeClass(arg.node.contentNode, "selectedNode");
		if (this._contentWidget)
			this._contentWidget.treeSelectionChanged(this._selected);
	},
	
	_expandAllTopLevelNodes: function() {
		// Get the tree's top level nodes
		var treeNodeList = this.getChildren();
		// Ensure the list is defined
		if ((treeNodeList == null) ||
			(treeNodeList.length == 0)) {
			// Nothing to do
			return;
		}
		// Expand each top level node
		for (var i = 0; i < treeNodeList.length; i++) {
			// Get the tree node
			var treeNode = treeNodeList[i];
			// Ensure the node is defined and is a folder
			if (treeNode == null) {
				continue;
			} else if (!treeNode.item.children) {
				continue;
			}
			// Expand the node
			this._expandNode(treeNode);
		}
	},
	
	_handleKeyPressed: function(event) {
		if (event.keyCode == dojo.keys.CTRL)
			this._ctrlPressed = true;
	},
	
	_handleKeyReleased: function(event) {
		if (event.keyCode == dojo.keys.CTRL)
			this._ctrlPressed = false;
	},
	
	_setDefaultSelection: function() {
		var node = this._searchNodesForDefault(this.getChildren());
		if (node) {
			this._setSelection({item: node.item, node: node});
		}
	},
	
	_searchNodesForDefault: function(nodes) {
		for (var i = 0; i < nodes.length; i++) {
			if (nodes[i].item && nodes[i].item.itemId == this._defaultValue)
				return nodes[i];
			var children = nodes[i].getChildren();
			if (children && children.length > 0) {
				var nodeInSubtree = this._searchNodesForDefault(children);
				if (nodeInSubtree)
					return nodeInSubtree;
			}
		}
		return null;
	}
});

})();
