/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("dijit.form.DateTextBox"); // dojo.widget.DropdownDatePicker

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
// Section Layout required here for debugging
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionLayout");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TwoColumnLayout");

dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {
	
var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var EditorUtil	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var LayoutUtil            = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ImagePool				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
var isGuest 				= com.ibm.team.repository.web.client.session.isGuest;


/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemOverview.html"),
		/*
		 * The sectionMap is used to map the sections to the overview layout
		 * 
		 */
		sectionMap: {
				details: {
						column: "left",
					 	renderFunctionName: "_renderDetails",
						exclude: {
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.AttachmentsPart" : true,
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart" : true,
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart" : true,
							"com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart" : true
						}
						},
				linksSummary: { // Quick Information
						highlightBackground: true,
						column: "right",
						className: "QuickInformationSection",
					 	renderFunctionName: "_renderGenericSection"
						},
				quickInfo: {  // Quick Information
						highlightBackground: true,
						column: "right",
						className: "QuickInformationSection",
					 	renderFunctionName: "_renderGenericSection"
						},
				description: {
						column: "left",
					 	renderFunctionName: "_renderDescription"
						},
				discussion: {
						column: "left",
					 	renderFunctionName: "_renderDiscussion"						
						}
			},
		/*
		 * The modifierMap is used to build the modifiers object passed to the widget
		 * It includes formatting and control information not defined in the 
		 * process spec.  Over time, it may make sense to move some of this information
		 * down to the process spec and then remove it from the modifier map.
		 * 
		 * Different modifiers are supported by different widgets.
		 * 
		 * Modifiers are first applied to specific attributes, but can also be
		 * specified for parts/widgets.
		 * 
		 * Currently functions are called in the parentWidget (e.g., this widget)
		 * 
		 * attributeLabel: the label used in the window before the field
		 *
		 * localModifierFunction: the name of a function to call before constructing the 
		 *     widget which allows the values, possible values and modifiers to be changed
		 * 
		 * spanColumns: True if the widget needs to span both columns
		 * 
		 * onCreate: the name of a function to call when the  widget is created
		 * onValueChanged: the name of a function to call when the widget value is changed
		 * 
		 * moreLabel, moreTitle, moreFunctionName: custom modifiers for the ContributorWidget
		 * noValueLabel: Label to show when a label or date has no value
		 * noValueTitle: Title/tooltip to show when a label or date has no value
		 */
//			this.editorUtil._bindValuesToSelect(valMap.internalSeverity, this.severity, this.severityIcon);
//			this.editorUtil._bindValuesToSelect(valMap.internalPriority, this.priority, this.priorityIcon);
//			this.editorUtil._bindValuesToSelect(valMap.target, this.intervalItemId);
		
		_modifierMap: {
				duration: {
					localModifierFunction: "_addAttributesToValues",
					_valuesToAdd: [ "correctedEstimate" ]
					},
				// psuedo Attribute for Duration with Correction.
				estimateAndCorrection: {
					localModifierFunction: "_addAttributesToValues",
					_valuesToAdd: [ "duration", "correctedEstimate" ]
					},
				workItemType: { // Type
//					onCreate: "_handleWorkItemTypeChange",
					onValueChanged: "_handleWorkItemTypeChange"
					},
				description: {
//					noValueLabel: "No Description."  See constructor
					},
				internalSeverity: {
//					attributeLabel: "Severity"
					},
				internalPriority: {
//					attributeLabel: "Priority"
					},
				creationDate: { // Created
//					noValueLabel: "Now",   // See Constructor
//					noValueTitle: "This work item has not yet been saved." // See constructor
					},
				creator: {  // Created By
					localModifierFunction: "_addWorkItemEmailSubjectToModifiers"
					 }, 
				owner: {
					localModifierFunction: "_addCurrentOwnerToPossibleValues",
					onCreate: "_handleOwnerChange",
					onValueChanged: "_handleOwnerChange",
//					moreLabel: "More...",				// See constructor
//					moreTitle: "Select an owner using the user selection dialog", // see constructor
					moreFunctionName: "_openOwnerSelectionDialog"
					 },
				target: {  // Planned For
/*					localModifierFunction: "_invertPossibleValuesOrder" */  
					},
				tags: {
//					rowWeight: 2  // counts as multiple rows
//					isSpanningColumns: true
					},
				category: {  // Filed Against
					localModifierFunction: "_modifyCategoryValues",
					onValueChanged: "_handleCategoryChange"
					},
// Duration is handled as a dynamically loaded widget
//				duration: {
//					attributeLabel: "Estimate",
//					localModifierFunction: "_modifyDurationValue"
//					},
				dueDate: {

					},
				version: {

					},
				foundIn: {
//					localModifierFunction: "_invertPossibleValuesOrder"
					},
				resolutionDate: {
//					noValueLabel: "Not Resolved"  // See constructor
					},
				resolver: {  // Resolved BY
//					noValueLabel: "Not Resolved"  // See Constructor
					},
				// Part attributes overrides apply to widgets
//				com_ibm_team_workitem_extension_customAttribute_risk: {
//					attributeLabel: "Risk"
//				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart: {
//					rowWeight: 2,  // counts as multiple rows
					skipNextSeparator: true,
//					attributeLabel: "Project Area",  // see constructor
					localModifierFunction: "_addTeamAreaToValues",
					addAttributesAfter: [ {
						attributeId: "com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart_TeamArea",
//						attributeLabel: "Team Area", // see constructor
						valuesFunction: "_getTeamAreaValues"} ]
				},
				com_ibm_team_workitem_web_ui_internal_view_editor_parts_SeparatorPart: {
					rowClass: "SeparatorTableRow",
					skipLabel: true,
					skipRequiredMarker: true
				}
			},
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.buttonSaveText = this.messages.buttonSave;
			this._modifierMap.description.noValueLabel = this.messages.labelNoDescription;
			this._modifierMap.creationDate.noValueLabel = this.messages.labelNow;
			this._modifierMap.creationDate.noValueTitle = this.messages.titleNotSavedYet;
			this._modifierMap.owner.moreLabel = this.messages.labelMore;
			this._modifierMap.owner.moreTitle = this.messages.titleMoreSelect;
			this._modifierMap.resolutionDate.noValueLabel = this.messages.labelNotResolved;
			this._modifierMap.resolver.noValueLabel = this.messages.labelNotResolved;
			this._modifierMap.com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart.attributeLabel = this.messages.labelProjectAreaAttribute;
			this._modifierMap.com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart.addAttributesAfter[0].attributeLabel = this.messages.labelTeamAreaAttribute;
		},	
		postCreate: function() {
			var _columnLayout= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.layout.TwoColumnLayout({});
			this.allWidgets = {};
			this._workItem = this.workItem;
			this._workItem.displayedAttributes = this._workItem.displayedAttributes ? this._workItem.displayedAttributes : {};
			
			if(this.pageProps && this.pageProps.sections) {
				for(var i=0;i<this.pageProps.sections.length;i++) {
					LayoutUtil.layoutSection(this, this.pageProps.sections[i], this.sectionMap, _columnLayout, this.parentController, this.allWidgets);
				}				
			}

			for(var widget in this.allWidgets) {
				if(this.allWidgets[widget].setFormElementValues) {
					this.allWidgets[widget].setFormElementValues(this.retainedState);					
				}
			}

			if(!isGuest() && this.retainedState.unsavedCommentText && this.retainedState.unsavedCommentText.length != 0) {
				this.setNewCommentText(this.retainedState.unsavedCommentText);
			}
			this._columnDIV.appendChild(_columnLayout.domNode);
		},
		destroyWidget: function()  {
			this.parentController.pageController.removeAllMouseListeners(this);
			for(var widget in this.allWidgets) {
				if(this.allWidgets[widget].destroyWidget) {
					this.allWidgets[widget].destroyWidget();					
				} else {
				if (this.allWidgets[widget].destroy) {
					this.allWidgets[widget].destroy();
				}
				}
			}
			this.destroy();
			
		},
	// Called when the tab is shown
		showWidget: function() {
			if(this.isShown) return;
			this.isShown = true; // only do this once
			for(var widget in this.allWidgets) {
				if(this.allWidgets[widget].showWidget) {
					this.allWidgets[widget].showWidget();					
				}
			}
		}, 
	_modifyCategoryValues: function(values, possibleValues, modifiers) {
		 return this.parentController.pageController._modifyCategoryValues(values, possibleValues, modifiers,this.parentController.isNewWorkItem() );
	},
		_handleOwnerChange: function(widget, value, isInitialization) {
			this.setLastSelectedOwner(value);
		},

		_handleCategoryChange: function(widget, value, isInitialization) {
			if(isInitialization) return;
			if (value.id != "showall") {
				e = { target: widget._select };
				this._updateOwners(e);
				
//				var teamAreaWidget = this.allWidgets["com_ibm_team_workitem_web_ui_internal_view_editor_parts_TeamAreaAttributePart_TeamArea"];
//				if(value && value.value && teamAreaWidget) {
//					var areaName = value.value.arguments;
//					if(areaName.indexOf("/") != -1) {
//						areaName = areaName.split("/")[0];
//					}
//					teamAreaWidget.setValue(areaName);	
//				}
				
			}
			return this.parentController.pageController._handleCategoryChange(widget, value, isInitialization);
			},
		_updateOwners: function(e) {
			this._handleDirtyingEvent(e);

			var self= this;
			var handlerObj= {
				success: function(values) {
					if(!self.allWidgets || ! self.allWidgets.owner) return;
					var ownerList = self._addCurrentOwnerToOwnerList(values);
					var selectedId = null;
					if(self.lastSelectedOwner) {
						ownerList = self._addExistingOwnerToOwnerList(values, self.lastSelectedOwner);
						selectedId = self.lastSelectedOwner.id;
					}
					self.allWidgets.owner.updateOptions(ownerList, selectedId);

			},
				error: function(errorObj) {
					self.parentController.handleError(errorObj, self.messages.errorFetchingOwners);
				}
			};
			var srh= new ServiceResponseHandler(handlerObj, "success", "error");
			var appArgs= {
				categoryItemId: this.allWidgets.category.getValue()
			};
			if(this.allWidgets.target && this.allWidgets.target.getValue()) {
				appArgs.intervalItemId = this.allWidgets.target.getValue();
			}
			
			WorkItemClient.getOwners(srh, appArgs);
		},
	_openOwnerSelectionDialog: function(contributorWidget) {
		this._selectOwner(contributorWidget);
	},
	_addTeamAreaToValues: function(values, possibleValues, modifiers) {
		values.push(this._workItem.attributes.projectArea.label);
	},
	_addAttributesToValues: function(values, possibleValues, modifiers) {
		if(!values || !modifiers || !modifiers._valuesToAdd || !this._workItem || !this._workItem.attributes) return;
		
		for(var i=0;i<modifiers._valuesToAdd.length;i++ ) {
			var attr = this._workItem.attributes[modifiers._valuesToAdd[i]];
			if(attr) {
				attr._attributeId = modifiers._valuesToAdd[i];
				values.push(attr);
			}
				
		}
		return { values: values };
	},
	_getTeamAreaValues: function(values, possibleValues, modifiers) {
		var value;
		try {
			value = this._workItem.attributes.teamArea.label;
		} 
		catch (e) {
		} // ignore
		return { values: [value] }
	},
	_invertPossibleValuesOrder: function(values, possibleValues, modifiers) {
		if(!possibleValues || possibleValues.length==0) return;
		try {
	        var newPossibleValues = [];
			newPossibleValues.push(possibleValues[0]);
			for(var i=possibleValues.length-1;i>0;i--) {
				newPossibleValues.push(possibleValues[i]);
			} 
			return { possibleValues: newPossibleValues };
		} catch(e) {
			console.debug("Error reordering possible values in WorkItemOverview");
		 }
	},	
	_addWorkItemEmailSubjectToModifiers: function(values, possibleValues, modifiers) {
		if(this._workItem && this._workItem.attributes.workItemType && this._workItem.attributes.workItemType.label && this._workItem.id && this._workItem.attributes.summary) {
			modifiers.emailSubject = this._workItem.attributes.workItemType.label + " " + this._workItem.id + ": " + EditorUtil.restoreTags(this._workItem.attributes.summary.content);			
		} else {
			modifiers.emailSubject = this.messages.msgDefaultEMailSubject;
		}

	},
	_addCurrentOwnerToPossibleValues:function(values, possibleValues, modifiers) {
		var currentOwner = this._workItem.attributes.owner
		var found = false;
		for(var i=0;i<possibleValues.length;i++) {
			if(possibleValues[i].id == currentOwner.itemId) {
				found = true;
				break;
			}
		}
		if(!found) {
			var item = currentOwner;
			if(!item.label && item.name) item.label = item.name;
			if(!item.id && item.itemId) item.id = item.itemId;
			item.isOutOfProjectArea = true;
			possibleValues.push(item);
		}
	},
	    _addWidgetRow: function(presentation, displaySpec, tableNode,  values, possibleValues, isRequired, sectionLayout ) {
			var creationSpec = LayoutUtil.getPresentationWidget(presentation);
			/*
			 * If the widget is missing, then only show it if we are in debug mode.
			 */
			if(creationSpec.isMissingWidget && !WorkItemUtil.isDebugMode()) return;

			var hideIfEmpty = false;
			if(presentation && presentation.properties) {
				for(var i=0;i<presentation.properties.length;i++) {
					if(presentation.properties[i].key == "hideIfEmpty") {
						hideIfEmpty = presentation.properties[i].value == "true";
						break;
					}
				}
			}
			
			var doHide = false;
			if(hideIfEmpty && (!values || values.length == 0 || (values[0].label!=null && this.parentController._isAttributeEqual(values[0].label, null))
			   || (values[0].userId && this.parentController._isAttributeEqual(values[0].userId, null)))){
				doHide = true;
			}
			
// template:
//			<tr>
//				<td class="AttributeTextLeft">Type: <span dojoAttachPoint="typeRequiredMarker" class="RequiredMarker"></span></td>
//				<td>
//					<select dojoAttachPoint="type" dojoAttachEvent="onchange: _handleSelectChangeWithImage" class="OverviewFormElement"></select>
//				</td>
//			</tr>
			
			var row = tableNode.insertRow(-1);
			if(doHide) {
				row.style.display = "none";
			}
			if(displaySpec["rowClass"]) {
				row.className=displaySpec["rowClass"];
			} else {
				row.className="LabelValueTableRow";
			}
			
			var labelColumn = row.insertCell(-1);
			labelColumn.className="Column LabelColumn";
						
			var labelSpan = document.createElement("SPAN");

			var modifiers = {};

			if(displaySpec["localModifierFunction"]) {
				var functionName = displaySpec["localModifierFunction"];
				if(this[functionName]){
					var result = this[functionName](values, possibleValues, displaySpec) ;
					if(result) {
						if(result.values) {
							values = result.values;
						}
						if(result.possibleValues) {
							possibleValues = result.possibleValues;
						}
						if(result.modifiers) {
							modifiers = result.modifiers;
						}
					}
				} else {
					console.debug("** ERROR ** Missing WorkItemOverview localModifierFunction named " + functionName);
				}
					
			}
	
			displaySpec.labelSpan = labelSpan;
			
			// Handle hard coded labels
			if(!presentation.attributeLabel && displaySpec.attributeLabel) {
				presentation.attributeLabel = displaySpec.attributeLabel;
			}
			
			if(!displaySpec.skipLabel) {
				if(presentation.attributeLabel) {
					if(presentation.attributeLabel.length > 0) {
						var text = document.createTextNode(presentation.attributeLabel + ": ");
						labelSpan.appendChild(text);
						} else {
						labelSpan.innerHTML = "";  /* innerHTML OK */
						}
				} else {
					displaySpec.hasDeferredLabel = true;
					labelSpan.innerHTML = "";  /* innerHTML OK */
				}
				labelColumn.appendChild(labelSpan);
			}
					
			if (!displaySpec.skipRequiredMarker) {
				var requiredMarkerSpan = document.createElement("SPAN");
				requiredMarkerSpan.className = "RequiredMarker";
				requiredMarkerSpan.innerHTML = "* " /* innerHTML OK */
				requiredMarkerSpan.title = this.messages.titleRequiredMarker;

				if(isRequired) {
					requiredMarkerSpan.style.display = "inline";
				} else {
					requiredMarkerSpan.style.display = "none";
				} 
				labelColumn.appendChild(requiredMarkerSpan);
			}
			
			modifiers.isRequired = isRequired;

			if (creationSpec.modifiers) {
				dojo.mixin(modifiers, creationSpec.modifiers);
			}
			
			if(modifiers.isSpanningColumns) {
				labelColumn.appendChild(document.createElement("BR"));
				labelColumn.className="Column LabelColumn2";
				labelColumn.colSpan = "2";
				valueColumn = labelColumn;
				
			} else {
				var valueColumn = row.insertCell(-1);
				valueColumn.className="Column ValueColumn2";
			}
						
			if(presentation.widget) {
				var valueDiv = document.createElement("DIV");
				valueColumn.appendChild(valueDiv);
				this._addWidget(creationSpec, presentation, valueDiv, values, possibleValues, modifiers, sectionLayout, hideIfEmpty);
			} else {
				var valueSPAN = document.createElement("SPAN");
				valueSPAN.innerHTML ="&nbsp;";  /* innerHTML OK */
				valueColumn.appendChild(valueSPAN);
			}
			},
	    _addWidget: function(creationSpec, presentation, valueNode,  values, possibleValues, modifiers, sectionLayout, hideIfEmpty) {
//			console.debug(presentation.widget +" "+ presentation.attributeId);

			if(!creationSpec || !creationSpec.widget) {
				creationSpec = LayoutUtil.mergeObjects(creationSpec, LayoutUtil.getPresentationWidget(presentation));
			}

			var widgetClass = creationSpec.widget;

			var propertyModifiers = LayoutUtil.getPropertyMap(presentation, this._modifierMap[presentation.attributeId]);
			LayoutUtil.mergeObjects(modifiers, propertyModifiers);
			
			creationSpec.hideIfEmpty = hideIfEmpty;
			
			// If the attribute is already displayed, substitute a separator with error message.
	    	var attributeId = propertyModifiers.attributeId != null ? propertyModifiers.attributeId : presentation.attributeId;
			creationSpec.isNotAvailable = false;
			if (attributeId) {
				if (this._workItem.displayedAttributes[attributeId]) {
					creationSpec.widget = com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget;
					creationSpec.isBlockedAttributePresentation = true;
					if(presentation.widget != "com.ibm.team.workitem.web.ui.internal.view.editor.parts.EnumerationAttributePart") {
						console.debug(dojo.string.substitute(this.messages.errorDuplicateAttributePresentation, [attributeId]));
						creationSpec.isNotAvailable = true;	
					}
				}
				else {
					this._workItem.displayedAttributes[attributeId] = true;
				}
			}
			
			var widgetClass = creationSpec.widget;

			var self = this;
			if(widgetClass) {
				// TODO: Remove this mapping
	    			var attributeId = propertyModifiers.attributeId != null ? propertyModifiers.attributeId : presentation.attributeId;
	    			attributeId = attributeId != null ? attributeId : presentation.widget.replace(/\./g,"_");
					var creationArgs = {
						parentWidget: this,
						creationSpec: creationSpec,
						workItem: this._workItem,
						parentController: this.parentController,
						attributeId: presentation.attributeId,
						attributeValue: this._workItem.attributes[attributeId],
						presentation: presentation,
						values: values,
 						valueNode: valueNode,
						sectionLayout: sectionLayout,
						_modifiers: modifiers,
						_possibleValues: possibleValues
					}

					var managerSelector = this.parentController.getManagerSelector(presentation.attributeId);
					if(managerSelector) {
						creationArgs[managerSelector] = true;
					}
					
					// If the widget is missing and we are not debugging, skip the widget and label
					if((creationSpec.isMissingWidget || creationSpec.isBlockedAttributePresentation) && !WorkItemUtil.isDebugMode()) return;

					if(creationSpec.isMissingWidget && !WorkItemUtil.isDebugMode()) return;
					
					if(creationSpec.isMissingWidget) {
						creationArgs.widgetName = creationSpec.widgetName;
						creationArgs.isMissingWidget = creationSpec.isMissingWidget;
					}
					
					try{
						var widget = new widgetClass(creationArgs, valueNode);
					} catch(e) {
						console.debug(e.msg);
						return;
					}

					
					if (presentation && presentation.displaySpec) {
					
						var displaySpec = presentation.displaySpec;
						
						if (widget.getLabel && displaySpec.labelSpan) {
							dojox.data.dom.removeChildren(displaySpec.labelSpan);
							var text = document.createTextNode(widget.getLabel());
							displaySpec.labelSpan.appendChild(text);
						}
						
						if (displaySpec.hasDeferredLabel  && displaySpec.labelSpan) {
							if (displaySpec.labelSpan.innerHTML == "" && presentation.attributeId) { /* innerHTML OK */
								var text = document.createTextNode(presentation.attributeId + ": ");
								displaySpec.labelSpan.appendChild(text);
							}
						}
					}					
					if(sectionLayout) {
						sectionLayout.addChildWidget(widget);	
					}
					
					var awKey = attributeId;
					if(displaySpec && displaySpec.attributeId) {
						awKey = displaySpec.attributeId;
					} 
					var i=0;
					var id = awKey; // Handle duplicate attributes
					while(this.allWidgets[id]) {
						id = awKey + i++;
					}
					this.allWidgets[id] = widget;
			}
			

		},
		/*
		 * Handle a change to the work item type
		 */
		_handleWorkItemTypeChange: function(widget, value, isInitialization) {
			if(!isInitialization) {
				e = { target: widget._select };
				this._handleDirtyingEvent(e);	
			}
			var imgSrc= value.iconUrl;
			this.parentController.updateType(value, value.label, imgSrc, isInitialization);
		},
		getDescriptionValue: function() {
			if(!this.allWidgets || !this.allWidgets.description ) return "";
			return this.allWidgets.description.getValue();
		},
		_renderDescription: function(sectionNode, section, sectionLayout) {
			if(!section || !section.presentations || section.presentations.length ==0) return;

			// Description section is a special case
			// if the attribute is description and the attribute label is the same as the section title
			// then hide the first label
			var hideFirstLabel = false;
			if (section.presentations && section.presentations.length > 0  &&  section.presentations[0].attributeId == "description" && section.presentations[0].attributeLabel ) {
				var sectionTitle = this.messages.labelDescription;
				if (section.properties) {
					for (var k = 0; k < section.properties.length; k++) {
						if (section.properties[k].key == "title") {
							sectionTitle = section.properties[k].value;
							break;
						}
					}
				}
				hideFirstLabel = section.presentations[0].attributeLabel==sectionTitle;
			}
			
			for(var i=0;i<section.presentations.length;i++) {
					var presentation = section.presentations[i];
					
					var attributeId = section.presentations[i].attributeId;
					var values = [];
					if(this.workItem && this.workItem.attributes && this.workItem.attributes[attributeId] ) {
						var attr = this.workItem.attributes[attributeId];
						if(dojo.isString(attr) || !dojo.isArray(attr)) {
							values = [ attr ];
						}
					}

					var hideIfEmpty = false;
					if(presentation && presentation.properties) {
						for(var k=0;k<presentation.properties.length;k++) {
							if(presentation.properties[k].key == "hideIfEmpty") {
								hideIfEmpty = presentation.properties[k].value == "true";
								break;
							}
						}
					}	
					if(hideIfEmpty && (!values || values.length == 0 || (values[0].label!=null && this.parentController._isAttributeEqual(values[0].label, null))
					   	|| (values[0].userId && this.parentController._isAttributeEqual(values[0].userId, null)))){
						continue;
					}
			
					if(i > 0 || (i==0 && !hideFirstLabel) ) {
						if(i>0) {
							var hr = document.createElement("HR");
							hr.className = "subDescriptionRule";
							sectionNode.appendChild(hr);							
						}
						var titleNode = document.createElement("DIV");
						titleNode.className = "subDescriptionTitle";
						
						if(section.presentations[i].attributeLabel) {
							var title = section.presentations[i].attributeLabel;
						} else {
							var title = dojo.string.substitute(this.messages.labelDescriptionPlus, [ "" + i]);
						}
						titleNode.appendChild(document.createTextNode(title));
						sectionNode.appendChild(titleNode);
					}
					var descriptionNode = document.createElement("DIV");
					sectionNode.appendChild(descriptionNode);

 
					if(attributeId == "description") {
						this.isDescriptionManager = true;
					}
					var hasSectionMenu = i==0 && hideFirstLabel && (section.presentations[0].widget.indexOf("HTMLAttributePart") != -1 || section.presentations[0].widget.indexOf("StringAttributePart") != -1) ;
					
					var possibleValues = {};

					// tags is special
					if (presentation) {
						if (presentation.attributeId == "tags" && this.editProps) {
							possibleValues = this.editProps.tags;
						}
						else {
							possibleValues = [];
							for (var k = 0; k < this.editProps.allValues.length; k++) {
								var value = this.editProps.allValues[k];
								if (value && value.attributeName && value.uiItems) {
									if (value.attributeName == presentation.attributeId) {
										possibleValues = value.uiItems;
										break;
									}
								}
							}
						}
					}

					this._addWidget({ setSectionMenu: hasSectionMenu, isDescription: true }, section.presentations[i], descriptionNode, values, possibleValues, {}, sectionLayout, hideIfEmpty);
			}
		},
		_renderDiscussion: function(sectionNode, section, sectionLayout) {
			var discussionNode = document.createElement("DIV");
			sectionNode.appendChild(discussionNode);
			
			var values = this.workItem.attributes.internalComments && this.workItem.attributes.internalComments.items ? this.workItem.attributes.internalComments.items : [];

			this._addWidget({iscommentManager: true}, section.presentations[0], discussionNode, values, {}, {}, sectionLayout);
		},
		/*
		 * Show a value-less section (e.g., quick information)
		 */
		_renderGenericSection: function(sectionNode, section, sectionLayout) {
			var contentNode = document.createElement("DIV");
			sectionNode.appendChild(contentNode);

			this._addWidget(null, section.presentations[0], contentNode, [], {}, {}, sectionLayout);		
	
		},

		_renderDetails:  function(sectionNode, section, sectionLayout) {
			if(!section || !section.presentations) return;
			var firstLeft = true;
			var firstRight =true;
			var detailsSplitTableWrapper = document.createElement("div");
			detailsSplitTableWrapper.className = "detailsSplitTableWrapper";
			
			var detailsSplitTable = document.createElement("TABLE");
			detailsSplitTable.className="DetailsSplitTable";
			var splitRow = detailsSplitTable.insertRow(-1);
			detailsSplitTableWrapper.appendChild(detailsSplitTable);
			
			var leftColumn = splitRow.insertCell(-1);
			var leftWidgetWrapper = document.createElement("div");
			leftWidgetWrapper.className = "DetailsInnerTableWrapper";
			var leftWidgetTable = document.createElement("TABLE");
			leftWidgetTable.className = "DetailsInnerTable";
			leftWidgetWrapper.appendChild(leftWidgetTable);
			leftColumn.appendChild(leftWidgetWrapper);
			leftColumn.className = "Column leftColumn";

			var rightColumn = splitRow.insertCell(-1);
			var rightWidgetWrapper = document.createElement("div");
			rightWidgetWrapper.className = "DetailsInnerTableWrapper";
			var rightWidgetTable = document.createElement("TABLE");
			rightWidgetTable.className = "DetailsInnerTable";
			rightWidgetWrapper.appendChild(rightWidgetTable);
			rightColumn.appendChild(rightWidgetWrapper);
			rightColumn.className = "Column rightColumn";
						
			var spanRow = detailsSplitTable.insertRow(-1);
			var spanColumn = spanRow.insertCell(-1);
			spanColumn.colSpan=2;
			spanColumn.className = "Column";
			var spanWidgetTable = document.createElement("TABLE");
			spanWidgetTable.className="SpanWidgetTable";
			spanColumn.appendChild(spanWidgetTable);
			
			var valueMap = LayoutUtil.getValueMap(this.editProps.allValues);
			
			// First figure out how many columns in the left and right
			var weightedLength = 0;
			var skippedColumnCount = 0;
	
			for (var i = 0; i < section.presentations.length; i++) {
				var presentation = section.presentations[i];
				
				if (!presentation.widget) {
					skippedColumnCount++;
					continue;
				}
				var displaySpec = {};
				if(presentation.attributeId) {
					presentation.isRequired = false;
					if(this.editProps.requiredProperties) {
						for(var j=0;j<this.editProps.requiredProperties.length;j++) {
							if(presentation.attributeId == this.editProps.requiredProperties[j]) {
								presentation.isRequired = true;
								continue;
							}
						}
					}
					
					var key = presentation.attributeId.replace(/\./g, "_");		
					if(this._modifierMap[key]) {
						displaySpec = this._modifierMap[key];			
					}
				}
				presentation.displaySpec = displaySpec;
				
				if(displaySpec.isInAlternateTable) {
					skippedColumnCount++;
				} else {
					if(displaySpec.rowWeight) {
						weightedLength = weightedLength + displaySpec.rowWeight; 
					} else {
						weightedLength++;
					}
				}
			}

			var rightColumnCount = Math.floor(weightedLength / 2) ;
			var leftColumnCount = weightedLength - rightColumnCount;
			
			var isIE6 = dojo.isIE && dojo.isIE < 7;
			
			weightedLength = 0;
			for(var i=0; i< section.presentations.length; i++) {
					var presentation = section.presentations[i];
					if(!presentation.widget) {
						skippedColumnCount++;
						continue; 
					}

					var displaySpec = presentation.displaySpec;

					var values = [];
					if(presentation.attributeId) {
						// priority and severity get mapped to internalSeverity and internalPrioirty
	    				var attributeId = displaySpec.attributeId != null ? displaySpec.attributeId : presentation.attributeId;
						values = this.workItem.attributes[attributeId] == null ? [] : this.workItem.attributes[attributeId];
						if(!dojo.isArray(values)) {
							values = [ values ];
						}
					}
					
					var possibleValues;

					// tags is special
					if(presentation.attributeId == "tags") {					
						possibleValues = this.editProps.tags;
					} else {
						possibleValues = [];
						for(var k=0;k<this.editProps.allValues.length;k++) {
							var value = this.editProps.allValues[k];
							if(value && value.attributeName && value.uiItems) {
								if(value.attributeName == presentation.attributeId) {
							  		possibleValues = value.uiItems;
									break;
								}
							}
						}
					}
					
					if(presentation.widget) {
						var key = presentation.widget.replace(/\./g, "_");
						if(this._modifierMap[key]) {
							displaySpec = LayoutUtil.mergeObjects(displaySpec, this._modifierMap[key]);
						}				
					}

// Fill the left column first then fill the right column.
// Some values can be skipped and some may fall into the span section (tags)
//					spanWidgetTable, leftWidgetTable, rightWidgetTable

					if(displaySpec.rowWeight) {
						weightedLength = weightedLength + displaySpec.rowWeight;
					} else {
						weightedLength++;
					}
					
					var skipWidget = false;
					var isSep = key == "com_ibm_team_workitem_web_ui_internal_view_editor_parts_SeparatorPart";
					if(isSep && this._skipNextSeparator) {
						this._skipNextSeparator = false;
						continue;
					}

					try {
						if (this.sectionMap[section.slot] && this.sectionMap[section.slot].exclude[presentation.widget]) {
							console.debug(dojo.string.substitute(this.messages.errorPartNotSupportedInSlot, [presentation.widget, section.slot]));
							skipWidget = true;
						}
					} 
					catch (e) {	} // ignore 

					if (displaySpec.skipNextSeparator) {
						this._skipNextSeparator = true;
					} else {
						this._skipNextSeparator = false;						
					}
					
					var widgetTableNode;
					if(displaySpec.isInAlternateTable) { 
						widgetTableNode = spanWidgetTable;
					} else {
						if( weightedLength <= leftColumnCount) {
							if (!(firstLeft && isSep)) {
								widgetTableNode = leftWidgetTable;
								firstLeft = false;
							} else {
								skipWidget = true;
							}
						} else  {
							if (!(firstRight && isSep)) {
								widgetTableNode = rightWidgetTable;
								firstRight = false;
							} else {
								skipWidget = true;
							}
						}
					}
					if(!skipWidget) {
						this._addWidgetRow(presentation, displaySpec, widgetTableNode, values, possibleValues, presentation.isRequired, sectionLayout );
						if(displaySpec.addAttributesAfter) {
							for(var j=0;j<displaySpec.addAttributesAfter.length;j++) {
								var presentation2 = dojo.mixin(presentation, {});
								presentation2.attributeLabel = displaySpec.addAttributesAfter[j].attributeLabel;
								var displaySpec2 = dojo.mixin(displaySpec, {});
								displaySpec2.attributeLabel = displaySpec.addAttributesAfter[j].attributeLabel;
								displaySpec2.attributeId = displaySpec.addAttributesAfter[j].attributeId;

								var result = {};
								if(displaySpec.addAttributesAfter[j].valuesFunction) {
									var result = this[displaySpec.addAttributesAfter[j].valuesFunction](values, possibleValues, displaySpec2) || {};
								}
								var values2 = result.values || values;
								var possibleValues2 = result.possibleValues || possibleValues;
								var displaySpec2 = result.modifiers || displaySpec2;
								presentation2.displaySpec = displaySpec2;
								this._addWidgetRow(presentation2, displaySpec2, widgetTableNode, values2, possibleValues2, presentation2.isRequired, sectionLayout );
							}
						}
						
					}
			}

			if(spanWidgetTable.rows && leftWidgetTable.rows) {
				var splitRow0=spanWidgetTable.rows[0];
				var leftRow0=leftWidgetTable.rows[0];
				if (splitRow0 && leftRow0) {
					if (splitRow0.childNodes[0].width < leftRow0.childNodes[0].width) {
						splitRow0.childNodes[0].width = leftRow0.childNodes[0].width;
					}
					else {
						leftRow0.childNodes[0].width = splitRow0.childNodes[0].width;
					}
				}				
			}
			sectionNode.appendChild(detailsSplitTableWrapper);
		},
		
		//
		// Editor part lifecycle methods
		//
		
		/**
		 * Unbind work item data from the work item overview editor part.
		 * Post M4: Reset is done by recreating the page rather than refreshing the values in place
		 * However, at some point (maybe) we should reimplement reset so that widget values are updated in place
		 */
//var SELECT_ELEMENTS= [
//	"type", "severity", "categoryItemId", "ownerItemId", "priority",
//	"intervalItemId", "version"
//];
//
//var PLAIN_TEXT_ELEMENTS= ["duration", "comment"];
//
//var READ_ONLY_ELEMENTS= [
//	"creationDate", "creator", "projectArea", "summaryLinks", "discussion", "commentCount", "newCommentDiv"
//];
//		reset: function() {
//			this._isLoading= true;
//			this._isDirty = false;
//			
//			if(!isGuest()) { 
//				this.createCommentCmd.className = "AddCommentLink";
//				this.createCommentCmdBottom.className = "AddCommentLinkBottom";
//			} else {
//				this.createCommentCmd.className = "AddCommentLinkDisabled";
//				this.createCommentCmdBottom.className = "AddCommentLinkBottomDisabled";
//			}
//	
//			// show unknown icons
//			var unknownIcon= ImageClient.getImageUrl(this.editorUtil._getUnknownIcon());
//			this.typeIcon.src= unknownIcon;
//			this.severityIcon.src= unknownIcon;
//			this.priorityIcon.src= unknownIcon;
//			
//			// clear select and text boxes
//			for (var i=0; i<SELECT_ELEMENTS.length; i++) {
//				this.editorUtil._resetSelect(this[SELECT_ELEMENTS[i]]);
//			}
//			for (i=0; i<PLAIN_TEXT_ELEMENTS.length; i++) {
//				if(this[PLAIN_TEXT_ELEMENTS[i]]) {
//					this.editorUtil._resetTextInput(this[PLAIN_TEXT_ELEMENTS[i]]);
//				}
//			}
//			for (i=0; i<READ_ONLY_ELEMENTS.length; i++) {
//				var node= this[READ_ONLY_ELEMENTS[i]];
//				try{
//					while (node && node.hasChildNodes && node.hasChildNodes()) {
//					// call to prevent IE6 memory leak in comment area
//						if(dojo.isIE) {
//							node.firstChild.outerHTML =''; // memory leak
//						}
//						node = dojox.data.dom.removeChildren(node);
//					}
//				} catch(e) {/* ignore */}
//
//			}
//			this.editorUtil._resetTextInput(this.description);
//			for (var id in this.PROPERTIES) {
//				this.editorUtil._clearRequired(this.PROPERTIES[id].requiredMarker);
//			}
//			
//			// to reset the date widget, we first have to re-initialize it to
//			// the current day so that this is the default when we open the
//			// popup calendar
//			this.dateWidget.setValue(new Date());
//			// then actually clear the field
//			this.dateWidget.setDisplayedValue(""); 
//
//		},
		findObjectWithProperty: function(/* property, path */ parms) {
			var object; 
			if (this[parms.property]) {
				object = this;
			} else {
				object = EditorUtil.findObjectWithPropertyInObject({
				object: this.allWidgets,
				path: parms.path,
				property: parms.property
					}, this); 
			}
			return object; 
		},
		disableBottomSaveCommand: function(isDisabled) {
			this._saveCmdBottom.disabled= isDisabled = isDisabled;
			this._saveCmdBottom.style.display= "block";
		},
		

		/* the owner values may not yet include the work item owner. If  not add it */
		_addCurrentOwnerToOwnerList: function(ownerList) {
			return this._addExistingOwnerToOwnerList(ownerList, this._workItem.attributes.owner);
		},
		_addExistingOwnerToOwnerList: function(ownerList, otherOwner) {
			if(!otherOwner) return ownerList;
			var found = false;
			var otherId = otherOwner.id || otherOwner.itemId;
			for(var i=0;i<ownerList.length;i++) {
				if(ownerList[i].id == otherId) {
					found = true;
					break;
				}
			}
			if(!found) {
				var item = otherOwner;
				item.isOutOfProjectArea = true;
				item.id = item.id || item.itemId;
				item.label = item.label || item.name;
				ownerList.push(item);
			}
			return ownerList;
		},
		setLastSelectedOwner: function(value) {
			this.lastSelectedOwner = value;
		},
		_selectOwner: function(widget) {

			var self= this;
				var params= {
					title: this.messages.titleSelectOwner,
					isMultiselect: false,
					useOkCancel: true, /* Not Add/Close */
					disableUsers: [{
						itemId: this._workItem.attributes.owner.id
					}]
				};
				this.ownerListWidget= new SelectContributorDialog(params);
				this.connect(this.ownerListWidget, "onClose", dojo.hitch(this, function() { this.parentController.focus();}));
				this.connect(this.ownerListWidget, "onOK", dojo.hitch(this, function(selectedOwnerList) { this._setSelectedOwner(selectedOwnerList, widget);}));
				this.ownerListWidget.searchText.focus();
			/* for now, reset the selection back, in case the user cancels */
			var select = widget._select;
			select.selectedIndex = select.currentDefaultIndex;
		},
		_setSelectedOwner: function(selectedOwnerList, widget) {
			if(!selectedOwnerList || selectedOwnerList.length ==0) {
				return;
			}
			
			var select = widget._select;
			var found = false;
			for(var i=0; i<select.options.length;i++ ) {
				if(select.options[i].valueObject && select.options[i].valueObject.id == selectedOwnerList[0].itemId) {
					found = true;
					break;
				}
			}
			var newPossibleValues = widget._possibleValues;
			if(!found) {
				selectedOwnerList[0].id = selectedOwnerList[0].itemId;
				selectedOwnerList[0].label = selectedOwnerList[0].name;
				selectedOwnerList[0].isOutOfProjectArea = true;
				newPossibleValues.push(selectedOwnerList[0]);		
			}
			this.setLastSelectedOwner(selectedOwnerList[0]);
			widget.updateOptions(newPossibleValues,selectedOwnerList[0].itemId );
		},

		// return an object containing form name-value pairs
		getFormElements: function() {		// return an object containing form name-value pairs
			var formElementValues= {};
			var self = this;
			for(var widget in this.allWidgets) {
					if(this.allWidgets[widget].getFormElementValues) {
						this.allWidgets[widget].getFormElementValues(formElementValues);
					}
				}
										
			return formElementValues;
		},
		handleMouseMove: function (e, isOverThisOrChild) {		
			// Handle mouse over
			if(isOverThisOrChild  && !this.isMouseOver) {
				this.isMouseOver = true;
			}
		
			// Handle mouse out
			if(!isOverThisOrChild && this.isMouseOver) {
				this.isMouseOver = false;
			}
		},

		_getMailto: function(address) {
			if(!address || address.length == 0) return null;
			return this.parentController._getMailto(address);
		}, 
		
		getNewCommentArea: function () {
			return this.allWidgets["com_ibm_team_workitem_web_ui_internal_view_editor_parts_DiscussionPart"];
		},
		// Return the new comment text, null if it is blank
		getNewCommentText: function() {
			var newCommentArea = this.getNewCommentArea(); // DiscussionWidget
			if(!newCommentArea) return "";
			var result = newCommentArea.getNewCommentTextValue();
			if(result == null) result = "";
			return result;
		},
		// Set the new comment text
		setNewCommentText: function(text) {
			var newCommentArea = this.getNewCommentArea();
			if(!newCommentArea) return null;
			newCommentArea.setNewCommentTextValue(text);
			this.parentController.markDirty();
		},
		// Set the new comment text
		clearNewCommentText: function() {
			var newCommentArea = this.getNewCommentArea();
			if(!newCommentArea) return null;
			newCommentArea.clearNewCommentTextValue();
		},
		hasRetainedState: function() {
			return this.getNewCommentText().length > 0;

		},
		clearRetainedState: function() {
			this.clearNewCommentText();
		},
		markFullyLoaded: function() {
			if(this.hasRetainedState()) {
				var newCommentArea = this.getNewCommentArea();
				if(!newCommentArea) return null;
				newCommentArea._showNewCommentArea(true); // give it focus
			}
		},
		//
		// event-related methods
		//
		
		_handleDirtyingEvent: function(e) {
			this.parentController.conditionalMarkDirty(e);
		},
		markDirty: function() {
			this._isDirty = true;
		},		
		_handleSaveClick: function(e) {
			this.parentController._handleSaveClick(e);
		},
		_handleSaveCommentClick: function(e) {
			e.preventDefault();
			if(!this.parentController._isDirty) return;
			this.parentController.save();
		},
		
		_handleNewCommentClick: function(e) {
			e.preventDefault();
			this._showNewCommentArea();
		},

		_showNewCommentArea: function() {
			if(this.createCommentCmd.className == "AddCommentLinkDisabled") {
				return;
			}
			this.newCommentArea.showNewComment();
			this.createCommentCmd.className = "AddCommentLinkDisabled";
			this.createCommentCmdBottom.className = "AddCommentLinkBottomDisabled";
		}
	}
);

})();
