/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");

dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that shows the Approvals Section
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;

var headingColor = "#666668";
var columnLayout = {
	name: {
		style: {
			position: "relative",
			left: "0px",
			color: headingColor,
			width: "100%"
		}
	}
}

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<div class="SubscribersPart SectionPart">' +
						'<div class="SubscribersContent" dojoAttachPoint="_contentNode">' +
						'</div></div>',
  		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			columnLayout.name.label = this.messages.labelName;
		},	
		postCreate: function() {	
		// this.parentController is the work item editor
		/*
		 * Subscribers is a special case where the subscriptions are managed by the 
		 * work item editor, since you can also add subscribers from the header
		 */
			this._childWidgets = [];
			this._isInitialized = false;
			this._workItem = this.parentController._workItem;
			
			this._subscriptionManager = this.parentController._subscriptionManager;
			this._subscriptionManager.addUpdateListener(this);
			
			if(!this.parentController.pageController.isGuest()) {
				this._addSubscriberAction = this.sectionLayout.addActionText(this.messages.buttonAddPrompt, this, "_handleNewItemPressed", this.messages.titleSubscribersAdd);
			}
		},
		destroyWidget: function() {
			this.parentController.pageController.removeAllMouseListeners(this);
			this.destroyChildWidgets();
			if(this._subscriptionManager) {
				this._subscriptionManager.removeUpdateListener(this);
			}
		},
		destroyChildWidgets: function() {
			for (var i = 0; i < this._childWidgets.length; i++) {
				if (this._childWidgets[i].destroyWidget) {
					this._childWidgets[i].destroyWidget();
					this._childWidgets[i] = null;
				}
				else {
					if (this._childWidgets[i].destroy) 
						this._childWidgets[i].destroy();
					this._childWidgets[i] = null;
				}
			}

			this._childWidgets = [];
			dojox.data.dom.removeChildren(this._contentNode);
		},
		/*
		 * Load the tab when it is shown
		 */
		showWidget: function() {
			if(this._isInitialized) return;
			this._isInitialized = true;
			this.refresh();
//			this._contentNode.innerHTML = "Loading...";
//			setTimeout(dojo.hitch(this, this.refresh), 100);
		},
		handleSubscriptionChange: function() {
			this.refresh();
		},
		/*
		 * Setup the page and read the states and types 
		 */
		refresh: function() {

			this.destroyChildWidgets();			
			this._markDirtyIfDifferent();
			var creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						isHeading: false,
						isFirst: false,
						tableClass: "SectionListTableNoBorder"
					};
			var listWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget(creationArgs);
			
			var rowItems = [];
			var subscribers = this._subscriptionManager.getSubscribers();
			
			subscribers.sort(function(a, b){
				try {
					return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1;
				} 
				catch (e) {
					return -1
				}
			});
			
			for (var i = 0; i < subscribers.length; i++) {
				if (subscribers[i]) {
					var item = {};
					item.columnWidgets = [this._getRow(listWidget, subscribers[i], /* noBorder */ true, /* isLast withborder */ false) ]
					item.noBorder = true ;
					item.value  = subscribers[i];
					item.showDelete = true;
					rowItems.push(item);
				}
			}

		
			listWidget.setRows(
					{
						itemList: rowItems,
						  deleteAction: {
								selector: "_handleDeleteSubscriber",
								title: this.messages.titleSubscriberRemove
						},
 						restoreAction: {
								selector: "_handleRestoreSubscriber",
								title: this.messages.titleSubscriberRestore
						}
					}
					)
			this._contentNode.appendChild(listWidget.domNode);
			this._childWidgets.push(listWidget);	

		},

		_markDirtyIfDifferent: function() {
			if(this._subscriptionManager.isChanged()) {
				this.parentWidget.parentController.markDirty();
			}
		},
		_handleDeleteSubscriber: function(e) {
			e.target.parentWidget.deleteSubscriber(e.target.value);
		},
		deleteSubscriber:function(subscriber) {
			this._subscriptionManager.deleteSubscribers([subscriber]);
			this.refresh();
		},
		_handleRestoreSubscriber: function(e) {
			e.target.parentWidget.restoreSubscriber(e.target.value);
		},
		restoreSubscriber:function(subscriber) {
			this._subscriptionManager.restoreSubscribers([subscriber]);
			this.refresh();
		},
		_getRow: function(listWidget, subscriber, noBorder , isLast) {
			var layout = WidgetUtil.mergeObjects({}, columnLayout.name);
			
			var split = subscriber.name.split(" ");
			var name = "";
			for(var i=0;i<split.length;i++) {
				var word = split[i];
				if(word.length> 16) {
					word = word.replace(/.{15}/g,"$& ");
				}
				name += " " + word;
			}
			
			name.replace(/  /g," ");
			layout.label = name;
			layout.isFirst = noBorder;
			layout.isLast = isLast;
			if(subscriber. emailAddress) {
				layout.href = this.parentController._getMailto(subscriber.emailAddress);
			}
			var widget =listWidget.getLabelWidget(layout);
			if(noBorder) widget.columnParms = { className: "SectionListTableHeaderNoBorder" } 
			return widget;
		},
	
		addSubscriber: function(contrib) {
			this._subscriptionManager.addSubscriber(contrib);
		},
	/*
		 * Add the array of selected contributors to the 'subscribers' multi-
		 * select box (in alphabetical ascending case-insensitive sort order).
		 * If we have at least one new subscriber and the caller specified true
		 * for markDirtyOnNew, mark the editor dirty.
		 */
		addSubscribers: function(newContribs) {
			this._subscriptionManager.addSubscribers(newContribs);
			this.refresh();
		},

		_handleNewItemPressed: function(action) {
			this._openSubscribersDialog();
		},

		
		_openSubscribersDialog: function() {
			var self= this;

				var params= {
					title: this.messages.titleSubscribersAdd,
					disableUsers: this.getCurrentSubscribers()
				};
				

			this.contributorListWidget= new SelectContributorDialog(params);
			this.connect(this.contributorListWidget, "onClose", dojo.hitch(this, function() { this.parentController.focus();}));
			this.connect(this.contributorListWidget, "onOK", dojo.hitch(this, function(selectedContribs) { this.addSubscribers(selectedContribs);}));
			this.contributorListWidget.searchText.focus();				
		},
		/**
		 * Returns an array of the itemIds of the current subscribers.
		 */
		getCurrentSubscribers: function() {
			return this._subscriptionManager.getCurrentSubscribers();
		},
		
		//
		// Part API Metnods
		//
		
		setReadOnly: function(isReadOnly) {
		},
		getFormElementValues: function(formElements) {
		},		
		setFormElementValues: function(retainedState){
			},
		/*
		 * Add values to the updateLinks commands
		 *
		 */
		 getSaveCommands: function (allCommands) {
// Subscription save commands are gotten from the subscriptionManager in workItemEditor
//			return this._subscriptionManager.getSaveCommands(allCommands);
		 },
	     getValue: function() {
//			return this.inputWidget.getValue();
		 }
	}
);

})();
