/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.DiscussionWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.DiscussionWidget",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/layout/templates/DiscussionWidget.html"),
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
			this.textCollapseAll = this.messages.buttonCollapseAll;
			this.textExpandAll = this.messages.buttonExpandAll;
			this.textAddComment = this.messages.buttonAddComment;
		},
		postCreate: function() {
			if(!this.attributeId) this.attributeId="comment";
			if(this.creationSpec && this.creationSpec.iscommentManager) this.iscommentManager = true; // TODO: defect 46775
			var comments = this.values;
			if (!comments) { comments= { length: 0 }; }

			this._updateCommentCount(comments.length);
			if(comments.length == 0) {
				this._expandCollapseDiv.style.visibility = "hidden";
				var noComments = document.createElement("DIV");
				noComments.className = "NoComments";
				noComments.appendChild(document.createTextNode(this.messages.labelNoComments));
				this.discussion.appendChild(noComments);				

			} else  {				
				this._expandCollapseDiv.style.visibility = "visible";

				for (var i=0; i<comments.length; i++) {
					comments[i].commentNumber = i + 1;
					var commentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment( {	
					    parentWidget: this.parentWidget,
					    parentController: this.parentController,
						comment: comments[i],
						editorUtil: this.parentWidget.editorUtil,
						workItem: this.parentWidget._workItem
					 } );
					comments[i].commentWidget = commentWidget;
					this.discussion.appendChild(commentWidget.domNode);
				}
			}			

			// add another comment for the "new comment" area
			commentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment( {
				parentWidget: this,	
				parentController: this.parentController,
				comment: null,
				editorUtil: this.parentWidget.editorUtil
				 } );
				 
			this.sectionLayout.setBorder(true);
			this.isCommentShowing = false;
			if (!this.parentController.pageController.isGuest()) {
				this._addCommentAction = this.sectionLayout.addActionText(this.messages.buttonAddComment, this, "_showNewCommentArea");
			}
			else {
				this.createCommentCmdBottom.style.visibility = "hidden";
			}
		    commentWidget._saveCmdBottom = this.parentWidget._saveCmdBottom; //CHANGE TO FUNCTION
			this.newCommentDiv.appendChild(commentWidget.domNode);
			this.newCommentArea = commentWidget;
			this.comment = commentWidget.commentText; // Bridge the form element
		},
		destroyWidget: function() {
			if(this.newCommentArea) {
				this.newCommentArea.destroyWidget();
			}
			this._doCommentCommand("destroyWidget");
			this._doCommentCommand("destroy");
			this.destroy();
		},		
		clearNewCommentTextValue: function() {
			this.setNewCommentTextValue("");
		},
		setNewCommentTextValue: function(value) {
			if(value && value.length > 0) {
				this._showNewCommentArea(true);				
			}
			this.newCommentArea.setTextValue(value);
		},
		getNewCommentTextValue: function () {
			var result =this.newCommentArea.getTextValue();
			if(result == "\n") return ""; // dijit.textarea bug
			return  result;
		},
		_updateCommentCount: function(count) {
					if(count > 1) {
						this.sectionLayout.setMessage(dojo.string.substitute(this.messages.labelSomeComments, [("" + count)]));						
					} else if(count == 1) 
						{
							this.sectionLayout.setMessage(this.messages.labelOneComment);
						} else 
						{
							this.sectionLayout.setMessage("")
						} 
		},
		getValue: function() {
			return this.getNewCommentTextValue();
		},
		getFormElementValues: function(formElements) {
			formElements[this.attributeId] = this.getNewCommentTextValue();
		}, 
		setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
				if(this.setNewCommentTextValue(retainedState[this.attributeId])) {
					this.setNewCommentTextValue(retainedState[this.attributeId]);
					this._isChanged = true;
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , this.getNewCommentTextValue());
					}
				}
			}
		 },
		_handleChange: function(e) {
			this.parentWidget._handleDirtyingEvent(e);
		},
		_handleNewCommentClick:function(e) {
			e.preventDefault();
			this._showNewCommentArea();
		},
		_showNewCommentArea: function(alwaysShow) {
			if(!this._addCommentAction) return;
			if(alwaysShow || !this.isShowingComment){
				this.isShowingComment = true;
				this.newCommentArea.showNewComment();
				dojox.data.dom.replaceChildren(this.createCommentCmdBottom, document.createTextNode(this.messages.buttonHideComment));
				dojox.data.dom.replaceChildren(this._addCommentAction._jazzAnchor, document.createTextNode(this.messages.buttonHideComment));
			} else {
				this.isShowingComment = false;
				this.newCommentArea.hideNewComment();
				var text;
				if(this.getNewCommentTextValue().length > 0) {
					text = this.messages.buttonShowComment;
				} else {
					text = this.messages.buttonAddComment;
				}
				
				dojox.data.dom.replaceChildren(this.createCommentCmdBottom, document.createTextNode(text));
				dojox.data.dom.replaceChildren(this._addCommentAction._jazzAnchor, document.createTextNode(text));
			}

		},
		_handleCollapseAll: function(e) {
			dojo.stopEvent(e);
			this._doExpandCollapse(false);
		},
		_handleExpandAll: function(e) {
			dojo.stopEvent(e);
			this._doExpandCollapse(true);
		},
		_doCommentCommand: function(commandName ){
			var comments= this.parentWidget._workItem.attributes.internalComments && this.parentWidget._workItem.attributes.internalComments.items ? this.parentWidget._workItem.attributes.internalComments.items : [];
			if (!comments || comments.length == 0) return;
			for (var i=0; i<comments.length; i++) {
				if(comments[i].commentWidget) {
					if(comments[i].commentWidget[commandName]) {
						comments[i].commentWidget[commandName]();
					}
				}
			}			
		},
		_doExpandCollapse: function(doExpand) {
				if (doExpand) {
					this._doCommentCommand("expand");
				} else {
					this._doCommentCommand("collapse");
				}
		}
	}
);

})();
