/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.navigation.RecentlyViewed"); //$NON-NLS-1$

dojo.require("dojo.cookie"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$

(function() {

var Session			= com.ibm.team.repository.web.client.session;
var ProjectArea		= com.ibm.team.process.web.client.projectarea;

var PlanningUtil	= com.ibm.team.apt.web.client.util;

var MAXRECENTITEMS	= 5;

/**
 * Widget to display predefined search links.
 */
dojo.declare("com.ibm.team.apt.web.ui.internal.view.navigation.RecentlyViewed",  //$NON-NLS-1$
			 [dijit._Widget], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	
	
	constructor: function() {
		this._recentItems= [];
		this._nodes= [];
	},
	
	// ---- API ------------------------------------------------------------------------------------------------------------

	addRecentPlan: function(planLabel, planItemId, planIcon) {
		return this._addPlan(planLabel, planItemId, planIcon);
	},
	
	// ---- widget lifecycle implementation --------------------------------------------------------------------------------

	postCreate: function() {
		this._recentItems= dojo.fromJson(dojo.cookie(this._getCookieName()) || "[]"); //$NON-NLS-1$
	},

	// ---- implementation -------------------------------------------------------------------------------------------------	

	_defaultPlanIcon: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/navigation/images/iteration_plan_default.gif"), //$NON-NLS-1$ //$NON-NLS-2$

	populate: function(navSection) {
		this.navSection = navSection;
		for (var i = this._recentItems.length - 1; i >= 0; i--) {
			var recentItem= this._recentItems[i];
			this._nodes[i] = this._doAddPlan(recentItem.planLabel, recentItem.planId, recentItem.planIcon);
			if (!this._recentItems[i].touched)
				this._recentItems[i].touched = new Date().getTime();
		}
	},
	
	_addPlan: function(planLabel, planId, planIcon) {
		for (var i= 0; i < this._recentItems.length; i++) {
			if(this._recentItems[i].planId == planId) {
				var node = this._nodes[i];
				this._recentItems[i].touched = new Date().getTime();
				this._recentItems[i].planLabel= planLabel;
				this._recentItems[i].planIcon= planIcon;
				break;
			}
		}
		
		if (!node) {
			this._recentItems.push({
				planLabel: planLabel,
				planId: planId,
				planIcon: planIcon,
				touched: new Date().getTime()
			});
			var node = this._doAddPlan(planLabel, planId, planIcon, this._nodes[this._nodes.length - 1]);
			this._nodes.push(node);
		}

		while (this._recentItems.length > MAXRECENTITEMS) {
			this._removeOldestEntry();
		}
		
		dojo.cookie(this._getCookieName(), dojo.toJson(this._removeImages(this._recentItems)), {expires: 356});
		
		return node;
	},
	
	_removeImages: function(recentItems) {
		var result= dojo.clone(recentItems);
		for (var i= 0; i < this._recentItems.length; i++) {
			this._recentItems[i].planIcon= null;
		}
		return result;
	},
	
	_removeOldestEntry: function() {
		var oldestTime = new Date().getTime();
		var oldestIndex;
		for (var i = 0; i < this._recentItems.length; i++) {
			if (this._recentItems[i].touched < oldestTime) {
				oldestTime = this._recentItems[i].touched;
				oldestIndex = i;
			}
		}
		this._removeEntry(oldestIndex);
	},
	
	_doAddPlan: function(planLabel, planId, planIcon, refNode) {
		var dummySpan= document.createElement("span"); //$NON-NLS-1$
		dummySpan.appendChild(document.createTextNode(planLabel));
		var escapedLabel= dummySpan.innerHTML;
		
		return this.navSection.addSectionRow(escapedLabel, PlanningUtil.getPlanViewURI(planId), planIcon || this._defaultPlanIcon, refNode);
	},
	
	_removeEntry: function(index) {
		this._recentItems.splice(index, 1);
		this._nodes[index].parentNode.removeChild(this._nodes[index]);
		this._nodes.splice(index, 1);
	},
	
	_getCookieName: function() {
		return "com.ibm.team.apt.web.ui.internal.view.navigation.RecentlyViewed" + "_" + ProjectArea.getCurrentProjectArea().itemId + "_" + Session.getAuthenticatedContributor().itemId; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	__sentinel: null // terminates this class definition
});
})();