/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.Search");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.query.SearchResultSelector");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function() { // module container

var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.query.Search",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/templates/Search.html"),
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		},
		DROP_DOWN_ICON_CLASS: "workItemsIcon",
		ACTIVE_SEARCH_ICON_CLASS: "search-button-active",
		// init params
		actionHandler: null,
		postCreate: function() {
			checkNotNull(this.actionHandler, "this.actionHandler");

			this.connect(this._searchText, "onkeyup", this.checkEmpty);
			this.connect(this._searchButton, "onclick", this._handleSearch);
			this.connect(this._searchForm, "onsubmit", this._handleSearch);
			this.connect(this._queryType,"onclick", this._openMenu);
			this.connect(this._searchText,"onkeypress",this._sniffKeySubmit);
			this.connect(this._queryType,"onkeypress",this._sniffKey);
			
			this._searchButton.title = this.messages.titleSearch;
			this._searchText.title = this.messages.titleSearch;
			
			this._prevSearchValue = "";
			var self = this;
			this._searchResultSelector= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.query.SearchResultSelector( {	
					_tagNode: this._searchText,
					parentWidget: this,
					_parentNode: this.domNode,
					_callback: function(str) {
						self._updateSearchText(str);
					}
				});

			this.openOnly = false;
			this._queryType.title =  this.messages.labelSearchAllWorkItems;
			this._searchText.title = this.messages.labelSearchAllWorkItems;
		},
		destroy: function() {
			try {
				if(this._searchResultSelector)	this._searchResultSelector.destroy();
				if(this._menu)	this._menu.destroy();
				this.inherited(arguments);
			} catch (e) {
				
			}
		},
		checkEmpty: function() {
			if (this._searchText.value.length < 1) {
				dojo.removeClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
			} else {
				dojo.addClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
			}
		},
		/*
		 * clears the search input box
		 */
		clear: function() {
			this._searchText.value= "";
		},

		/*
		 * sets focus in the search input box
		 */
		focus: function() {
			this._searchText.focus();
		},
		_updateSearchText: function(str) {
			if(str.id && str.id  > 0) {
				this._searchText.value = str.id;
			}
		},
		_handleSearch: function(e) {
			if (e) {
				e.preventDefault();	
			}
			var trimmedStr= dojo.trim(this._searchText.value);
			if (trimmedStr.length === 0) {
				return;
			} else {
				this.actionHandler(trimmedStr, this.openOnly);
			}
		},
		_sniffKey: function(e){
			if (e){
				dojo.stopEvent(e);
			}
			if (e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) this._openMenu();
		},
		_sniffKeySubmit: function(e){
			if (e.keyCode == dojo.keys.ENTER) this._handleSearch();
			this._checkSearchSelector(e);
			
		},
		_checkSearchSelector: function(e) {
			if(!e) return;
			
			// ctrl-space
			if(!this._searchResultSelector.isShowingNow && e.ctrlKey && e.charCode == 32) {
				this._searchResultSelector.showMatchingResults(this._searchText.value);
				return;
			}
			this._searchResultSelector.enableDisplay(e.keyCode != dojo.keys.ENTER);	

			if (!this._searchResultSelector.isShowingNow) { return; }
			
			switch(e.keyCode) {
			// tab
			case 9:
			// enter
			case 13:
				this._searchResultSelector.selectCurrentValue();
				e.stopPropagation();
				break;
			// down key
			case 40:
				this._searchResultSelector.highlightNextTag("down");
				break;
			// up key
			case 38:
				this._searchResultSelector.highlightNextTag("up");
				break;
			case 27: // esc
				this._searchResultSelector.close();
			}
			this._searchText.value = this._searchText.value.replace(/\n/g, " ");
			this._searchText.value = this._searchText.value.replace(/\r/g, " ");
		},
		_cleanUpSearchText: function(str) {
			return str;
		},
		_handleSearchInputKeystroke: function(e)  {
			
			var orig= this._prevSearchValue;

			var newVal= this._cleanUpSearchText(this._searchText.value);
			var orig = this._cleanUpSearchText(this._prevSearchValue);	

			var maxLen= Math.max(orig.length, newVal.length);

			for (var i=0; i<maxLen; i++) {
				if (orig.charAt(i) !== newVal.charAt(i)) {
					// if we reached this point, the text of the field has changed
					var searchTerm= dojo.trim(newVal);
					if (searchTerm) {
						// if we have a valid search term (i.e. a non-empty
						// string), we'll show the tag picker
						this._searchResultSelector.showMatchingResults(searchTerm);
					} else {
						// if we don't have a valid search term, hide the tag
						// picker
						this._searchResultSelector.close();
					}
					break;
				}
			}
			if (dojo.trim(this._searchText.value) === "") {
				this._searchResultSelector.close();
			}
			// now that we're done processing the key event, set the current
			// value of the tags value as the old value for next time
			this._prevSearchValue= this._searchText.value;
			
		},
		_handleSearchInputBlur: function(e) {
			
			if(!this._searchResultSelector.isMouseOver) {
				this._searchResultSelector.close();				
			}

			this._searchText.value = this._cleanUpSearchText(this._searchText.value);

		},
		_openMenu: function(e) {
			this._searchResultSelector.close();
			if (e){
				dojo.stopEvent(e);
			}
			if (this._menu) {
				this._closeMenu();
				return;
			}
			if (this.openOnly) {
				var allIcon = this.DROP_DOWN_ICON_CLASS;
				var allOpenIcon = this.DROP_DOWN_ICON_CLASS+"-selected";
				this._queryType.title =  this.messages.labelSearchAllWorkItems;
				this._searchText.title = this.messages.labelSearchAllWorkItems;
			} else {
				var allIcon = this.DROP_DOWN_ICON_CLASS+"-selected";
				var allOpenIcon = this.DROP_DOWN_ICON_CLASS;
				this._queryType.title =  this.messages.labelSearchAllOpenWorkItems;
				this._searchText.title = this.messages.labelSearchAllOpenWorkItems;
			}
			var menu = new dijit.Menu(); 		
			menu.addChild(new dijit.MenuItem({label: this.messages.labelAllWorkItems, iconClass: allIcon, onClick: dojo.hitch(this,this._searchAllWorkItems)}));
			menu.addChild(new dijit.MenuItem({label: this.messages.labelAllOpenWorkItems, iconClass: allOpenIcon, onClick: dojo.hitch(this,this._searchAllOpenWorkItems)}));
		
			var pos = dojo.coords(this._searchForm);
			pos.x += document.documentElement.scrollLeft;
			pos.y += document.documentElement.scrollTop;
			
			dojo.addClass(this._queryType,"QueryType-selected");
			dijit.popup.open({ popup: menu, parent: this, x: pos.x + 5, y: pos.y + pos.h - 2, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
			
			//register event handler that causes menu to close
			this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
			
			this._menu = menu;
			
			menu.focus();
		},
		_closeMenu: function(e){
			if (e) {
				dojo.stopEvent(e);
			}
			dojo.removeClass(this._queryType,"QueryType-selected");
			this.disconnect(this._menuCloseOnBody);
			dijit.popup.close(this._menu);
			this._menu = null;
			this._searchText.focus();
		},
		_searchAllWorkItems: function() {
			this.openOnly = false;
		},
		_searchAllOpenWorkItems: function() {
			this.openOnly = true;
		}
	}
);

})();