/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.TagWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector");

(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var TagSelector= com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.TagWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/TagWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.presentation= args.presentation;
		this.selection= args.selection;
	},
	
	postCreate: function() {

		if (this.selection)
			this._setValues(this.selection);
			
		this._previousTagsValue= this._tagInput.value;
			
		var attributeId= this.presentation.attribute.id;
		var srh= new ServiceResponseHandler(this, "_installTagSelector", "_handleError");
		WorkItemClient.getAllValues(srh, { newWorkItem: true, ids: [ attributeId ] });
	},
	destroy: function() {
			try {
				if(this._tagSelector) this._tagSelector.destroy();
				this.inherited(arguments);
			} catch (e) {
				
			}
	},
	/* String[] */ getValues: function() {
		var selection= new Array();
		var stringInput= this._tagInput.value;
		var parts= stringInput.split(",");
		for (var i= 0; i < parts.length; i++) {
			selection.push(dojo.trim(parts[i]));
		}
		return selection;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_setValues: function(/* UIItemDTO[] */ selection) {
		var list= new Array();
		for (var i= 0; i < selection.length; i++) {
			list.push(selection[i].id);
		}
		this._tagInput.value= list.join(", ");
	},
	
	_installTagSelector: function(/* UIItemSetDTO */ valueSet) {
		if (!valueSet || valueSet.length == 0)
			return;
		
		var self= this;
		this._tagSelector= new TagSelector( {	
			_tagNode: this._tagInput,
			parentWidget: this,
			_parentNode: this.domNode,
			_callback: function(str) {
				self._updateTagText(str);
			}
		});
		var tagValues= this._getTagValues(valueSet[0].uiItems);
		this._tagSelector.setAllTags(tagValues);
	},
	
	_getTagValues: function(/* UIItemDTO */ values) {
		if (!values)
			return [];
		var tags= new Array();
		for (var i= 0; i < values.length; i++) {
			tags.push(values[i].id);
		}
		return tags;
	},
	
	
	//////////////////////////////////////////////////////////////////
	// TODO: Copied code. Consolidate

	_updateTagText: function(str) {
		var allTagsArr= this._tagInput.value.split(",");
		allTagsArr[this._tagReplaceIndex]= this._tagReplaceIndex > 0 ? " " + str : str;
		this._tagInput.value= allTagsArr.join(",");
		this._tagInput.focus();
		this._previousTagsValue= this._tagInput.value;
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
 	_handleTagKeydown: function(e)  {
		
		if (!this._tagSelector.isShowingNow) { return; }
		
		switch(e.keyCode) {
		// tab
		case 9:
		// enter
		case 13:
			this._tagSelector.selectCurrentTag();
			break;
		// down key
		case 40:
			this._tagSelector.highlightNextTag("down");
			break;
		// up key
		case 38:
			this._tagSelector.highlightNextTag("up");
			break;
		}
		
	},
	
	_handleTagKeystroke: function(e)  {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
		
		var orig= this._previousTagsValue;

		var newVal= this._cleanUpTags(this._tagInput.value);
		var orig = this._cleanUpTags(this._previousTagsValue);	

		var maxLen= Math.max(orig.length, newVal.length);
//		console.debug("newval:>" + newVal + '<')
		var tagIndex= 0;
		for (var i=0; i<maxLen; i++) {
			if (orig.charAt(i) !== newVal.charAt(i)) {
				// if we reached this point, the text of the field has changed
				if (newVal.charAt(i) === "," && newVal.length > orig.length) {
					// if the user just typed a comma, close the popup
					this._tagSelector.close();
					break;
				}
				// tokenize the entire tag field by commas
				var allTagsArr= newVal.split(",");
				// we've been keeping track of how many commas we hit before
				// we discovered a change between the current value and the
				// previous value, so the search term is simply the token
				// at the current index
				var searchTerm= dojo.trim(allTagsArr[tagIndex], /*trimFromStart*/1);
//				console.debug("searchTerm:>" + searchTerm + '<')
				if (searchTerm) {
					// if we have a valid search term (i.e. a non-empty
					// string), we'll show the tag picker
					
					// the set of terms to exclude are all of current tags
					// other than the search term
					allTagsArr.splice(tagIndex, 1);
					// remember which of the tokens should be replaced if
					// the user clicks on one of the matching tags
					this._tagReplaceIndex= tagIndex;
					// display the matching 
					this._tagSelector.showMatchingTags(searchTerm, allTagsArr);
				} else {
					// if we don't have a valid search term, hide the tag
					// picker
					this._tagSelector.close();
				}
				break;
			}
			if (newVal.charAt(i) === ",") {
				// if we're here, the tags are still the same and we've hit
				// a comma; increment the tagIndex to denote that we're now
				// within a new term
				tagIndex++;
			}
		}
		if (dojo.trim(this._tagInput.value) === "") {
			this._tagSelector.close();
		}
		// now that we're done processing the key event, set the current
		// value of the tags value as the old value for next time
		this._previousTagsValue= this._tagInput.value;
	},
	
	_handleTagBlur: function(e) {
		if(!this._tagSelector.isMouseOver) {
			this._tagSelector.close();				
		}
	},
	
	_cleanUpTags: function(value) {
		var newVal= value.toLowerCase();
		// Replace spaces with commas, double commas and empty spaces
		// There's probably a better way to do do this...
		while(newVal.indexOf("  ") != -1) {
			newVal = newVal.replace(/  /g,' ');  // double space to spaces				
		}
		newVal = newVal.replace(/ /g,','); // spaces to commas
		while (newVal.indexOf(",,") != -1) {
			newVal = newVal.replace(/,,/g, ','); // double commas to commas
		}
		newVal = newVal.replace(/,/g,', '); // commas to space+comma
		while (newVal.indexOf(", , ") != -1) {
			newVal = newVal.replace(/, , /g, ', '); // remove extra space+commas
		}
		
		return newVal
	},
	
	_handleError: function(e) {
		if (this.errorCallback)
			this.errorCallback("Loading tags failed", e);
	}
	
});
	
})();
