/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.util.internal.Util");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

(function() {

/*
 * Aliases
 */
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

/*
 * General purpose utility methods.
 */
com.ibm.team.dashboard.web.util.internal.Util = {
	
	/*
	 * Returns an HREF that can be used as a link to the given dashboard. Tab is optional,
	 * and if specified, will take you to that specific tab.
	 */
	/* string */ getDashboardItemHref: function(/* DashboardItem */ dashboardItem, /* string */ tabId) {
		return this.getDashboardHref(dashboardItem.getScope(), dashboardItem.isPublic(), dashboardItem.getUserId(), dashboardItem.getId(), dashboardItem.getTeamPath(), tabId);
	},

	/*
	 * Returns an HREF that can be used as a link to the given dashboard info. Tab is
	 * optional, and if specified, will take you to that specific tab.
	 */
	/* string */ getDashboardHref: function(/* number */ scope, /* boolean */ isPublic, /* string */ userId, /* string */ dashboardItemId, /* string */ teamName, /* string */ tabId) {
		var href = "#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD;
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (isPublic) {
					href += "&" + DashboardConstants.URL_PARAM_USER + "=" + userId;
				}
				else {
					href += "&" + DashboardConstants.URL_PARAM_ID + "=" + dashboardItemId;
				}
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				href += "&" + DashboardConstants.URL_PARAM_TEAM + "=" + teamName;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				break;
		}
		if (tabId) {
			href += "&" + DashboardConstants.URL_PARAM_TAB + "=" + tabId;
		}
		return href;
	},
	
	/*
	 * Returns an args object that can be passed to DashboardPage#viewDashboard to view
	 * the given dashboard. Tab is optional, and if specified, will take you to that
	 * specific tab.
	 */
	/* object */ getDashboardItemParams: function(/* DashboardItem */ dashboardItem, /* string */ tabId) {
		return this.getDashboardParams(dashboardItem.getScope(), dashboardItem.isPublic(), dashboardItem.getUserId(), dashboardItem.getId(), dashboardItem.getTeamPath(), tabId);
	},

	/*
	 * Returns an args object that can be passed to DashboardPage#viewDashboard to view
	 * the given dashboard info. Tab is optional, and if specified, will take you to that
	 * specific tab.
	 */
	/* object */ getDashboardParams: function(/* number */ scope, /* boolean */ isPublic, /* string */ userId, /* string */ dashboardItemId, /* string */ teamName, /* string */ tabId) {
		var params = new Object();
		switch(scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				if (isPublic) {
					params[DashboardConstants.URL_PARAM_USER] = decodeURIComponent(userId);
				}
				else {
					params[DashboardConstants.URL_PARAM_ID] = decodeURIComponent(dashboardItemId);
				}
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				params[DashboardConstants.URL_PARAM_TEAM] = decodeURIComponent(teamName);
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				break;
		}
		if (tabId) {
			params[DashboardConstants.URL_PARAM_TAB] = decodeURIComponent(tabId);
		}
		return params;
	},

	/*
	 * Converts an associative array into an array of entries (reverse of toObject),
	 * i.e. { id1: value1, ... } -> [ {id: id1, value: value1}, ... ]
	 */
	/* object[] */ toEntryArray: function(/* object */ obj) {
		var entries = new Array();
		if (obj) {
			for (i in obj) {
				entries.push({id:i, value:obj[i]});
			}
		}
		return entries;
	},
	
	/*
	 * Converts an array of entries into an associative array (reverse of toEntryArray),
	 * i.e. [ {id: id1, value: value1}, ... ] -> { id1: value1, ... }
	 */
	/* object */ toObject: function(/* object[] */ array) {
		var obj = new Object();
		if (array) {
			for (var i=0;i<array.length;i++) {
				var entry = array[i];
				obj[entry.id] = entry.value;
			}
		}
		return obj;
	}
};

})();